@@ -1,20 +1,19 @@
 /*
- * Copyright (c) 2012-2014 Red Hat.
+ * Copyright (c) 2012-2017 Red Hat.
  * Copyright (c) 1997,2004 Silicon Graphics, Inc.  All Rights Reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2 of the License, or (at your
- * option) any later version.
- * 
- * This program is distributed in the hope that it will be useful, but
+ *
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- * for more details.
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
  */
-
-#ifndef _PMAPI_H
-#define _PMAPI_H
+#ifndef PCP_PMAPI_H
+#define PCP_PMAPI_H
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -24,6 +23,7 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/time.h>
+#include <time.h>
 
 /*
  * Platform and environment customization
@@ -59,7 +59,7 @@ typedef unsigned int	pmInDom;	/* Instance-Domain */
  *	{ 0, 1, -1, 0, PM_TIME_HOUR, 6 }
  * represents hours/million-events
  */
-typedef struct {
+typedef struct pmUnits {
 #ifdef HAVE_BITFIELDS_LTOR
     signed int		dimSpace : 4;	/* space dimension */
     signed int		dimTime : 4;	/* time dimension */
@@ -102,7 +102,7 @@ typedef struct {
 #define PM_COUNT_ONE	0	/* 1 */
 
 /* Performance Metric Descriptor */
-typedef struct {
+typedef struct pmDesc {
     pmID	pmid;		/* unique identifier */
     int		type;		/* base data type (see below) */
     pmInDom	indom;		/* instance domain */
@@ -186,6 +186,16 @@ typedef struct {
 /* retired PM_ERR_PMCDLICENSE (-PM_ERR_BASE-51) PMCD is not licensed to accept connections */
 #define PM_ERR_TYPE		(-PM_ERR_BASE-52)   /* Unknown or illegal metric type */
 #define PM_ERR_THREAD		(-PM_ERR_BASE-53)   /* Operation not supported for multi-threaded applications */
+#define PM_ERR_NOCONTAINER	(-PM_ERR_BASE-54)   /* Container not found */
+#define PM_ERR_BADSTORE		(-PM_ERR_BASE-55)   /* Bad input to pmstore */
+#define PM_ERR_LOGOVERLAP	(-PM_ERR_BASE-56)   /* Archives overlap in time */
+#define PM_ERR_LOGHOST		(-PM_ERR_BASE-57)   /* Archives differ by host */
+  /* retired PM_ERR_LOGTIMEZONE	(-PM_ERR_BASE-58) Archives differ in time zone */
+#define PM_ERR_LOGCHANGETYPE	(-PM_ERR_BASE-59)   /* The type of a metric has changed in an archive */
+#define PM_ERR_LOGCHANGESEM	(-PM_ERR_BASE-60)   /* The semantics of a metric has changed in an archive */
+#define PM_ERR_LOGCHANGEINDOM	(-PM_ERR_BASE-61)   /* The instance domain of a metric has changed in an archive */
+#define PM_ERR_LOGCHANGEUNITS	(-PM_ERR_BASE-62)   /* The units of a metric have changed in an archive */
+#define PM_ERR_NEEDCLIENTCERT	(-PM_ERR_BASE-63)   /* PMCD requires a client certificate */
 
 /* retired PM_ERR_CTXBUSY (-PM_ERR_BASE-97) Context is busy */
 #define PM_ERR_TOOSMALL		(-PM_ERR_BASE-98)   /* Insufficient elements in list */
@@ -199,22 +209,22 @@ typedef struct {
 /*
  * Report PMAPI errors messages
  */
-extern char *pmErrStr(int);				/* NOT thread-safe */
-extern char *pmErrStr_r(int, char *, int);
+PCP_CALL extern char *pmErrStr(int);			/* NOT thread-safe */
+PCP_CALL extern char *pmErrStr_r(int, char *, int);
 /* safe size for a pmErrStr_r buffer to accommodate all error messages */
 #define PM_MAXERRMSGLEN		128
 
 /*
  * Load a Performance Metrics Name Space
  */
-extern int pmLoadNameSpace(const char *);
-extern int pmLoadASCIINameSpace(const char *, int);
-extern void pmUnloadNameSpace(void);
+PCP_CALL extern int pmLoadNameSpace(const char *);
+PCP_CALL extern int pmLoadASCIINameSpace(const char *, int);
+PCP_CALL extern void pmUnloadNameSpace(void);
 
 /*
  * Where is PMNS located - added for distributed PMNS.
  */
-extern int pmGetPMNSLocation(void);
+PCP_CALL extern int pmGetPMNSLocation(void);
 #define PMNS_LOCAL   1
 #define PMNS_REMOTE  2
 #define PMNS_ARCHIVE 3
@@ -223,39 +233,39 @@ extern int pmGetPMNSLocation(void);
  * Trim a name space with respect to the current context
  * (usually from an archive, or after processing an archive)
  */
-extern int pmTrimNameSpace(void);
+PCP_CALL extern int pmTrimNameSpace(void);
 
 /*
  * Expand a list of names to a list of metrics ids
  */
-extern int pmLookupName(int, char **, pmID *);
+PCP_CALL extern int pmLookupName(int, char **, pmID *);
 
 /*
  * Find the names of descendent nodes in the PMNS
  * and in the latter case get the status of each child.
  */
-extern int pmGetChildren(const char *, char ***);
-extern int pmGetChildrenStatus(const char *, char ***, int **);
+PCP_CALL extern int pmGetChildren(const char *, char ***);
+PCP_CALL extern int pmGetChildrenStatus(const char *, char ***, int **);
 #define PMNS_LEAF_STATUS     0	/* leaf node in PMNS tree */
 #define PMNS_NONLEAF_STATUS  1	/* non-terminal node in PMNS tree */
 
 /*
  * Reverse Lookup: find name(s) given a metric id
  */
-extern int pmNameID(pmID, char **);		/* one */
-extern int pmNameAll(pmID, char ***);		/* all */
+PCP_CALL extern int pmNameID(pmID, char **);		/* one */
+PCP_CALL extern int pmNameAll(pmID, char ***);		/* all */
 
 /*
  * Handy recursive descent of the PMNS
  */
-extern int pmTraversePMNS(const char *, void(*)(const char *));
-extern int pmTraversePMNS_r(const char *, void(*)(const char *, void *), void *);
+PCP_CALL extern int pmTraversePMNS(const char *, void(*)(const char *));
+PCP_CALL extern int pmTraversePMNS_r(const char *, void(*)(const char *, void *), void *);
 
 /*
  * Given a metric, find it's descriptor (caller supplies buffer for desc),
  * from the current context.
  */
-extern int pmLookupDesc(pmID, pmDesc *);
+PCP_CALL extern int pmLookupDesc(pmID, pmDesc *);
 
 /*
  * Return the internal instance identifier, from the current context,
@@ -263,8 +273,8 @@ extern int pmLookupDesc(pmID, pmDesc *);
  * Archive variant scans the union of the indom entries in the archive
  * log.
  */
-extern int pmLookupInDom(pmInDom, const char *);
-extern int pmLookupInDomArchive(pmInDom, const char *);
+PCP_CALL extern int pmLookupInDom(pmInDom, const char *);
+PCP_CALL extern int pmLookupInDomArchive(pmInDom, const char *);
 
 /*
  * Return the external instance name, from the current context,
@@ -272,8 +282,8 @@ extern int pmLookupInDomArchive(pmInDom, const char *);
  * Archive variant scans the union of the indom entries in the archive
  * log.
  */
-extern int pmNameInDom(pmInDom, int, char **);
-extern int pmNameInDomArchive(pmInDom, int, char **);
+PCP_CALL extern int pmNameInDom(pmInDom, int, char **);
+PCP_CALL extern int pmNameInDomArchive(pmInDom, int, char **);
 
 /*
  * Return all of the internal instance identifiers (instlist) and external
@@ -282,58 +292,59 @@ extern int pmNameInDomArchive(pmInDom, int, char **);
  * Archive variant returns the union of the indom entries in the archive
  * log.
  */
-extern int pmGetInDom(pmInDom, int **, char ***);
-extern int pmGetInDomArchive(pmInDom, int **, char ***);
+PCP_CALL extern int pmGetInDom(pmInDom, int **, char ***);
+PCP_CALL extern int pmGetInDomArchive(pmInDom, int **, char ***);
 
 /*
  * Given context ID, return the host name associated with that context,
  * or the empty string if no name can be found
  */
-extern const char *pmGetContextHostName(int);
-extern char *pmGetContextHostName_r(int, char *, int);
+PCP_CALL extern const char *pmGetContextHostName(int);
+PCP_CALL extern char *pmGetContextHostName_r(int, char *, int);
 
 /*
  * Return the handle of the current context
  */
-extern int pmWhichContext(void);
+PCP_CALL extern int pmWhichContext(void);
 
 /*
  * Destroy a context and close its connection
  */
-extern int pmDestroyContext(int);
+PCP_CALL extern int pmDestroyContext(int);
 
 /*
  * Establish a new context (source of performance data + instance profile)
  * for the named host
  */
-extern int pmNewContext(int, const char *);
+PCP_CALL extern int pmNewContext(int, const char *);
 #define PM_CONTEXT_UNDEF	-1	/* current context is undefined */
 #define PM_CONTEXT_HOST		1	/* host via pmcd */
 #define PM_CONTEXT_ARCHIVE	2	/* PCP archive */
 #define PM_CONTEXT_LOCAL	3	/* local host, no pmcd connection */
 #define PM_CONTEXT_TYPEMASK	0xff	/* mask to separate types / flags */
-/* #define PM_CTXFLAG_SHALLOW	(1U<<8)	-- don't actually connect to host */
-/* #define PM_CTXFLAG_EXCLUSIVE	(1U<<9)	-- don't share socket among ctxts */
+#define PM_CTXFLAG_SHALLOW	(1U<<8)	/* DEPRECATED (don't actually connect to host) */
+#define PM_CTXFLAG_EXCLUSIVE	(1U<<9)	/* DEPRECATED (don't share socket among ctxts) */
 #define PM_CTXFLAG_SECURE	(1U<<10)/* encrypted socket comms channel */
 #define PM_CTXFLAG_COMPRESS	(1U<<11)/* compressed socket host channel */
 #define PM_CTXFLAG_RELAXED	(1U<<12)/* encrypted if possible else not */
 #define PM_CTXFLAG_AUTH		(1U<<13)/* make authenticated connection */
+#define PM_CTXFLAG_CONTAINER	(1U<<14)/* container connection attribute */
 
 /*
  * Duplicate current context -- returns handle to new one for pmUseContext()
  */
-extern int pmDupContext(void);
+PCP_CALL extern int pmDupContext(void);
 
 /*
  * Restore (previously established or duplicated) context
  */
-extern int pmUseContext(int);
+PCP_CALL extern int pmUseContext(int);
 
 /*
  * Reconnect an existing context (when pmcd dies, etc). All existing context
  * settings are preserved and the previous context settings are restored.
  */
-extern int pmReconnectContext(int);
+PCP_CALL extern int pmReconnectContext(int);
 
 /*
  * Add to instance profile.
@@ -342,7 +353,7 @@ extern int pmReconnectContext(int);
  * e.g. to select all available instances in all domains,
  * then use pmAddProfile(PM_INDOM_NULL, 0, NULL).
  */
-extern int pmAddProfile(pmInDom, int, int *);
+PCP_CALL extern int pmAddProfile(pmInDom, int, int *);
 
 /*
  * Delete from instance profile.
@@ -350,7 +361,7 @@ extern int pmAddProfile(pmInDom, int, int *);
  * E.g. to disable all instances in all domains then use
  * pmDelProfile(PM_INDOM_NULL, 0, NULL).
  */
-extern int pmDelProfile(pmInDom, int, int *);
+PCP_CALL extern int pmDelProfile(pmInDom, int, int *);
 
 /* 
  * ---------- Collection services ---------- 
@@ -381,7 +392,7 @@ extern int pmDelProfile(pmInDom, int, int *);
  *   2. len is the length of the len field + the real size of vbuf
  *	(which includes the null-byte terminator if there is one)
  */
-typedef struct {
+typedef struct pmValueBlock {
 #ifdef HAVE_BITFIELDS_LTOR
     unsigned int	vtype : 8;	/* value type */
     unsigned int	vlen : 24;	/* bytes for vtype/vlen + vbuf */
@@ -395,7 +406,7 @@ typedef struct {
 #define PM_VAL_HDR_SIZE	4		/* bytes for the vtype/vlen header */
 #define PM_VAL_VLEN_MAX	0x00ffffff	/* maximum vbuf[] size */
 
-typedef struct {
+typedef struct pmValue {
     int		inst;		/* instance identifier */
     union {
 	pmValueBlock	*pval;	/* pointer to value-block */
@@ -403,7 +414,7 @@ typedef struct {
     } value;
 } pmValue;
 
-typedef struct {
+typedef struct pmValueSet {
     pmID	pmid;		/* metric identifier */
     int		numval;		/* number of values */
     int		valfmt;		/* value style */
@@ -417,14 +428,14 @@ typedef struct {
 
 
 /* Result returned by pmFetch() */
-typedef struct {
+typedef struct pmResult {
     struct timeval	timestamp;	/* time stamped by collector */
     int			numpmid;	/* number of PMIDs */
     pmValueSet		*vset[1];	/* set of value sets, one per PMID */
 } pmResult;
 
 /* High resolution event timer result from pmUnpackHighResEventRecords() */
-typedef struct {
+typedef struct pmHighResResult {
     struct timespec	timestamp;	/* time stamped by collector */
     int			numpmid;	/* number of PMIDs */
     pmValueSet		*vset[1];	/* set of value sets, one per PMID */
@@ -451,7 +462,7 @@ typedef union {
  * and the number of value sets returned is guaranteed to be the same as
  * the number of metrics in the agument.  
  */
-extern int pmFetch(int, pmID *, pmResult **);
+PCP_CALL extern int pmFetch(int, pmID *, pmResult **);
 
 /*
  * PMCD state changes returned as pmFetch function result for PM_CONTEXT_HOST
@@ -465,18 +476,18 @@ extern int pmFetch(int, pmID *, pmResult **);
 /*
  * Variant that is used to return a pmResult from an archive
  */
-extern int pmFetchArchive(pmResult **);
+PCP_CALL extern int pmFetchArchive(pmResult **);
 
 /*
  * struct timeval is sometimes 2 x 64-bit ... we use a 2 x 32-bit format for
  * PDUs, internally within libpcp and for (external) archive logs
  */
-typedef struct {
+typedef struct __pmTimeval {
     __int32_t	tv_sec;		/* seconds since Jan. 1, 1970 */
     __int32_t	tv_usec;	/* and microseconds */
 } __pmTimeval;
 
-typedef struct {
+typedef struct __pmTimespec {
     __int64_t	tv_sec;		/* seconds since Jan. 1, 1970 */
     __int64_t	tv_nsec;	/* and nanoseconds */
 } __pmTimespec;
@@ -496,7 +507,7 @@ typedef struct {
 #define PM_LOG_VERS02	0x2
 #define PM_LOG_VOL_TI	-2	/* temporal index */
 #define PM_LOG_VOL_META	-1	/* meta data */
-typedef struct {
+typedef struct pmLogLabel {
     int		ll_magic;	/* PM_LOG_MAGIC | log format version no. */
     pid_t	ll_pid;				/* PID of logger */
     struct timeval	ll_start;		/* start of this log */
@@ -508,59 +519,61 @@ typedef struct {
  * Get the label record from the current archive context, and discover
  * when the archive ends
  */
-extern int pmGetArchiveLabel(pmLogLabel *);
-extern int pmGetArchiveEnd(struct timeval *);
+PCP_CALL extern int pmGetArchiveLabel(pmLogLabel *);
+PCP_CALL extern int pmGetArchiveEnd(struct timeval *);
 
 /* Free result buffer */
-extern void pmFreeResult(pmResult *);
-extern void pmFreeHighResResult(pmHighResResult *);
+PCP_CALL extern void pmFreeResult(pmResult *);
+PCP_CALL extern void pmFreeHighResResult(pmHighResResult *);
 
 /* Value extract from pmValue and type conversion */
-extern int pmExtractValue(int, const pmValue *, int, pmAtomValue *, int);
+PCP_CALL extern int pmExtractValue(int, const pmValue *, int, pmAtomValue *, int);
 
 /* Print single pmValue */
-extern void pmPrintValue(FILE *, int, int, const pmValue *, int);
+PCP_CALL extern void pmPrintValue(FILE *, int, int, const pmValue *, int);
 
 /* Scale conversion, based on value format, value type and scale */
-extern int pmConvScale(int, const pmAtomValue *, const pmUnits *, pmAtomValue *, 
+PCP_CALL extern int pmConvScale(int, const pmAtomValue *, const pmUnits *, pmAtomValue *, 
 		       const pmUnits *);
 
 /* Sort instances for each metric within a pmResult */
-extern void pmSortInstances(pmResult *);
+PCP_CALL extern void pmSortInstances(pmResult *);
 
 /* Adjust collection time and/or mode for pmFetch */
-extern int pmSetMode(int, const struct timeval *, int);
+PCP_CALL extern int pmSetMode(int, const struct timeval *, int);
 #define PM_MODE_LIVE	0
 #define PM_MODE_INTERP	1
 #define PM_MODE_FORW	2
 #define PM_MODE_BACK	3
 
 /* Modify the value of one or more metrics */
-extern int pmStore(const pmResult *);
+PCP_CALL extern int pmStore(const pmResult *);
 
 /* Get help and descriptive text */
-extern int pmLookupText(pmID, int, char **);
-extern int pmLookupInDomText(pmInDom, int, char **);
+PCP_CALL extern int pmLookupText(pmID, int, char **);
+PCP_CALL extern int pmLookupInDomText(pmInDom, int, char **);
 #define PM_TEXT_ONELINE	1
 #define PM_TEXT_HELP	2
 
 /*
  * Some handy formatting routines for messages, and other output
  */
-extern const char *pmIDStr(pmID);			/* NOT thread-safe */
-extern char *pmIDStr_r(pmID, char *, int);
-extern const char *pmInDomStr(pmInDom);			/* NOT thread-safe */
-extern char *pmInDomStr_r(pmInDom, char *, int);
-extern const char *pmTypeStr(int);			/* NOT thread-safe */
-extern char *pmTypeStr_r(int, char *, int);
-extern const char *pmUnitsStr(const pmUnits *);		/* NOT thread-safe */
-extern char *pmUnitsStr_r(const pmUnits *, char *, int);
-extern const char *pmAtomStr(const pmAtomValue *, int);	/* NOT thread-safe */
-extern char *pmAtomStr_r(const pmAtomValue *, int, char *, int);
-extern const char *pmNumberStr(double);			/* NOT thread-safe */
-extern char *pmNumberStr_r(double, char *, int);
-extern const char *pmEventFlagsStr(int);		/* NOT thread-safe */
-extern char *pmEventFlagsStr_r(int, char *, int);
+PCP_CALL extern const char *pmIDStr(pmID);			/* NOT thread-safe */
+PCP_CALL extern char *pmIDStr_r(pmID, char *, int);
+PCP_CALL extern const char *pmInDomStr(pmInDom);		/* NOT thread-safe */
+PCP_CALL extern char *pmInDomStr_r(pmInDom, char *, int);
+PCP_CALL extern const char *pmTypeStr(int);			/* NOT thread-safe */
+PCP_CALL extern char *pmTypeStr_r(int, char *, int);
+PCP_CALL extern const char *pmSemStr(int);			/* NOT thread-safe */
+PCP_CALL extern char *pmSemStr_r(int, char *, int);
+PCP_CALL extern const char *pmUnitsStr(const pmUnits *);	/* NOT thread-safe */
+PCP_CALL extern char *pmUnitsStr_r(const pmUnits *, char *, int);
+PCP_CALL extern const char *pmAtomStr(const pmAtomValue *, int);/* NOT thread-safe */
+PCP_CALL extern char *pmAtomStr_r(const pmAtomValue *, int, char *, int);
+PCP_CALL extern const char *pmNumberStr(double);		/* NOT thread-safe */
+PCP_CALL extern char *pmNumberStr_r(double, char *, int);
+PCP_CALL extern const char *pmEventFlagsStr(int);		/* NOT thread-safe */
+PCP_CALL extern char *pmEventFlagsStr_r(int, char *, int);
 
 /* Extended time base definitions and macros */
 #define PM_XTB_FLAG	0x1000000
@@ -569,22 +582,22 @@ extern char *pmEventFlagsStr_r(int, char *, int);
 #define PM_XTB_GET(x) (((x) & PM_XTB_FLAG) ? (((x) & 0xff0000) >> 16) : -1)
 
 /* Parse -t, -S, -T, -A and -O options */
-extern int pmParseInterval(const char *, struct timeval *, char **);
-extern int pmParseTimeWindow(
+PCP_CALL extern int pmParseInterval(const char *, struct timeval *, char **);
+PCP_CALL extern int pmParseTimeWindow(
       const char *, const char *, const char *, const char *,
       const struct timeval *, const struct timeval *,
       struct timeval *, struct timeval *, struct timeval *, char **);
 
 /* Reporting timezone */
-extern int pmUseZone(const int);
-extern int pmNewZone(const char *);
-extern int pmNewContextZone(void);
-extern int pmWhichZone(char **);
-extern char *pmCtime(const time_t *, char *);
-extern struct tm *pmLocaltime(const time_t *, struct tm *);
+PCP_CALL extern int pmUseZone(const int);
+PCP_CALL extern int pmNewZone(const char *);
+PCP_CALL extern int pmNewContextZone(void);
+PCP_CALL extern int pmWhichZone(char **);
+PCP_CALL extern char *pmCtime(const time_t *, char *);
+PCP_CALL extern struct tm *pmLocaltime(const time_t *, struct tm *);
 
 /* Parse host:metric[instances] or archive/metric[instances] */
-typedef struct {
+typedef struct pmMetricSpec {
     int		isarch;         /* source type: 0 -> live host, 1 -> archive, 2 -> local context */
     char	*source;        /* name of source host or archive */
     char	*metric;        /* name of metric */
@@ -593,9 +606,9 @@ typedef struct {
 } pmMetricSpec;
 
 /* Parsing of host:metric[instances] or archive/metric[instances] */
-extern int pmParseMetricSpec(const char *, int, char *, pmMetricSpec **,
+PCP_CALL extern int pmParseMetricSpec(const char *, int, char *, pmMetricSpec **,
 			     char **);
-extern void pmFreeMetricSpec(pmMetricSpec *p);
+PCP_CALL extern void pmFreeMetricSpec(pmMetricSpec *p);
 
 /*
  * Configurable error reporting
@@ -606,15 +619,20 @@ extern void pmFreeMetricSpec(pmMetricSpec *p);
 # define __PM_PRINTFLIKE(idx,cnt)
 #endif
 
-extern int pmprintf(const char *, ...) __PM_PRINTFLIKE(1,2);
-extern int pmflush(void);
+PCP_CALL extern int pmprintf(const char *, ...) __PM_PRINTFLIKE(1,2);
+PCP_CALL extern int pmflush(void);
 
 /*
  * Wrapper for config/environment variables. Warning: this will exit() with
  * a FATAL error if /etc/pcp.conf does not exist and $PCP_CONF is not set.
- * Return values point to strings in the environment.
+ * Use the pmGetOptionalConfig variant if this behaviour is not sought.
+ * Return values point to strings in the environment, or NULL in the optional
+ * case when the requested configuration variable was not found.
  */
-extern char *pmGetConfig(const char *);
+PCP_CALL extern char *pmGetConfig(const char *);
+PCP_CALL extern char *pmGetOptionalConfig(const char *);
+
+PCP_CALL extern int pmGetVersion(void);
 
 /*
  * Common command line argument parsing interfaces
@@ -625,45 +643,65 @@ extern char *pmGetConfig(const char *);
 #define PMAPI_OPTIONS_TEXT(s)	{ "", 0, '|', 0, (s) }
 #define PMAPI_OPTIONS_END	{ NULL, 0, 0, 0, NULL }
 
-#define PMOPT_ALIGN	{ "align",	1, 'A',	"TIME", \
+#define PMLONGOPT_ALIGN		"align"
+#define PMOPT_ALIGN	{ PMLONGOPT_ALIGN,	1, 'A',	"TIME", \
 			"align sample times on natural boundaries" }
-#define PMOPT_ARCHIVE	{ "archive",	1, 'a',	"FILE", \
+#define PMLONGOPT_ARCHIVE	"archive"
+#define PMOPT_ARCHIVE	{ PMLONGOPT_ARCHIVE,	1, 'a',	"FILE", \
 			"metrics source is a PCP log archive" }
-#define PMOPT_DEBUG	{ "debug",	1, 'D',	"DBG", \
+#define PMLONGOPT_DEBUG		"debug"
+#define PMOPT_DEBUG	{ PMLONGOPT_DEBUG,	1, 'D',	"DBG", \
 			NULL }
-#define PMOPT_GUIMODE	{ "guimode",	0, 'g',	0, \
+#define PMLONGOPT_GUIMODE	"guimode"
+#define PMOPT_GUIMODE	{ PMLONGOPT_GUIMODE,	0, 'g',	0, \
 			"start in GUI mode with new time control" }
-#define PMOPT_HOST	{ "host",	1, 'h', "HOST", \
+#define PMLONGOPT_HOST		"host"
+#define PMOPT_HOST	{ PMLONGOPT_HOST,	1, 'h', "HOST", \
 			"metrics source is PMCD on host" }
-#define PMOPT_HOSTSFILE	{ "hostsfile",	1, 'H', "FILE", \
+#define PMLONGOPT_HOSTSFILE	"hostsfile"
+#define PMOPT_HOSTSFILE	{ PMLONGOPT_HOSTSFILE,	1, 'H', "FILE", \
 			"read metric source hosts from a file" }
-#define PMOPT_SPECLOCAL	{ "spec-local",	1, 'K',	"SPEC", \
+#define PMLONGOPT_SPECLOCAL	"spec-local"
+#define PMOPT_SPECLOCAL	{ PMLONGOPT_SPECLOCAL,	1, 'K',	"SPEC", \
 			"optional additional PMDA spec for local connection" }
-#define PMOPT_LOCALPMDA	{ "local-PMDA",	0, 'L', 0, \
+#define PMLONGOPT_LOCALPMDA	"local-PMDA"
+#define PMOPT_LOCALPMDA	{ PMLONGOPT_LOCALPMDA,	0, 'L', 0, \
 			"metrics source is local connection to a PMDA" }
-#define PMOPT_NAMESPACE	{ "namespace",	1, 'n', "FILE", \
+#define PMLONGOPT_NAMESPACE	"namespace"
+#define PMOPT_NAMESPACE	{ PMLONGOPT_NAMESPACE,	1, 'n', "FILE", \
 			"use an alternative PMNS" }
-#define PMOPT_DUPNAMES	{ "dupnames",	1, 'N', "FILE", \
-			"use an alternative PMNS (duplicate names allowed)" }
-#define PMOPT_ORIGIN	{ "origin",	1, 'O', "TIME", \
+#define PMLONGOPT_UNIQNAMES	"uniqnames"
+#define PMOPT_UNIQNAMES	{ PMLONGOPT_UNIQNAMES,	1, 'N', "FILE", \
+			"like -n but only one name allowed for each PMID" }
+#define PMLONGOPT_ORIGIN	"origin"
+#define PMOPT_ORIGIN	{ PMLONGOPT_ORIGIN,	1, 'O', "TIME", \
 			"initial sample time within the time window" }
-#define PMOPT_GUIPORT	{ "guiport",	1, 'p', "N", \
+#define PMLONGOPT_GUIPORT	"guiport"
+#define PMOPT_GUIPORT	{ PMLONGOPT_GUIPORT,	1, 'p', "N", \
 			"port for connection to existing time control" }
-#define PMOPT_START	{ "start",	1, 'S', "TIME", \
+#define PMLONGOPT_START		"start"
+#define PMOPT_START	{ PMLONGOPT_START,	1, 'S', "TIME", \
 			"start of the time window" }
-#define PMOPT_SAMPLES	{ "samples",	1, 's', "N", \
+#define PMLONGOPT_SAMPLES	"samples"
+#define PMOPT_SAMPLES	{ PMLONGOPT_SAMPLES,	1, 's', "N", \
 			"terminate after this many samples" }
-#define PMOPT_FINISH	{ "finish",	1, 'T', "TIME", \
+#define PMLONGOPT_FINISH	"finish"
+#define PMOPT_FINISH	{ PMLONGOPT_FINISH,	1, 'T', "TIME", \
 			"end of the time window" }
-#define PMOPT_INTERVAL	{ "interval",	1, 't', "DELTA", \
+#define PMLONGOPT_INTERVAL	"interval"
+#define PMOPT_INTERVAL	{ PMLONGOPT_INTERVAL,	1, 't', "DELTA", \
 			"sampling interval" }
-#define PMOPT_VERSION	{ "version",	0, 'V', 0, \
+#define PMLONGOPT_VERSION	"version"
+#define PMOPT_VERSION	{ PMLONGOPT_VERSION,	0, 'V', 0, \
 			"display version number and exit" }
-#define PMOPT_TIMEZONE	{ "timezone",	1, 'Z', "TZ", \
+#define PMLONGOPT_TIMEZONE	"timezone"
+#define PMOPT_TIMEZONE	{ PMLONGOPT_TIMEZONE,	1, 'Z', "TZ", \
 			"set reporting timezone" }
-#define PMOPT_HOSTZONE	{ "hostzone",	0, 'z', 0, \
+#define PMLONGOPT_HOSTZONE	"hostzone"
+#define PMOPT_HOSTZONE	{ PMLONGOPT_HOSTZONE,	0, 'z', 0, \
 			"set reporting timezone to local time of metrics source" }
-#define PMOPT_HELP	{ "help",	0, '?', 0, \
+#define PMLONGOPT_HELP		"help"
+#define PMOPT_HELP	{ PMLONGOPT_HELP,	0, '?', 0, \
 			"show this usage message and exit" }
 
 #define PMAPI_GENERAL_OPTIONS	\
@@ -695,6 +733,12 @@ extern char *pmGetConfig(const char *);
 #define PMLONGOPT_HOST_LIST "host-list"
 #define PMOPT_HOST_LIST { PMLONGOPT_HOST_LIST, 1, 0, "HOSTS", \
 		"comma-separated list of metric source hosts" }
+#define PMLONGOPT_CONTAINER "container"
+#define PMOPT_CONTAINER { PMLONGOPT_CONTAINER, 1, 0, "NAME", \
+		"specify an individual container to be queried" }
+#define PMLONGOPT_DERIVED "derived"
+#define PMOPT_DERIVED { PMLONGOPT_DERIVED, 1, 0, "FILE", \
+		"load derived metric definitions from FILE(s)" }
 
 /* pmOptions flags */
 #define PM_OPTFLAG_INIT		(1<<0)	/* initialisation done */
@@ -712,10 +756,11 @@ extern char *pmGetConfig(const char *);
 #define PM_OPTFLAG_NOFLUSH	(1<<12)	/* caller issues pmflush */
 #define PM_OPTFLAG_QUIET	(1<<13)	/* silence getopt errors */
 
-struct __pmOptions;
-typedef int (*pmOptionOverride)(int, struct __pmOptions *);
+struct pmOptions;
+#define __pmOptions pmOptions /* backwards-compatible */
+typedef int (*pmOptionOverride)(int, struct pmOptions *);
 
-typedef struct {
+typedef struct pmLongOptions {
     const char *	long_opt;
     int			has_arg;
     int			short_opt;
@@ -723,7 +768,7 @@ typedef struct {
     const char *	message;
 } pmLongOptions;
 
-typedef struct __pmOptions {
+typedef struct pmOptions {
     int			version;
     int			flags;
 
@@ -779,23 +824,24 @@ typedef struct __pmOptions {
     unsigned int	zeroes  : 28;
 } pmOptions;
 
-extern int pmgetopt_r(int, char *const *, pmOptions *);
-extern int pmGetOptions(int, char *const *, pmOptions *);
-extern int pmGetContextOptions(int, pmOptions *);
-extern void pmUsageMessage(pmOptions *);
-extern void pmFreeOptions(pmOptions *);
+PCP_CALL extern int pmgetopt_r(int, char *const *, pmOptions *);
+PCP_CALL extern int pmGetOptions(int, char *const *, pmOptions *);
+PCP_CALL extern int pmGetContextOptions(int, pmOptions *);
+PCP_CALL extern void pmUsageMessage(pmOptions *);
+PCP_CALL extern void pmFreeOptions(pmOptions *);
 
 /*
  * Derived Metrics support
  */
-extern int pmLoadDerivedConfig(const char *);
-extern char *pmRegisterDerived(const char *, const char *);
-extern char *pmDerivedErrStr(void);
+PCP_CALL extern int pmLoadDerivedConfig(const char *);
+PCP_CALL extern int pmRegisterDerivedMetric(const char *, const char *, char **);
+PCP_CALL extern char *pmRegisterDerived(const char *, const char *);
+PCP_CALL extern char *pmDerivedErrStr(void);
 
 /*
  * Event Record support
  */
-typedef struct {
+typedef struct pmEventParameter {
     pmID		ep_pmid;
     /* vtype and vlen fields the format same as for pmValueBlock */
 #ifdef HAVE_BITFIELDS_LTOR
@@ -808,14 +854,14 @@ typedef struct {
     /* actual value (vbuf) goes here ... */
 } pmEventParameter;
 
-typedef struct {
+typedef struct pmEventRecord {
     __pmTimeval		er_timestamp;	/* must be 2 x 32-bit format */
     unsigned int	er_flags;	/* event record characteristics */
     int			er_nparams;	/* number of er_param[] entries */
     pmEventParameter	er_param[1];
 } pmEventRecord;
 
-typedef struct {
+typedef struct pmHighResEventRecord {
     __pmTimespec	er_timestamp;	/* must be 2 x 64-bit format */
     unsigned int	er_flags;	/* event record characteristics */
     int			er_nparams;	/* number of er_param[] entries */
@@ -830,7 +876,7 @@ typedef struct {
 #define PM_EVENT_FLAG_PARENT	(1U<<4)	/* 2nd parameter is parents ID */
 #define PM_EVENT_FLAG_MISSED	(1U<<31)/* nparams shows #missed events */
 
-typedef struct {
+typedef struct pmEventArray {
 		/* align initial declarations with start of pmValueBlock */
 #ifdef HAVE_BITFIELDS_LTOR
     unsigned int	ea_type : 8;	/* value type */
@@ -844,7 +890,7 @@ typedef struct {
     pmEventRecord	ea_record[1];
 } pmEventArray;
 
-typedef struct {
+typedef struct pmHighResEventArray {
 		/* align initial declarations with start of pmValueBlock */
 #ifdef HAVE_BITFIELDS_LTOR
     unsigned int	ea_type : 8;	/* value type */
@@ -859,26 +905,44 @@ typedef struct {
 } pmHighResEventArray;
 
 /* Unpack a PM_TYPE_EVENT value into a set on pmResults */
-extern int pmUnpackEventRecords(pmValueSet *, int, pmResult ***);
+PCP_CALL extern int pmUnpackEventRecords(pmValueSet *, int, pmResult ***);
 
 /* Free set of pmResults from pmUnpackEventRecords */
-extern void pmFreeEventResult(pmResult **);
+PCP_CALL extern void pmFreeEventResult(pmResult **);
 
 /* Unpack a PM_TYPE_HIGHRES_EVENT value into a set on pmHighResResults */
-extern int pmUnpackHighResEventRecords(pmValueSet *, int, pmHighResResult ***);
+PCP_CALL extern int pmUnpackHighResEventRecords(pmValueSet *, int, pmHighResResult ***);
 
 /* Free set of pmHighResResults from pmUnpackEventRecords */
-extern void pmFreeHighResEventResult(pmHighResResult **);
+PCP_CALL extern void pmFreeHighResEventResult(pmHighResResult **);
 
 /* Service discovery, for clients. */
 #define PM_SERVER_SERVICE_SPEC	"pmcd"
 #define PM_SERVER_PROXY_SPEC	"pmproxy"
 #define PM_SERVER_WEBD_SPEC	"pmwebd"
 
-extern int pmDiscoverServices(const char *, const char *, char ***);
+PCP_CALL extern int pmDiscoverServices(const char *, const char *, char ***);
+
+PCP_CALL extern int pmParseUnitsStr(const char *, pmUnits *, double *, char **);
+
+typedef struct __pmFetchGroup *pmFG;	/* opaque handle */
+PCP_CALL extern int pmCreateFetchGroup(pmFG *, int, const char *);
+PCP_CALL extern int pmGetFetchGroupContext(pmFG);
+PCP_CALL extern int pmExtendFetchGroup_item(pmFG, const char *, const char *,
+			const char *, pmAtomValue *, int, int *);
+PCP_CALL extern int pmExtendFetchGroup_indom(pmFG, const char *, const char *,
+			int[], char *[], pmAtomValue[], int, int[],
+			unsigned int, unsigned int *, int *);
+PCP_CALL extern int pmExtendFetchGroup_event(pmFG, const char *, const char *,
+			const char *, const char *,
+			struct timespec[], pmAtomValue[], int, int[],
+			unsigned int, unsigned int *, int *);
+PCP_CALL extern int pmExtendFetchGroup_timestamp(pmFG, struct timeval *);
+PCP_CALL extern int pmFetchGroup(pmFG);
+PCP_CALL extern int pmDestroyFetchGroup(pmFG);
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* _PMAPI_H */
+#endif /* PCP_PMAPI_H */
