@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2015 Red Hat.
+ * Copyright (c) 2012-2015,2017 Red Hat.
  * Copyright (c) 2000,2004,2005 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This library is free software; you can redistribute it and/or modify it
@@ -685,9 +685,10 @@ __pmConnectTo(int fd, const __pmSockAddr *addr, int port)
     if (__pmSetFileStatusFlags(fd, fdFlags | FNDELAY) < 0) {
 	char	errmsg[PM_MAXERRMSGLEN];
 
-        __pmNotifyErr(LOG_ERR, "%s:__pmConnectTo: cannot set FNDELAY - "
+	__pmNotifyErr(LOG_ERR, "%s:__pmConnectTo: cannot set FNDELAY - "
 		      "fcntl(%d,F_SETFL,0x%x) failed: %s\n",
-		      __FILE__, fd, fdFlags|FNDELAY , osstrerror_r(errmsg, sizeof(errmsg)));
+		      __FILE__, fd, fdFlags | FNDELAY,
+		      osstrerror_r(errmsg, sizeof(errmsg)));
     }
     
     if (__pmConnectWithFNDELAY(fd, &myAddr, sizeof(myAddr)) < 0) {
@@ -881,6 +882,7 @@ __pmAuxConnectPMCDPort(const char *hostname, int pmcd_port)
 {
     __pmSockAddr	*myAddr;
     __pmHostEnt		*servInfo;
+    const __pmAddrInfo	*ai;
     int			fd;
     int			sts;
     int			fdFlags[FD_SETSIZE];
@@ -922,6 +924,16 @@ __pmAuxConnectPMCDPort(const char *hostname, int pmcd_port)
     for (myAddr = __pmHostEntGetSockAddr(servInfo, &enumIx);
 	 myAddr != NULL;
 	 myAddr = __pmHostEntGetSockAddr(servInfo, &enumIx)) {
+	/*
+	 * We should be ignoring any entry without SOCK_STREAM (= 1),
+	 * since we're connecting with TCP only. The other entries
+	 * (SOCK_RAW, SOCK_DGRAM) are not relevant.
+	 */
+	ai = __pmHostEntGetAddrInfo(servInfo, enumIx);
+	if (ai->ai_socktype != SOCK_STREAM) {
+	    __pmSockAddrFree(myAddr);
+	    continue;
+	}
 	/* Create a socket */
 	if (__pmSockAddrIsInet(myAddr))
 	    fd = __pmCreateSocket();
@@ -942,11 +954,7 @@ __pmAuxConnectPMCDPort(const char *hostname, int pmcd_port)
 	fdFlags[fd] = __pmConnectTo(fd, myAddr, pmcd_port);
 	__pmSockAddrFree(myAddr);
 	if (fdFlags[fd] < 0) {
-	    /*
-	     * Mark failure in case we fall out the end of the loop
-	     * and try next address
-	     */
-	    __pmCloseSocket(fd);
+	    /* __pmConnectTo() has closed the fd, try next address. */
 	    continue;
 	}
 
@@ -1085,10 +1093,8 @@ __pmAuxConnectPMCDUnixSocket(const char *sock_path)
     /* Attempt to connect */
     fdFlags = __pmConnectTo(fd, myAddr, -1);
     __pmSockAddrFree(myAddr);
-    if (fdFlags < 0) {
-	__pmCloseSocket(fd);
+    if (fdFlags < 0)
 	return -ECONNREFUSED;
-    }
 
     /*
      * FNDELAY and we're in progress - wait on select
@@ -1200,6 +1206,23 @@ __pmHostEntGetSockAddr(const __pmHostEnt *he, void **ei)
     return addr;
 }
 
+const __pmAddrInfo *
+__pmHostEntGetAddrInfo(const __pmHostEnt *he, const void *ei)
+{
+    const __pmAddrInfo *ai;
+
+    /*
+     * If it is not NULL, the enumerator index, ei, is a pointer to the current
+     * address info.
+     */
+    if (ei == NULL)
+        ai = he->addresses;
+    else
+        ai = ei;
+
+    return ai;
+}
+
 char *
 __pmGetNameInfo(__pmSockAddr *address)
 {
@@ -1270,8 +1293,30 @@ __pmGetAddrInfo(const char *hostName)
     if (pmDebug & DBG_TRACE_DESPERATE) {
 	if (hostEntry == NULL)
 	    fprintf(stderr, "%s:__pmGetAddrInfo(%s) -> NULL\n", __FILE__, hostName);
-	else
-	    fprintf(stderr, "%s:__pmGetAddrInfo(%s) -> %s\n", __FILE__, hostName, hostEntry->name);
+	else {
+	    __pmSockAddr	*addr;
+	    void		*enumIx = NULL;
+	    char		*str;
+	    int			found = 0;
+	    fprintf(stderr, "%s:__pmGetAddrInfo(%s) -> ", __FILE__, hostName);
+	    for (addr = __pmHostEntGetSockAddr(hostEntry, &enumIx);
+		 addr != NULL;
+		 addr = __pmHostEntGetSockAddr(hostEntry, &enumIx)) {
+		str = __pmSockAddrToString(addr);
+		if (str != NULL) {
+		    if (found == 0)
+			fprintf(stderr, "%s", str);
+		    else
+			fprintf(stderr, ", %s", str);
+		    found++;
+		    free(str);
+		}
+	    }
+	    if (found)
+		fprintf(stderr, "\n");
+	    else
+		fprintf(stderr, "no ip addrs?\n");
+	}
     }
 #endif
     return hostEntry;
@@ -1510,8 +1555,13 @@ __pmRecv(int socket, void *buffer, size_t length, int flags)
     size = recv(socket, buffer, length, flags);
 #ifdef PCP_DEBUG
     if ((pmDebug & DBG_TRACE_PDU) && (pmDebug & DBG_TRACE_DESPERATE)) {
-	    fprintf(stderr, "%s:__pmRecv(%d, ..., %d, " PRINTF_P_PFX "%x) -> %d\n",
+	fprintf(stderr, "%s:__pmRecv(%d, ..., %d, " PRINTF_P_PFX "%x) -> %d",
 		__FILE__, socket, (int)length, flags, (int)size);
+	if (size < 0) {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, " %s", pmErrStr_r(-oserror(), errmsg, sizeof(errmsg)));
+	}
+	fputc('\n', stderr);
     }
 #endif
     return size;
