@@ -27,7 +27,6 @@ extern int __pmConvertTimeout(int) _PCP_HIDDEN;
 extern int __pmConnectWithFNDELAY(int, void *, __pmSockLen) _PCP_HIDDEN;
 
 extern int __pmPtrToHandle(__pmContext *) _PCP_HIDDEN;
-extern int __pmGetArchiveEnd_locked(__pmContext *, struct timeval *) _PCP_HIDDEN;
 
 extern int __pmGetDate(struct timespec *, char const *, struct timespec const *)  _PCP_HIDDEN;
 
@@ -92,6 +91,14 @@ extern void __htonll(char *) _PCP_HIDDEN;	/* 64bit int */
 #endif /* HAVE_NETWORK_BYTEORDER */
 
 #ifdef PM_MULTI_THREAD
+extern void __pmInitMutex(pthread_mutex_t *) _PCP_HIDDEN;	/* mutex initializer */
+
+/* local lock initilizer methods */
+extern void init_pmns_lock(void) _PCP_HIDDEN;
+extern void init_AF_lock(void) _PCP_HIDDEN;
+extern void init_secureserver_lock(void) _PCP_HIDDEN;
+extern void init_connect_lock(void) _PCP_HIDDEN;
+
 #ifdef HAVE___THREAD
 /*
  * C compiler is probably gcc and supports __thread declarations
@@ -118,16 +125,15 @@ __pmTPDGet(void)
 }
 
 #define PM_TPD(x)  __pmTPDGet()->x
-#endif
+#endif /* HAVE___THREAD */
 #else /* !PM_MULTI_THREAD */
 /* No threads - just access global variables as-is */
 #define PM_TPD(x) x
-#endif
+#endif /* PM_MULTI_THREAD */
 
 #if defined(PM_MULTI_THREAD) && defined(PM_MULTI_THREAD_DEBUG)
 extern void __pmDebugLock(int, void *, const char *, int) _PCP_HIDDEN;
 extern int __pmIsContextLock(void *) _PCP_HIDDEN;
-extern int __pmIsChannelLock(void *) _PCP_HIDDEN;
 extern int __pmIsDeriveLock(void *) _PCP_HIDDEN;
 extern int __pmIsAuxconnectLock(void *) _PCP_HIDDEN;
 extern int __pmIsConfigLock(void *) _PCP_HIDDEN;
@@ -141,6 +147,10 @@ extern int __pmIsOptfetchLock(void *) _PCP_HIDDEN;
 extern int __pmIsErrLock(void *) _PCP_HIDDEN;
 extern int __pmIsLockLock(void *) _PCP_HIDDEN;
 extern int __pmIsLogutilLock(void *) _PCP_HIDDEN;
+extern int __pmIsPmnsLock(void *) _PCP_HIDDEN;
+extern int __pmIsAFLock(void *) _PCP_HIDDEN;
+extern int __pmIsSecureserverLock(void *) _PCP_HIDDEN;
+extern int __pmIsConnectLock(void *) _PCP_HIDDEN;
 #endif
 
 /* AF_UNIX socket family internals */
@@ -186,6 +196,8 @@ struct __pmHostEnt {
     char		*name;
     __pmAddrInfo	*addresses;
 };
+
+extern const __pmAddrInfo *__pmHostEntGetAddrInfo(const __pmHostEnt *, const void *) _PCP_HIDDEN;
 #endif
 
 extern unsigned __pmFirstInetSubnetAddr(unsigned, int) _PCP_HIDDEN;
@@ -283,7 +295,32 @@ extern int __pmProxyAddPorts(int **, int) _PCP_HIDDEN;
 extern int __pmWebdAddPorts(int **, int) _PCP_HIDDEN;
 extern int __pmAddPorts(const char *, int **, int) _PCP_HIDDEN;
 
-extern void __pmCloseChannelbyContext(__pmContext *, int, int) _PCP_HIDDEN;
-extern void __pmCloseChannelbyFd(int, int, int) _PCP_HIDDEN;
+/*
+ * Internal variants for with extra __pmContext parameter to allow
+ * for context being locked or not locked
+ */
+extern int pmLookupName_ctx(__pmContext *, int, char **, pmID *) _PCP_HIDDEN;
+extern int pmNameAll_ctx(__pmContext *, pmID, char ***) _PCP_HIDDEN;
+extern int pmLookupDesc_ctx(__pmContext *, pmID, pmDesc *) _PCP_HIDDEN;
+extern int pmNameInDom_ctx(__pmContext *, pmInDom, int, char **) _PCP_HIDDEN;
+extern int pmGetInDomArchive_ctx(__pmContext *, pmInDom, int **, char ***) _PCP_HIDDEN;
+extern int pmFetch_ctx(__pmContext *, int, pmID *, pmResult **) _PCP_HIDDEN;
+extern int pmStore_ctx(__pmContext *, const pmResult *) _PCP_HIDDEN;
+extern int __pmDecodeResult_ctx(__pmContext *, __pmPDU *, pmResult **) _PCP_HIDDEN;
+extern int __pmSendResult_ctx(__pmContext *, int, int, const pmResult *) _PCP_HIDDEN;
+extern void __pmDumpResult_ctx(__pmContext *, FILE *, const pmResult *) _PCP_HIDDEN;
+extern int pmGetArchiveEnd_ctx(__pmContext *, struct timeval *) _PCP_HIDDEN;
+extern int __pmGetArchiveEnd_ctx(__pmContext *, struct timeval *) _PCP_HIDDEN;
+extern int __pmLogGenerateMark_ctx(__pmContext *, int, pmResult **) _PCP_HIDDEN;
+
+#ifdef BUILD_WITH_LOCK_ASSERTS
+#include <assert.h>
+#define PM_ASSERT_IS_LOCKED(lock) assert(__pmIsLocked(&(lock)))
+#define PM_ASSERT_IS_UNLOCKED(lock) assert(!__pmIsLocked(&(lock)))
+#else
+#define PM_ASSERT_IS_LOCKED(lock)
+#define PM_ASSERT_IS_UNLOCKED(lock)
+#endif /* BUILD_WITH_LOCK_ASSERTS */
+
 
 #endif /* _LIBPCP_INTERNAL_H */
