@@ -36,6 +36,7 @@
 #include <assert.h>
 #include "pmapi.h"
 #include "impl.h"
+#include "internal.h"
 
 #define UPD_MARK_NONE	0
 #define UPD_MARK_FORW	1
@@ -124,8 +125,9 @@ static long	nr[PM_MODE_BACK+1];
  * called with the context lock held
  */
 static int
-cache_read(__pmArchCtl *acp, int mode, pmResult **rp)
+cache_read(__pmContext *ctxp, int mode, pmResult **rp)
 {
+    __pmArchCtl	*acp = ctxp->c_archctl;
     long	posn;
     cache_t	*cp;
     cache_t	*lfup;
@@ -140,14 +142,14 @@ cache_read(__pmArchCtl *acp, int mode, pmResult **rp)
      * changed direction, then we need to generate that record again.
      */
     if (acp->ac_mark_done != 0 && acp->ac_mark_done != mode) {
-	sts = __pmLogGenerateMark(acp->ac_log, acp->ac_mark_done, rp);
+	sts = __pmLogGenerateMark_ctx(ctxp, acp->ac_mark_done, rp);
 	acp->ac_mark_done = 0;
 	return sts;
     }
 
     /* Look for a cache hit. */
     if (acp->ac_vol == acp->ac_log->l_curvol) {
-	posn = ftell(acp->ac_log->l_mfp);
+	posn = __pmFtell(acp->ac_log->l_mfp);
 	assert(posn >= 0);
     }
     else
@@ -166,7 +168,7 @@ cache_read(__pmArchCtl *acp, int mode, pmResult **rp)
 #ifdef PCP_DEBUG
     if ((pmDebug & DBG_TRACE_LOG) && (pmDebug & DBG_TRACE_DESPERATE)) {
 	fprintf(stderr, "cache_read: fd=%d mode=%s vol=%d (curvol=%d) %s_posn=%ld ",
-	    fileno(acp->ac_log->l_mfp),
+	    __pmFileno(acp->ac_log->l_mfp),
 	    mode == PM_MODE_FORW ? "forw" : "back",
 	    acp->ac_vol, acp->ac_log->l_curvol,
 	    mode == PM_MODE_FORW ? "head" : "tail",
@@ -185,9 +187,9 @@ cache_read(__pmArchCtl *acp, int mode, pmResult **rp)
 	    *rp = cp->rp;
 	    cp->used++;
 	    if (mode == PM_MODE_FORW)
-		fseek(acp->ac_log->l_mfp, cp->tail_posn, SEEK_SET);
+		__pmFseek(acp->ac_log->l_mfp, cp->tail_posn, SEEK_SET);
 	    else
-		fseek(acp->ac_log->l_mfp, cp->head_posn, SEEK_SET);
+		__pmFseek(acp->ac_log->l_mfp, cp->head_posn, SEEK_SET);
 #ifdef PCP_DEBUG
 	    if ((pmDebug & DBG_TRACE_LOG) && (pmDebug & DBG_TRACE_DESPERATE)) {
 		__pmTimeval	tmp;
@@ -226,7 +228,7 @@ cache_read(__pmArchCtl *acp, int mode, pmResult **rp)
     }
     save_curvol = acp->ac_log->l_curvol;
 
-    lfup->sts = __pmLogRead(acp->ac_log, mode, NULL, &lfup->rp, PMLOGREAD_NEXT);
+    lfup->sts = __pmLogRead_ctx(ctxp, mode, NULL, &lfup->rp, PMLOGREAD_NEXT);
     if (lfup->sts < 0)
 	lfup->rp = NULL;
     *rp = lfup->rp;
@@ -236,7 +238,7 @@ cache_read(__pmArchCtl *acp, int mode, pmResult **rp)
 
     /*
      * vol/arch switch since last time, or vol/arch switch or virtual mark
-     * record generated in __pmLogRead() ...
+     * record generated in __pmLogRead_ctx() ...
      * new vol/arch, stdio stream and we don't know where we started from
      * ... don't cache
      */
@@ -269,12 +271,12 @@ cache_read(__pmArchCtl *acp, int mode, pmResult **rp)
 	}
 	if (mode == PM_MODE_FORW) {
 	    lfup->head_posn = posn;
-	    lfup->tail_posn = ftell(acp->ac_log->l_mfp);
+	    lfup->tail_posn = __pmFtell(acp->ac_log->l_mfp);
 	    assert(lfup->tail_posn >= 0);
 	}
 	else {
 	    lfup->tail_posn = posn;
-	    lfup->head_posn = ftell(acp->ac_log->l_mfp);
+	    lfup->head_posn = __pmFtell(acp->ac_log->l_mfp);
 	    assert(lfup->head_posn >= 0);
 	}
 #ifdef PCP_DEBUG
@@ -296,7 +298,7 @@ cache_read(__pmArchCtl *acp, int mode, pmResult **rp)
 }
 
 void
-__pmLogCacheClear(FILE *mfp)
+__pmLogCacheClear(__pmFILE *mfp)
 {
     /* retired ... functionality moved to __pmFreeInterpData() */
     return;
@@ -635,7 +637,7 @@ do_roll(__pmContext *ctxp, double t_req, int *seen_mark)
      * to make sure we are up to t_req
      */
     if (ctxp->c_delta > 0) {
-	while (cache_read(ctxp->c_archctl, PM_MODE_FORW, &logrp) >= 0) {
+	while (cache_read(ctxp, PM_MODE_FORW, &logrp) >= 0) {
 	    tmp.tv_sec = (__int32_t)logrp->timestamp.tv_sec;
 	    tmp.tv_usec = (__int32_t)logrp->timestamp.tv_usec;
 	    t_this = __pmTimevalSub(&tmp, __pmLogStartTime(ctxp->c_archctl));
@@ -647,7 +649,7 @@ do_roll(__pmContext *ctxp, double t_req, int *seen_mark)
 		fprintf(stderr, "do_roll: forw to t=%.6f%s\n",
 		    t_this, logrp->numpmid == 0 ? " <mark>" : "");
 #endif
-	    ctxp->c_archctl->ac_offset = ftell(ctxp->c_archctl->ac_log->l_mfp);
+	    ctxp->c_archctl->ac_offset = __pmFtell(ctxp->c_archctl->ac_log->l_mfp);
 	    assert(ctxp->c_archctl->ac_offset >= 0);
 	    ctxp->c_archctl->ac_vol = ctxp->c_archctl->ac_log->l_curvol;
 	    sts = update_bounds(ctxp, t_req, logrp, UPD_MARK_FORW, NULL, seen_mark);
@@ -656,7 +658,7 @@ do_roll(__pmContext *ctxp, double t_req, int *seen_mark)
 	}
     }
     else {
-	while (cache_read(ctxp->c_archctl, PM_MODE_BACK, &logrp) >= 0) {
+	while (cache_read(ctxp, PM_MODE_BACK, &logrp) >= 0) {
 	    tmp.tv_sec = (__int32_t)logrp->timestamp.tv_sec;
 	    tmp.tv_usec = (__int32_t)logrp->timestamp.tv_usec;
 	    t_this = __pmTimevalSub(&tmp, __pmLogStartTime(ctxp->c_archctl));
@@ -668,7 +670,7 @@ do_roll(__pmContext *ctxp, double t_req, int *seen_mark)
 		fprintf(stderr, "do_roll: back to t=%.6f%s\n",
 		    t_this, logrp->numpmid == 0 ? " <mark>" : "");
 #endif
-	    ctxp->c_archctl->ac_offset = ftell(ctxp->c_archctl->ac_log->l_mfp);
+	    ctxp->c_archctl->ac_offset = __pmFtell(ctxp->c_archctl->ac_log->l_mfp);
 	    assert(ctxp->c_archctl->ac_offset >= 0);
 	    ctxp->c_archctl->ac_vol = ctxp->c_archctl->ac_log->l_curvol;
 	    sts = update_bounds(ctxp, t_req, logrp, UPD_MARK_BACK, NULL, seen_mark);
@@ -761,7 +763,7 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	 * Past end of the current archive ... see if it has grown since we
 	 * last looked, or if we can switch to the next archive in the context.
 	 */
-	if (__pmGetArchiveEnd(ctxp->c_archctl->ac_log, &end) >= 0) {
+	if (__pmGetArchiveEnd_ctx(ctxp, &end) >= 0) {
 	    tmp.tv_sec = (__int32_t)end.tv_sec;
 	    tmp.tv_usec = (__int32_t)end.tv_usec;
 	    ctxp->c_archctl->ac_end = __pmTimevalSub(&tmp, __pmLogStartTime(ctxp->c_archctl));
@@ -819,7 +821,7 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 		    instlist[0] = PM_IN_NULL;
 		}
 		else {
-		    sts = pmGetInDomArchive(pcp->desc.indom, &instlist, &namelist);
+		    sts = pmGetInDomArchive_ctx(ctxp, pcp->desc.indom, &instlist, &namelist);
 		    if (sts > 0) {
 			/* Pre allocate enough space for the instance domain. */
 			hsts = __pmHashPreAlloc(sts, &pcp->hc);
@@ -901,7 +903,7 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
     if (ctxp->c_archctl->ac_serial == 0) {
 	/* need gross positioning from temporal index */
 	__pmLogSetTime(ctxp);
-	ctxp->c_archctl->ac_offset = ftell(ctxp->c_archctl->ac_log->l_mfp);
+	ctxp->c_archctl->ac_offset = __pmFtell(ctxp->c_archctl->ac_log->l_mfp);
 	assert(ctxp->c_archctl->ac_offset >= 0);
 	ctxp->c_archctl->ac_vol = ctxp->c_archctl->ac_log->l_curvol;
 
@@ -911,14 +913,14 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	 * to make sure
 	 */
 	if (ctxp->c_delta > 0) {
-	    while (cache_read(ctxp->c_archctl, PM_MODE_BACK, &logrp) >= 0) {
+	    while (cache_read(ctxp, PM_MODE_BACK, &logrp) >= 0) {
 		tmp.tv_sec = (__int32_t)logrp->timestamp.tv_sec;
 		tmp.tv_usec = (__int32_t)logrp->timestamp.tv_usec;
 		t_this = __pmTimevalSub(&tmp, __pmLogStartTime(ctxp->c_archctl));
 		if (t_this <= t_req) {
 		    break;
 		}
-		ctxp->c_archctl->ac_offset = ftell(ctxp->c_archctl->ac_log->l_mfp);
+		ctxp->c_archctl->ac_offset = __pmFtell(ctxp->c_archctl->ac_log->l_mfp);
 		assert(ctxp->c_archctl->ac_offset >= 0);
 		ctxp->c_archctl->ac_vol = ctxp->c_archctl->ac_log->l_curvol;
 		sts = update_bounds(ctxp, t_req, logrp, UPD_MARK_NONE, NULL, NULL);
@@ -928,14 +930,14 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	    }
 	}
 	else {
-	    while (cache_read(ctxp->c_archctl, PM_MODE_FORW, &logrp) >= 0) {
+	    while (cache_read(ctxp, PM_MODE_FORW, &logrp) >= 0) {
 		tmp.tv_sec = (__int32_t)logrp->timestamp.tv_sec;
 		tmp.tv_usec = (__int32_t)logrp->timestamp.tv_usec;
 		t_this = __pmTimevalSub(&tmp, __pmLogStartTime(ctxp->c_archctl));
 		if (t_this > t_req) {
 		    break;
 		}
-		ctxp->c_archctl->ac_offset = ftell(ctxp->c_archctl->ac_log->l_mfp);
+		ctxp->c_archctl->ac_offset = __pmFtell(ctxp->c_archctl->ac_log->l_mfp);
 		assert(ctxp->c_archctl->ac_offset >= 0);
 		ctxp->c_archctl->ac_vol = ctxp->c_archctl->ac_log->l_curvol;
 		sts = update_bounds(ctxp, t_req, logrp, UPD_MARK_NONE, NULL, NULL);
@@ -960,7 +962,7 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 
     /* get to the last remembered place */
     __pmLogChangeVol(ctxp->c_archctl->ac_log, ctxp->c_archctl->ac_vol);
-    fseek(ctxp->c_archctl->ac_log->l_mfp, ctxp->c_archctl->ac_offset, SEEK_SET);
+    __pmFseek(ctxp->c_archctl->ac_log->l_mfp, ctxp->c_archctl->ac_offset, SEEK_SET);
 
     seen_mark = 0;	/* interested in <mark> records seen from here on */
 
@@ -1040,11 +1042,11 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	 * position ourselves, ... and search
 	 */
 	__pmLogChangeVol(ctxp->c_archctl->ac_log, ctxp->c_archctl->ac_vol);
-	fseek(ctxp->c_archctl->ac_log->l_mfp, ctxp->c_archctl->ac_offset, SEEK_SET);
+	__pmFseek(ctxp->c_archctl->ac_log->l_mfp, ctxp->c_archctl->ac_offset, SEEK_SET);
 	done = 0;
 
 	while (done < back) {
-	    if (cache_read(ctxp->c_archctl, PM_MODE_BACK, &logrp) < 0) {
+	    if (cache_read(ctxp, PM_MODE_BACK, &logrp) < 0) {
 		/* ran into start of log */
 #ifdef PCP_DEBUG
 		if (pmDebug & DBG_TRACE_INTERP) {
@@ -1059,7 +1061,7 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	    t_this = __pmTimevalSub(&tmp, __pmLogStartTime(ctxp->c_archctl));
 	    if (ctxp->c_delta < 0 && t_this >= t_req) {
 		/* going backwards, and not up to t_req yet */
-		ctxp->c_archctl->ac_offset = ftell(ctxp->c_archctl->ac_log->l_mfp);
+		ctxp->c_archctl->ac_offset = __pmFtell(ctxp->c_archctl->ac_log->l_mfp);
 		assert(ctxp->c_archctl->ac_offset >= 0);
 		ctxp->c_archctl->ac_vol = ctxp->c_archctl->ac_log->l_curvol;
 	    }
@@ -1176,12 +1178,12 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	 * position ourselves ... and search
 	 */
 	__pmLogChangeVol(ctxp->c_archctl->ac_log, ctxp->c_archctl->ac_vol);
-	fseek(ctxp->c_archctl->ac_log->l_mfp, ctxp->c_archctl->ac_offset, SEEK_SET);
+	__pmFseek(ctxp->c_archctl->ac_log->l_mfp, ctxp->c_archctl->ac_offset, SEEK_SET);
 	done = 0;
 
 	sts = 0;
 	while (done < forw) {
-	    if ((sts = cache_read(ctxp->c_archctl, PM_MODE_FORW, &logrp)) < 0) {
+	    if ((sts = cache_read(ctxp, PM_MODE_FORW, &logrp)) < 0) {
 		/* ran into end of log */
 #ifdef PCP_DEBUG
 		if (pmDebug & DBG_TRACE_INTERP) {
@@ -1196,7 +1198,7 @@ __pmLogFetchInterp(__pmContext *ctxp, int numpmid, pmID pmidlist[], pmResult **r
 	    t_this = __pmTimevalSub(&tmp, __pmLogStartTime(ctxp->c_archctl));
 	    if (ctxp->c_delta > 0 && t_this <= t_req) {
 		/* going forwards, and not up to t_req yet */
-		ctxp->c_archctl->ac_offset = ftell(ctxp->c_archctl->ac_log->l_mfp);
+		ctxp->c_archctl->ac_offset = __pmFtell(ctxp->c_archctl->ac_log->l_mfp);
 		assert(ctxp->c_archctl->ac_offset >= 0);
 		ctxp->c_archctl->ac_vol = ctxp->c_archctl->ac_log->l_curvol;
 	    }
