@@ -1135,8 +1135,7 @@ __pmParseUnitsStrPart(const char *str, const char *str_end, pmUnitsBig * out, do
     int sts = 0;
     unsigned i;
     const char *ptr;		/* scanning along str */
-    enum dimension_t
-    { d_none, d_space, d_time, d_count } dimension;
+    enum { d_none, d_space, d_time, d_count } dimension;
     struct unit_keyword_t
     {
 	const char *keyword;
@@ -1150,10 +1149,10 @@ __pmParseUnitsStrPart(const char *str, const char *str_end, pmUnitsBig * out, do
 	{"milliseconds", PM_TIME_MSEC}, {"millisecond", PM_TIME_MSEC},
 	{"millisec", PM_TIME_MSEC}, {"ms", PM_TIME_MSEC},
 	{"seconds", PM_TIME_SEC}, {"second", PM_TIME_SEC},
-	{"sec", PM_TIME_SEC},
-	{"s", PM_TIME_SEC},
+	{"sec", PM_TIME_SEC}, {"s", PM_TIME_SEC},
 	{"minutes", PM_TIME_MIN}, {"minute", PM_TIME_MIN}, {"min", PM_TIME_MIN},
-	{"hours", PM_TIME_HOUR}, {"hour", PM_TIME_HOUR}, {"hr", PM_TIME_HOUR},
+	{"hours", PM_TIME_HOUR}, {"hour", PM_TIME_HOUR},
+	{"hr", PM_TIME_HOUR}, {"h", PM_TIME_HOUR},
 	{"time-0", 0}, /* { "time-1", 1 }, */ {"time-2", 2}, {"time-3", 3},
 	{"time-4", 4}, {"time-5", 5}, {"time-6", 6}, {"time-7", 7},
 	{"time-8", 8}, {"time-9", 9}, {"time-10", 10}, {"time-11", 11},
@@ -1343,11 +1342,9 @@ out:
     return sts;
 }
 
-
-
 /* Parse a general "N $units / M $units" string into a pmUnits tuple and a multiplier. */
 int
-pmParseUnitsStr(const char *str, pmUnits * out, double *multiplier, char **errMsg)
+pmParseUnitsStr(const char *str, pmUnits *out, double *multiplier, char **errMsg)
 {
     const char *slash;
     double dividend_mult, divisor_mult;
@@ -1460,8 +1457,8 @@ pmParseUnitsStr(const char *str, pmUnits * out, double *multiplier, char **errMs
 	    [PM_TIME_HOUR] = 3600
 	};
 	/* guaranteed by __pmParseUnitsStrPart; ensure in-range array access */
-	assert(dividend.scaleTime >= PM_TIME_NSEC && dividend.scaleTime <= PM_TIME_HOUR);
-	assert(divisor.scaleTime >= PM_TIME_NSEC && divisor.scaleTime <= PM_TIME_HOUR);
+	assert(dividend.scaleTime <= PM_TIME_HOUR);
+	assert(divisor.scaleTime <= PM_TIME_HOUR);
 	*multiplier *= pow(time_scales[dividend.scaleTime], -(double) dividend.dimTime);
 	*multiplier *= pow(time_scales[divisor.scaleTime], (double) divisor.dimTime);
 	if (out->dimTime == 0)	/* became dimensionless? */
