@@ -1,7 +1,7 @@
 /*
  * General Utility Routines
  *
- * Copyright (c) 2012-2016 Red Hat.
+ * Copyright (c) 2012-2017 Red Hat.
  * Copyright (c) 2009 Aconex.  All Rights Reserved.
  * Copyright (c) 1995-2002,2004 Silicon Graphics, Inc.  All Rights Reserved.
  * 
@@ -504,6 +504,9 @@ pmEventFlagsStr_r(int flags, char *buf, int buflen)
      */
     int started = 0;
 
+    if (buflen < 26)
+	return NULL;
+
     if (flags & PM_EVENT_FLAG_MISSED)
 	return strcpy(buf, "missed");
 
@@ -671,7 +674,7 @@ restore_debug(int saved)
 }
 
 static void
-dump_valueset(FILE *f, pmValueSet *vsp)
+dump_valueset(__pmContext *ctxp, FILE *f, pmValueSet *vsp)
 {
     pmDesc	desc;
     char	errmsg[PM_MAXERRMSGLEN];
@@ -683,7 +686,7 @@ dump_valueset(FILE *f, pmValueSet *vsp)
     int		n, j;
 
     pmid = pmIDStr_r(vsp->pmid, strbuf, sizeof(strbuf));
-    if ((n = pmNameAll(vsp->pmid, &names)) < 0)
+    if ((n = pmNameAll_ctx(ctxp, vsp->pmid, &names)) < 0)
 	fprintf(f, "  %s (%s):", pmid, "<noname>");
     else {
 	int	j;
@@ -705,7 +708,7 @@ dump_valueset(FILE *f, pmValueSet *vsp)
 	return;
     }
     if (__pmGetInternalState() == PM_STATE_PMCS ||
-	pmLookupDesc(vsp->pmid, &desc) < 0) {
+	pmLookupDesc_ctx(ctxp, vsp->pmid, &desc) < 0) {
 	/* don't know, so punt on the most common cases */
 	desc.indom = PM_INDOM_NULL;
 	have_desc = 0;
@@ -718,7 +721,7 @@ dump_valueset(FILE *f, pmValueSet *vsp)
 	if (vsp->numval > 1 || vp->inst != PM_INDOM_NULL) {
 	    fprintf(f, "    inst [%d", vp->inst);
 	    if (have_desc &&
-		pmNameInDom(desc.indom, vp->inst, &p) >= 0) {
+		pmNameInDom_ctx(ctxp, desc.indom, vp->inst, &p) >= 0) {
 		fprintf(f, " or \"%s\"]", p);
 		free(p);
 	    }
@@ -743,36 +746,57 @@ dump_valueset(FILE *f, pmValueSet *vsp)
     }
 }
 
+/* Internal variant of __pmDumpResult() with current context. */
 void
-__pmDumpResult(FILE *f, const pmResult *resp)
+__pmDumpResult_ctx(__pmContext *ctxp, FILE *f, const pmResult *resp)
 {
     int		i, saved;
 
+    if (ctxp != NULL)
+	PM_ASSERT_IS_LOCKED(ctxp->c_lock);
+
     saved = save_debug();
     fprintf(f, "pmResult dump from " PRINTF_P_PFX "%p timestamp: %d.%06d ",
 	resp, (int)resp->timestamp.tv_sec, (int)resp->timestamp.tv_usec);
     __pmPrintStamp(f, &resp->timestamp);
     fprintf(f, " numpmid: %d\n", resp->numpmid);
     for (i = 0; i < resp->numpmid; i++)
-	dump_valueset(f, resp->vset[i]);
+	dump_valueset(ctxp, f, resp->vset[i]);
     restore_debug(saved);
 }
 
 void
-__pmDumpHighResResult(FILE *f, const pmHighResResult *hresp)
+__pmDumpResult(FILE *f, const pmResult *resp)
+{
+    __pmDumpResult_ctx(NULL, f, resp);
+}
+
+/* Internal variant of __pmDumpHighResResult() with current context. */
+void
+__pmDumpHighResResult_ctx(__pmContext *ctxp, FILE *f, const pmHighResResult *hresp)
 {
     int		i, saved;
 
+    if (ctxp != NULL)
+	PM_ASSERT_IS_LOCKED(ctxp->c_lock);
+
     saved = save_debug();
     fprintf(f, "pmHighResResult dump from " PRINTF_P_PFX "%p timestamp: %d.%09d ",
 	hresp, (int)hresp->timestamp.tv_sec, (int)hresp->timestamp.tv_nsec);
     __pmPrintHighResStamp(f, &hresp->timestamp);
     fprintf(f, " numpmid: %d\n", hresp->numpmid);
     for (i = 0; i < hresp->numpmid; i++)
-	dump_valueset(f, hresp->vset[i]);
+	dump_valueset(ctxp, f, hresp->vset[i]);
     restore_debug(saved);
 }
 
+void
+__pmDumpHighResResult(FILE *f, const pmHighResResult *hresp)
+{
+    __pmDumpHighResResult_ctx(NULL, f, hresp);
+
+}
+
 static void
 print_event_summary(FILE *f, const pmValue *val, int highres)
 {
@@ -1412,7 +1436,7 @@ pmfstate(int state)
     static int	errtype = PM_QUERYERR;
     char	errmsg[PM_MAXERRMSGLEN];
 
-    ASSERT_IS_LOCKED(util_lock);
+    PM_ASSERT_IS_LOCKED(util_lock);
 
     if (state > PM_QUERYERR)
 	errtype = state;
@@ -1658,9 +1682,19 @@ __pmSetClientId(const char *id)
     char        	*ipaddr = NULL;
     __pmHostEnt		*servInfo;
     int			vblen;
+    __pmContext		*ctxp;
 
-    if ((sts = pmLookupName(1, &name, &pmid)) < 0)
+    if ((sts = pmWhichContext()) < 0) {
 	return sts;
+    }
+    ctxp = __pmHandleToPtr(sts);
+    if (ctxp == NULL)
+	return PM_ERR_NOCONTEXT;
+
+    if ((sts = pmLookupName_ctx(ctxp, 1, &name, &pmid)) < 0) {
+	PM_UNLOCK(ctxp->c_lock);
+	return sts;
+    }
 
     /*
      * Try to obtain the address and the actual host name.
@@ -1713,6 +1747,7 @@ __pmSetClientId(const char *id)
     /* build pmResult for pmStore() */
     pmvb = (pmValueBlock *)calloc(1, PM_VAL_HDR_SIZE+vblen);
     if (pmvb == NULL) {
+	PM_UNLOCK(ctxp->c_lock);
 	__pmNoMem("__pmSetClientId", PM_VAL_HDR_SIZE+vblen, PM_RECOV_ERR);
 	return -ENOMEM;
     }
@@ -1737,8 +1772,9 @@ __pmSetClientId(const char *id)
     memset(&store, 0, sizeof(store));
     store.numpmid = 1;
     store.vset[0] = &pmvs;
-    sts = pmStore(&store);
+    sts = pmStore_ctx(ctxp, &store);
     free(pmvb);
+    PM_UNLOCK(ctxp->c_lock);
     return sts;
 }
 
