@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2015 Red Hat.
+ * Copyright (c) 2013-2016 Red Hat.
  * Copyright (c) 2008-2010 Aconex.  All Rights Reserved.
  * 
  * This library is free software; you can redistribute it and/or modify it
@@ -11,6 +11,11 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
+ *
+ *
+ * Thread-safe notes
+ *
+ * rand() et al are not thread-safe, but we don't really care here.
  */
 
 /*
@@ -85,11 +90,11 @@ MapSignals(int sig, int *index)
 int
 __pmSetSignalHandler(int sig, __pmSignalHandler func)
 {
-    int sts, index;
+    int sts, index = 0;
     char *signame, evname[64];
     HANDLE eventhdl, waithdl;
 
-    if ((signame = MapSignals(sig, &index)) < 0)
+    if ((signame = MapSignals(sig, &index)) == NULL)
 	return index;
 
     if (signals[index].callback) {	/* remove old handler */
@@ -543,11 +548,13 @@ static char *eventlogPrefix;
 void
 openlog(const char *ident, int option, int facility)
 {
+    PM_LOCK(__pmLock_extcall);
     if (eventlog)
 	closelog();
-    eventlog = RegisterEventSource(NULL, "Application");
+    eventlog = RegisterEventSource(NULL, "Application");	/* THREADSAFE */
     if (ident)
 	eventlogPrefix = strdup(ident);
+    PM_UNLOCK(__pmLock_extcall);
 }
 
 void
@@ -594,19 +601,22 @@ syslog(int priority, const char *format, ...)
 void
 closelog(void)
 {
+    PM_LOCK(__pmLock_extcall);
     if (eventlog) {
-	DeregisterEventSource(eventlog);
+	DeregisterEventSource(eventlog);		/* THREADSAFE */
 	if (eventlogPrefix)
 	    free(eventlogPrefix);
 	eventlogPrefix = NULL;
     }
     eventlog = NULL;
+    PM_UNLOCK(__pmLock_extcall);
 }
 
 const char *
 strerror_r(int errnum, char *buf, size_t buflen)
 {
-    return strerror_s(buf, buflen, errnum);
+    strerror_s(buf, buflen, errnum);
+    return (const char *)buf;
 }
 
 /* Windows socket error codes - what a nightmare! */
@@ -779,3 +789,47 @@ __pmGetUserIdentity(const char *username, __pmUserID *uid, __pmGroupID *gid, int
 {
     return -ENOTSUP;	/* NYI */
 }
+
+/*
+ * always called with __pmLock_extcall already held, so THREADSAFE
+ */
+int
+setenv(const char *name, const char *value, int overwrite)
+{
+    char	*ebuf;
+
+    if (getenv(name) != NULL) {		/* THREADSAFE */
+	/* already in the environment */
+	if (!overwrite)
+	    return(0);
+    }
+
+    if ((ebuf = (char *)malloc(strlen(name) + strlen(value) + 2)) == NULL)
+	return -1;
+
+    strncpy(ebuf, name, strlen(name)+1);
+    strncat(ebuf, "=", 1);
+    strncat(ebuf, value, strlen(value));
+
+    return _putenv(ebuf);		/* THREADSAFE */
+}
+
+/*
+ * always called with __pmLock_extcall already held, so THREADSAFE
+ */
+int
+unsetenv(const char *name)
+{
+    char	*ebuf;
+    int		sts;
+
+    if ((ebuf = (char *)malloc(strlen(name) + 2)) == NULL)
+	return -1;
+
+    /* strange but true */
+    strncpy(ebuf, name, strlen(name)+1);
+    strncat(ebuf, "=", 1);
+    sts = _putenv(ebuf);		/* THREADSAFE */
+    free(ebuf);
+    return sts;
+}
