@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013,2015 Red Hat.
+ * Copyright (c) 2013,2015-2017 Red Hat.
  * Copyright (c) 2005 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This library is free software; you can redistribute it and/or modify it
@@ -16,12 +16,11 @@
 #include "pmapi.h"
 #include "impl.h"
 #include "pmda.h"
+#include "libdefs.h"
 #include <fcntl.h>
 #include <ctype.h>
 #include <sys/stat.h>
 
-static __uint32_t hash(const signed char *, int, __uint32_t);
-
 /*
  * simple linked list for each cache at this stage
  */
@@ -773,8 +772,7 @@ load_cache(hdr_t *h)
 		s = 0;
 	    break;
     }
-    if (s == 0 || h->ins_mode < 0 || h->ins_mode > 1 || h->maxinst < 0
-	|| h->maxinst > DEFAULT_MAXINST) {
+    if (s == 0 || h->ins_mode < 0 || h->ins_mode > 1 || h->maxinst < 0) {
 	__pmNotifyErr(LOG_ERR, 
 	     "pmdaCacheOp: %s: illegal cache header record: %s",
 	     filename, buf);
@@ -886,9 +884,10 @@ save_cache(hdr_t *h, int hstate)
     int		cnt;
     time_t	now;
     int		sep = __pmPathSeparator();
+    int		state = h->hstate & ~CACHE_STRINGS;
     char	strbuf[20];
 
-    if ((h->hstate & hstate) == 0 || (h->hstate & CACHE_STRINGS) != 0) {
+    if ((state & hstate) == 0) {
 	/* nothing to be done */
 	return 0;
     }
@@ -927,7 +926,7 @@ save_cache(hdr_t *h, int hstate)
 	cnt++;
     }
     fclose(fp);
-    h->hstate = 0;
+    h->hstate &= ~(DIRTY_INSTANCE | DIRTY_STAMP);
 
 #ifdef PCP_DEBUG
     if (pmDebug & DBG_TRACE_INDOM) {
@@ -1470,134 +1469,3 @@ int pmdaCacheResize(pmInDom indom, int maximum)
     h->hstate |= DIRTY_STAMP;
     return 0;
 }
-
-/*
---------------------------------------------------------------------
-lookup2.c, by Bob Jenkins, December 1996, Public Domain.
-hash(), hash2(), hash3, and mix() are externally useful functions.
-Routines to test the hash are included if SELF_TEST is defined.
-You can use this free for any purpose.  It has no warranty.
---------------------------------------------------------------------
-*/
-
-#include <stdio.h>
-#include <stddef.h>
-#include <stdlib.h>
-typedef  unsigned long  int  ub4;   /* unsigned 4-byte quantities */
-typedef  unsigned       char ub1;
-
-#define hashsize(n) ((ub4)1<<(n))
-#define hashmask(n) (hashsize(n)-1)
-
-/*
---------------------------------------------------------------------
-mix -- mix 3 32-bit values reversibly.
-For every delta with one or two bit set, and the deltas of all three
-  high bits or all three low bits, whether the original value of a,b,c
-  is almost all zero or is uniformly distributed,
-* If mix() is run forward or backward, at least 32 bits in a,b,c
-  have at least 1/4 probability of changing.
-* If mix() is run forward, every bit of c will change between 1/3 and
-  2/3 of the time.  (Well, 22/100 and 78/100 for some 2-bit deltas.)
-mix() was built out of 36 single-cycle latency instructions in a 
-  structure that could supported 2x parallelism, like so:
-      a -= b; 
-      a -= c; x = (c>>13);
-      b -= c; a ^= x;
-      b -= a; x = (a<<8);
-      c -= a; b ^= x;
-      c -= b; x = (b>>13);
-      ...
-  Unfortunately, superscalar Pentiums and Sparcs can't take advantage 
-  of that parallelism.  They've also turned some of those single-cycle
-  latency instructions into multi-cycle latency instructions.  Still,
-  this is the fastest good hash I could find.  There were about 2^^68
-  to choose from.  I only looked at a billion or so.
---------------------------------------------------------------------
-*/
-#define mix(a,b,c) \
-{ \
-  a -= b; a -= c; a ^= (c>>13); \
-  b -= c; b -= a; b ^= (a<<8); \
-  c -= a; c -= b; c ^= (b>>13); \
-  a -= b; a -= c; a ^= (c>>12);  \
-  b -= c; b -= a; b ^= (a<<16); \
-  c -= a; c -= b; c ^= (b>>5); \
-  a -= b; a -= c; a ^= (c>>3);  \
-  b -= c; b -= a; b ^= (a<<10); \
-  c -= a; c -= b; c ^= (b>>15); \
-}
-
-/*
---------------------------------------------------------------------
-hash() -- hash a variable-length key into a 32-bit value
-  k     : the key (the unaligned variable-length array of bytes)
-  len   : the length of the key, counting by bytes
-  level : can be any 4-byte value
-Returns a 32-bit value.  Every bit of the key affects every bit of
-the return value.  Every 1-bit and 2-bit delta achieves avalanche.
-About 36+6len instructions.
-
-The best hash table sizes are powers of 2.  There is no need to do
-mod a prime (mod is sooo slow!).  If you need less than 32 bits,
-use a bitmask.  For example, if you need only 10 bits, do
-  h = (h & hashmask(10));
-In which case, the hash table should have hashsize(10) elements.
-
-If you are hashing n strings (ub1 **)k, do it like this:
-  for (i=0, h=0; i<n; ++i) h = hash( k[i], len[i], h);
-
-By Bob Jenkins, 1996.  bob_jenkins@burtleburtle.net.  You may use this
-code any way you wish, private, educational, or commercial.  It's free.
-
-See http://burlteburtle.net/bob/hash/evahash.html
-Use for hash table lookup, or anything where one collision in 2^32 is
-acceptable.  Do NOT use for cryptographic purposes.
---------------------------------------------------------------------
-*/
-
-static __uint32_t
-hash(const signed char *k, int length, __uint32_t initval)
-{
-   __uint32_t a,b,c,len;
-
-   /* Set up the internal state */
-   len = length;
-   a = b = 0x9e3779b9;  /* the golden ratio; an arbitrary value */
-   c = initval;           /* the previous hash value */
-
-   /*---------------------------------------- handle most of the key */
-   while (len >= 12)
-   {
-      a += (k[0] +((__uint32_t)k[1]<<8) +((__uint32_t)k[2]<<16)
-      +((__uint32_t)k[3]<<24));
-      b += (k[4] +((__uint32_t)k[5]<<8) +((__uint32_t)k[6]<<16)
-      +((__uint32_t)k[7]<<24));
-      c += (k[8] +((__uint32_t)k[9]<<8)
-      +((__uint32_t)k[10]<<16)+((__uint32_t)k[11]<<24));
-      mix(a,b,c);
-      k += 12; len -= 12;
-   }
-
-   /*------------------------------------- handle the last 11 bytes */
-   c += length;
-   switch(len)              /* all the case statements fall through */
-   {
-   case 11: c+=((__uint32_t)k[10]<<24);
-   case 10: c+=((__uint32_t)k[9]<<16);
-   case 9 : c+=((__uint32_t)k[8]<<8);
-      /* the first byte of c is reserved for the length */
-   case 8 : b+=((__uint32_t)k[7]<<24);
-   case 7 : b+=((__uint32_t)k[6]<<16);
-   case 6 : b+=((__uint32_t)k[5]<<8);
-   case 5 : b+=k[4];
-   case 4 : a+=((__uint32_t)k[3]<<24);
-   case 3 : a+=((__uint32_t)k[2]<<16);
-   case 2 : a+=((__uint32_t)k[1]<<8);
-   case 1 : a+=k[0];
-     /* case 0: nothing left to add */
-   }
-   mix(a,b,c);
-   /*-------------------------------------------- report the result */
-   return c;
-}
