@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, Red Hat.
+ * Copyright (c) 2012-2017, Red Hat.  All Rights Reserved.
  * Copyright (c) 2007-2008, Aconex.  All Rights Reserved.
  * Copyright (c) 2006, Ken McDonell.  All Rights Reserved.
  * 
@@ -18,13 +18,13 @@
 
 #define DESPERATE 0
 
-GroupControl::GroupControl()
+GroupControl::GroupControl(bool restrictArchives) :
+    QmcGroup(restrictArchives)
 {
     my.samples = 0;
     my.visible = 0;
     my.realDelta = 0;
     my.realPosition = 0;
-    my.timeData = NULL;
     my.timeState = StartState;
     my.buttonState = QedTimeButton::Timeless;
     my.pmtimeState = QmcTime::StoppedState;
@@ -49,12 +49,12 @@ GroupControl::init(int samples, int visible,
     }
     my.delta = *interval;
     my.position = *position;
-    my.realDelta = tosec(*interval);
-    my.realPosition = tosec(*position);
+    my.realDelta = __pmtimevalToReal(interval);
+    my.realPosition = __pmtimevalToReal(position);
 
-    my.timeData = (double *)malloc(samples * sizeof(double));
+    my.timeData.clear();
     for (int i = 0; i < samples; i++)
-	my.timeData[i] = my.realPosition - (i * my.realDelta);
+	my.timeData.push_back(my.realPosition - (i * my.realDelta));
 }
 
 bool
@@ -117,8 +117,9 @@ GroupControl::updateTimeAxis(void)
 	pmchart->timeAxis()->setAxisScale(QwtPlot::xBottom,
 		my.timeData[my.visible - 1], my.timeData[0],
 		pmchart->timeAxis()->scaleValue(my.realDelta, my.visible));
-	pmchart->setDateLabel(my.position.tv_sec, tz);
+	pmchart->timeAxis()->clearScaleCache();
 	pmchart->timeAxis()->replot();
+	pmchart->setDateLabel(my.position.tv_sec, tz);
     } else {
 	pmchart->timeAxis()->noArchiveSources();
 	pmchart->setDateLabel(tr("[No open archives]"));
@@ -128,7 +129,7 @@ GroupControl::updateTimeAxis(void)
 	int i = my.visible - 1;
 	console->post(PmChart::DebugProtocol,
 		"GroupControl::updateTimeAxis: tz=%s; visible points=%d",
-		(const char *)tz.toAscii(), i);
+		(const char *)tz.toLatin1(), i);
 	console->post(PmChart::DebugProtocol,
 		"GroupControl::updateTimeAxis: first time is %.3f (%s)",
 		my.timeData[i], timeString(my.timeData[i]));
@@ -265,8 +266,8 @@ GroupControl::adjustWorldView(QmcTime::Packet *packet, bool vcrMode)
 {
     my.delta = packet->delta;
     my.position = packet->position;
-    my.realDelta = tosec(packet->delta);
-    my.realPosition = tosec(packet->position);
+    my.realDelta = __pmtimevalToReal(&packet->delta);
+    my.realPosition = __pmtimevalToReal(&packet->position);
 
     console->post("GroupControl::adjustWorldView: "
 		  "sh=%d vh=%d delta=%.2f position=%.2f (%s) state=%s",
@@ -379,7 +380,7 @@ GroupControl::adjustLiveWorldViewForward(QmcTime::Packet *packet)
 void
 GroupControl::adjustArchiveWorldViewForward(QmcTime::Packet *packet, bool setup)
 {
-    console->post("GroupControl::adjustArchiveWorldViewForward");
+    console->post("GroupControl::adjustArchiveWorldViewForward setup=%d", setup);
     my.timeState = ForwardState;
 
     int setmode = PM_MODE_INTERP;
@@ -412,7 +413,7 @@ GroupControl::adjustArchiveWorldViewForward(QmcTime::Packet *packet, bool setup)
 	my.timeData[i] = position;
 
 	struct timeval timeval;
-	fromsec(position, &timeval);
+	__pmtimevalFromReal(position, &timeval);
 	setArchiveMode(setmode, &timeval, delta);
 	console->post("Fetching data[%d] at %s", i, timeString(position));
 	fetch();
@@ -422,8 +423,9 @@ GroupControl::adjustArchiveWorldViewForward(QmcTime::Packet *packet, bool setup)
 		      "setting time position[%d]=%.2f[%s] state=%s count=%d",
 			i, position, timeString(position),
 			timeState(), gadgetCount());
-	for (int j = 0; j < gadgetCount(); j++)
-	    my.gadgetsList.at(j)->updateValues(true, false, my.samples, my.visible,
+	int cnt = gadgetCount();
+	for (int j = 0; j < cnt; j++)
+	    my.gadgetsList.at(j)->updateValues(true, j < cnt-1 ? false : true, my.samples, my.visible,
 						left, right, interval);
     }
 
@@ -473,7 +475,7 @@ GroupControl::adjustArchiveWorldViewBackward(QmcTime::Packet *packet, bool setup
 	my.timeData[i] = position;
 
 	struct timeval timeval;
-	fromsec(position, &timeval);
+	__pmtimevalFromReal(position, &timeval);
 	setArchiveMode(setmode, &timeval, -delta);
 	console->post("Fetching data[%d] at %s", i, timeString(position));
 	fetch();
@@ -501,6 +503,7 @@ GroupControl::adjustArchiveWorldViewBackward(QmcTime::Packet *packet, bool setup
 void
 GroupControl::adjustArchiveWorldViewStopped(QmcTime::Packet *packet, bool needFetch)
 {
+    console->post("GroupControl::adjustArchiveWorldViewStopped needFetch=%d", needFetch);
     if (needFetch) {	// stopped, but VCR reposition event occurred
 	adjustArchiveWorldViewForward(packet, needFetch);
     } else {
@@ -530,7 +533,7 @@ sideStep(double n, double o, double interval)
 void
 GroupControl::step(QmcTime::Packet *packet)
 {
-    double stepPosition = tosec(packet->position);
+    double stepPosition = __pmtimevalToReal(&packet->position);
 
     console->post(PmChart::DebugProtocol,
 	"GroupControl::step: stepping to time %.2f, delta=%.2f, state=%s",
@@ -551,13 +554,13 @@ GroupControl::step(QmcTime::Packet *packet)
     int last = my.samples - 1;
     if (packet->state == QmcTime::ForwardState) { // left-to-right (all but 1st)
 	if (my.samples > 1)
-	    memmove(&my.timeData[1], &my.timeData[0], sizeof(double) * last);
-	my.timeData[0] = my.realPosition;
+	    my.timeData.pop_back();
+	my.timeData.push_front(my.realPosition);
     }
     else if (packet->state == QmcTime::BackwardState) { // right-to-left
 	if (my.samples > 1)
-	    memmove(&my.timeData[0], &my.timeData[1], sizeof(double) * last);
-	my.timeData[last] = my.realPosition - torange(my.delta, last);
+	    my.timeData.pop_front();
+	my.timeData.push_back(my.realPosition - torange(my.delta, last));
     }
 
     fetch();
@@ -573,6 +576,7 @@ GroupControl::VCRMode(QmcTime::Packet *packet, bool dragMode)
 {
     if (!dragMode)
 	adjustWorldView(packet, true);
+    console->post("GroupControl::VCRMode dragMode=%d\n", dragMode);
 }
 
 void
@@ -582,7 +586,7 @@ GroupControl::setTimezone(QmcTime::Packet *packet, char *tz)
 
     useTZ(QString(tz));
 
-    if (isActive(packet))
+    if (isActive(packet)) 
 	updateTimeAxis();
 }
 
@@ -593,13 +597,10 @@ GroupControl::setSampleHistory(int v)
     if (my.samples != v) {
 	my.samples = v;
 
-	my.timeData = (double *)malloc(my.samples * sizeof(my.timeData[0]));
-	if (my.timeData == NULL)
-	    nomem();
-
 	double right = my.realPosition;
+	my.timeData.clear();
 	for (v = 0; v < my.samples; v++)
-	    my.timeData[v] = my.realPosition - (v * my.realDelta);
+	    my.timeData.push_back(my.realPosition - (v * my.realDelta));
 	double left = my.timeData[v-1];
 	for (v = 0; v < gadgetCount(); v++)
 	    my.gadgetsList.at(v)->resetValues(my.samples, left, right);
@@ -627,7 +628,7 @@ GroupControl::visibleHistory(void)
     return my.visible;
 }
 
-double *
+QVector<double> &
 GroupControl::timeAxisData(void)
 {
     return my.timeData;
