@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2014 Red Hat.
+ * Copyright (c) 2012-2016 Red Hat.
  * Copyright (c) 2010 Aconex.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -102,8 +102,8 @@ refresh_cgroup_devices(void)
 	}
 	/* keeping track of all fields (major/minor/inst/name) */
 	pmdaCacheStore(diskindom, PMDA_CACHE_ADD, namebuf, dev);
-	pmdaCacheLookupName(diskindom, namebuf, &dev->inst, NULL);
-	pmdaCacheLookup(diskindom, dev->inst, &dev->name, NULL);
+	(void)pmdaCacheLookupName(diskindom, namebuf, &dev->inst, NULL);
+	(void)pmdaCacheLookup(diskindom, dev->inst, &dev->name, NULL);
 
 	snprintf(buf, sizeof(buf), "%u:%u", major, minor);
 	pmdaCacheStore(devtindom, PMDA_CACHE_ADD, buf, (void *)dev);
@@ -141,7 +141,7 @@ refresh_cgroup_subsys(void)
 	sts = pmdaCacheLookupName(subsys, name, NULL, (void **)&ssp);
 	if (sts != PMDA_CACHE_INACTIVE) {
 	    if ((ssp = (subsys_t *)malloc(sizeof(subsys_t))) == NULL)
-		return;
+		continue;
 	}
 	ssp->hierarchy = hierarchy;
 	ssp->num_cgroups = num_cgroups;
@@ -233,6 +233,7 @@ cgroup_find_subsys(pmInDom indom, filesys_t *fs)
 
     memset(opts, 0, sizeof(opts));
     strncpy(buffer, fs->options, sizeof(buffer));
+    buffer[sizeof(buffer)-1] = '\0';
 
     s = strtok(buffer, ",");
     while (s) {
@@ -253,6 +254,32 @@ cgroup_find_subsys(pmInDom indom, filesys_t *fs)
     return dunno;
 }
 
+int
+cgroup_mounts_subsys(const char *system, char *buffer, int length)
+{
+    pmInDom mounts = INDOM(CGROUP_MOUNTS_INDOM);
+    pmInDom subsys = INDOM(CGROUP_SUBSYS_INDOM);
+    filesys_t *fs;
+    char *name;
+    int sts;
+
+    /* Iterate over cgroup.mounts.subsys indom, comparing the value
+     * with the given subsys - if a match is found, return the inst
+     * name, else NULL.
+     */
+    pmdaCacheOp(mounts, PMDA_CACHE_WALK_REWIND);
+    while ((sts = pmdaCacheOp(mounts, PMDA_CACHE_WALK_NEXT)) != -1) {
+	if (!pmdaCacheLookup(mounts, sts, &name, (void **)&fs))
+	    continue;
+	if (strcmp(system, cgroup_find_subsys(subsys, fs)) != 0)
+	    continue;
+	snprintf(buffer, length, "%s%s/", proc_statspath, name);
+	buffer[length-1] = '\0';
+	return strlen(buffer);
+    }
+    return 0;
+}
+
 static const char *
 cgroup_name(const char *path, int offset)
 {
@@ -268,29 +295,46 @@ cgroup_name(const char *path, int offset)
     return name;
 }
 
-static void
-cgroup_scan(const char *mnt, const char *path, cgroup_refresh_t refresh)
+static int
+check_refresh(const char *cgroup, const char *container, int container_length)
 {
-    int length;
+    /*
+     * See whether a refresh needed - if no container, always needed,
+     * otherwise we only refresh for a matching container name/path.
+     */
+    if (container_length > 0) {
+	while (*cgroup == '/')
+	    cgroup++;	/* do not compare any leading slashes */
+	return (strncmp(cgroup, container, container_length) == 0);
+    }
+    return 1;
+}
+
+static void
+cgroup_scan(const char *mnt, const char *path, cgroup_refresh_t refresh,
+		const char *container, int container_length)
+{
+    int length, mntlen = strlen(mnt) + 1;
     DIR *dirp;
     struct stat sbuf;
     struct dirent *dp;
     const char *cgname;
-    char cgpath[MAXPATHLEN];
+    char cgpath[MAXPATHLEN] = { 0 };
 
     if (path[0] == '\0') {
 	snprintf(cgpath, sizeof(cgpath), "%s%s", proc_statspath, mnt);
 	length = strlen(cgpath);
     } else {
 	snprintf(cgpath, sizeof(cgpath), "%s%s/%s", proc_statspath, mnt, path);
-	length = strlen(proc_statspath) + strlen(mnt) + 1;
+	length = strlen(proc_statspath) + mntlen;
     }
 
     if ((dirp = opendir(cgpath)) == NULL)
 	return;
 
     cgname = cgroup_name(cgpath, length);
-    refresh(cgpath, cgname);
+    if (check_refresh(cgpath + mntlen, container, container_length))
+	refresh(cgpath, cgname);
 
     /* descend into subdirectories to find all cgroups */
     while ((dp = readdir(dirp)) != NULL) {
@@ -308,8 +352,9 @@ cgroup_scan(const char *mnt, const char *path, cgroup_refresh_t refresh)
 	    continue;
 
 	cgname = cgroup_name(cgpath, length);
-	refresh(cgpath, cgname);
-	cgroup_scan(mnt, cgname, refresh);
+	if (check_refresh(cgpath + mntlen, container, container_length))
+	    refresh(cgpath, cgname);
+	cgroup_scan(mnt, cgname, refresh, container, container_length);
     }
     closedir(dirp);
 }
@@ -322,7 +367,8 @@ cgroup_scan(const char *mnt, const char *path, cgroup_refresh_t refresh)
  * its role is to refresh the values for that one named cgroup.
  */
 void
-refresh_cgroups(const char *subsys, cgroup_setup_t setup, cgroup_refresh_t refresh)
+refresh_cgroups(const char *subsys, const char *container,
+	int length, cgroup_setup_t setup, cgroup_refresh_t refresh)
 {
     int sts;
     filesys_t *fs;
@@ -335,42 +381,59 @@ refresh_cgroups(const char *subsys, cgroup_setup_t setup, cgroup_refresh_t refre
 	if (scan_filesys_options(fs->options, subsys) == NULL)
 	    continue;
 	setup();
-	cgroup_scan(fs->path, "", refresh);
+	cgroup_scan(fs->path, "", refresh, container, length);
     }
 }
 
+static int
+read_oneline(const char *file, char *buffer, size_t length)
+{
+    FILE *fp;
+    int sts;
+
+    if ((fp = fopen(file, "r")) == NULL)
+	return -ENOENT;
+    if (fgets(buffer, length, fp) != NULL) {
+	buffer[length-1] = '\0';
+	sts = 0;
+    } else {
+	sts = -ENOMEM;
+    }
+    fclose(fp);
+    return sts;
+}
+
 static int
 read_oneline_string(const char *file)
 {
-    char buffer[4096], *result;
+    char buffer[4096];
     size_t length;
-    FILE *fp;
+    int sts;
 
-    if ((fp = fopen(file, "r")) == NULL)
-	return -ENOENT;
-    result = fgets(buffer, sizeof(buffer), fp);
-    fclose(fp);
-    if (!result)
-	return -ENOMEM;
-    length = strlen(result);
-    while (result[--length] == '\n')
-	result[length] = '\0';
-    return proc_strings_insert(result);
+    if ((sts = read_oneline(file, buffer, sizeof(buffer))) < 0)
+	return sts;
+    length = strlen(buffer);
+    while (buffer[--length] == '\n')
+	buffer[length] = '\0';
+    return proc_strings_insert(buffer);
 }
 
-static __uint64_t
-read_oneline_ull(const char *file)
+static int
+read_oneline_ull(const char *file, __uint64_t *value)
 {
-    char buffer[4096], *result, *endp;
-    FILE *fp;
+    char buffer[4096], *endp;
+    int sts = read_oneline(file, buffer, sizeof(buffer));
+    *value = sts < 0 ? ULONGLONG_MAX : strtoull(buffer, &endp, 0);
+    return sts;
+}
 
-    if ((fp = fopen(file, "r")) == NULL)
-	return -ENOENT;
-    result = fgets(buffer, sizeof(buffer), fp);
-    fclose(fp);
-    if (!result)
-	return -ENOMEM;
-    return strtoull(result, &endp, 0);
+static int
+read_oneline_ll(const char *file, __int64_t *value)
+{
+    char buffer[4096], *endp;
+    int sts = read_oneline(file, buffer, sizeof(buffer));
+    *value = sts < 0 ? sts : strtoll(buffer, &endp, 0);
+    return sts;
 }
 
 void
@@ -505,7 +568,7 @@ refresh_cpuacct(const char *path, const char *name)
     snprintf(file, sizeof(file), "%s/cpuacct.stat", path);
     read_cpuacct_stats(file, cpuacct);
     snprintf(file, sizeof(file), "%s/cpuacct.usage", path);
-    cpuacct->usage = read_oneline_ull(file);
+    read_oneline_ull(file, &cpuacct->usage);
     snprintf(file, sizeof(file), "%s/cpuacct.usage_percpu", path);
     read_percpuacct_usage(file, name);
     pmdaCacheStore(indom, PMDA_CACHE_ADD, name, cpuacct);
@@ -517,6 +580,43 @@ setup_cpusched(void)
     pmdaCacheOp(INDOM(CGROUP_CPUSCHED_INDOM), PMDA_CACHE_INACTIVE);
 }
 
+static int
+read_cpu_stats(const char *file, cgroup_cpustat_t *ccp)
+{
+    static cgroup_cpustat_t cpustat;
+    static struct {
+	char		*field;
+	__uint64_t	*offset;
+    } cpustat_fields[] = {
+	{ "nr_periods",			&cpustat.nr_periods },
+	{ "nr_throttled",		&cpustat.nr_throttled },
+	{ "throttled_time",		&cpustat.throttled_time },
+    };
+    char buffer[4096], name[64];
+    unsigned long long value;
+    FILE *fp;
+    int i;
+
+    memset(&cpustat, 0, sizeof(cpustat));
+    if ((fp = fopen(file, "r")) == NULL) {
+	memcpy(ccp, &cpustat, sizeof(cpustat));
+	return -ENOENT;
+    }
+    while (fgets(buffer, sizeof(buffer), fp) != NULL) {
+	if (sscanf(buffer, "%s %llu\n", &name[0], &value) < 2)
+	    continue;
+	for (i = 0; cpustat_fields[i].field != NULL; i++) {
+	    if (strcmp(name, cpustat_fields[i].field) != 0)
+		continue;
+	    *cpustat_fields[i].offset = value;
+	    break;
+	}
+    }
+    fclose(fp);
+    memcpy(ccp, &cpustat, sizeof(cpustat));
+    return 0;
+}
+
 void
 refresh_cpusched(const char *path, const char *name)
 {
@@ -533,9 +633,15 @@ refresh_cpusched(const char *path, const char *name)
 	if (!cpusched)
 	    return;
     }
+    snprintf(file, sizeof(file), "%s/cpu.stat", path);
+    read_cpu_stats(file, &cpusched->stat);
     snprintf(file, sizeof(file), "%s/cpu.shares", path);
-    cpusched->shares = read_oneline_ull(file);
-    /* cpu.stat - nr_periods, nr_throttled, throttled_time */
+    read_oneline_ull(file, &cpusched->shares);
+    snprintf(file, sizeof(file), "%s/cpu.cfs_period_us", path);
+    read_oneline_ull(file, &cpusched->cfs_period);
+    snprintf(file, sizeof(file), "%s/cpu.cfs_quota_us", path);
+    read_oneline_ll(file, &cpusched->cfs_quota);
+
     pmdaCacheStore(indom, PMDA_CACHE_ADD, name, cpusched);
 }
 
@@ -632,6 +738,12 @@ refresh_memory(const char *path, const char *name)
     }
     snprintf(file, sizeof(file), "%s/memory.stat", path);
     read_memory_stats(file, memory);
+    snprintf(file, sizeof(file), "%s/memory.limit_in_bytes", path);
+    read_oneline_ull(file, &memory->limit);
+    snprintf(file, sizeof(file), "%s/memory.usage_in_bytes", path);
+    read_oneline_ull(file, &memory->usage);
+    snprintf(file, sizeof(file), "%s/memory.failcnt", path);
+    read_oneline_ull(file, &memory->failcnt);
 
     pmdaCacheStore(indom, PMDA_CACHE_ADD, name, memory);
 }
@@ -659,7 +771,7 @@ refresh_netcls(const char *path, const char *name)
 	    return;
     }
     snprintf(file, sizeof(file), "%s/net_cls.classid", path);
-    netcls->classid = read_oneline_ull(file);
+    read_oneline_ull(file, &netcls->classid);
     pmdaCacheStore(indom, PMDA_CACHE_ADD, name, netcls);
 }
 
@@ -687,6 +799,10 @@ get_blkiops(int style, cgroup_perdevblkio_t *perdev)
 	return &perdev->stats.io_service_time;
     case CG_BLKIO_IOWAITTIME_TOTAL:
 	return &perdev->stats.io_wait_time;
+    case CG_BLKIO_THROTTLEIOSERVICEBYTES_TOTAL:
+	return &perdev->stats.throttle_io_service_bytes;
+    case CG_BLKIO_THROTTLEIOSERVICED_TOTAL:
+	return &perdev->stats.throttle_io_serviced;
     }
     return NULL;
 }
@@ -741,7 +857,6 @@ read_blkio_devices_stats(const char *file, const char *name, int style,
     pmInDom devtindom = INDOM(DEVT_INDOM);
     cgroup_perdevblkio_t *blkdev;
     cgroup_blkiops_t *blkios;
-    __uint64_t result = 0;
     char *devname = NULL;
     char buffer[4096];
     FILE *fp;
@@ -773,7 +888,6 @@ read_blkio_devices_stats(const char *file, const char *name, int style,
 
 	i = sscanf(buffer, "Total %llu\n", &value);
 	if (i == 2) {	/* final field - per-cgroup Total operations */
-	    result = value;
 	    break;
 	}
 
@@ -787,7 +901,7 @@ read_blkio_devices_stats(const char *file, const char *name, int style,
 	    memset(&blkiops, 0, sizeof(cgroup_blkiops_t));
 	devname = realname;
 	for (i = 0; blkio_fields[i].field != NULL; i++) {
-	    if (strcmp(name, blkio_fields[i].field) != 0)
+	    if (strcmp(op, blkio_fields[i].field) != 0)
 		continue;
 	    *blkio_fields[i].offset = value;
 	    if (strcmp("Total", blkio_fields[i].field) != 0)
@@ -891,6 +1005,12 @@ refresh_blkio(const char *path, const char *name)
     snprintf(file, sizeof(file), "%s/blkio.time", path);
     read_blkio_devices_value(file, name,
 		CG_BLKIO_TIME, &blkio->total.time);
+    snprintf(file, sizeof(file), "%s/blkio.throttle.io_service_bytes", path);
+    read_blkio_devices_stats(file, name,
+		CG_BLKIO_THROTTLEIOSERVICEBYTES_TOTAL, &blkio->total.throttle_io_service_bytes);
+    snprintf(file, sizeof(file), "%s/blkio.throttle.io_serviced", path);
+    read_blkio_devices_stats(file, name,
+		CG_BLKIO_THROTTLEIOSERVICED_TOTAL, &blkio->total.throttle_io_serviced);
 
     pmdaCacheStore(indom, PMDA_CACHE_ADD, name, blkio);
 }
