@@ -1,4 +1,6 @@
 /*
+ * Copyright (c) 2015 Red Hat.
+ * Copyright (c) 2014-2015 Martins Innus.  All Rights Reserved.
  * Copyright (c) 2000,2004 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -15,9 +17,10 @@
 #include "impl.h"
 #include "pmda.h"
 #include "clusters.h"
-#include <ctype.h>
+#include "proc_pid.h"
+#include "indom.h"
 
-#define MAX_CLUSTERS 64
+#include <ctype.h>
 
 typedef struct {
     int	    item;
@@ -29,19 +32,39 @@ static __pmnsTree *dynamic_proc_tree;
 
 enum {
     DYNPROC_GROUP_PSINFO = 0,
-    DYNPROC_GROUP_FD = 1,
-    DYNPROC_GROUP_ID = 2,
-    DYNPROC_GROUP_MEMORY = 3,
-    DYNPROC_GROUP_IO = 4,
-    DYNPROC_GROUP_SCHEDSTAT = 5,
+    DYNPROC_GROUP_FD,
+    DYNPROC_GROUP_ID,
+    DYNPROC_GROUP_MEMORY,
+    DYNPROC_GROUP_IO,
+    DYNPROC_GROUP_SCHEDSTAT,
+    DYNPROC_GROUP_NAMESPACE,
+
     NUM_DYNPROC_GROUPS
 };
 
 enum {
     DYNPROC_PROC = 0,
+    DYNPROC_HOTPROC = 1,
+
     NUM_DYNPROC_TREES
 };
 
+/*
+ * Map proc cluster id's to new hotproc varients that don't conflict
+ */
+
+static int proc_hotproc_cluster_list[][2] = {
+	{ CLUSTER_PID_STAT,	    CLUSTER_HOTPROC_PID_STAT },
+	{ CLUSTER_PID_STATM,	    CLUSTER_HOTPROC_PID_STATM },
+	{ CLUSTER_PID_CGROUP,	    CLUSTER_HOTPROC_PID_CGROUP },
+	{ CLUSTER_PID_LABEL,	    CLUSTER_HOTPROC_PID_LABEL },
+	{ CLUSTER_PID_STATUS,	    CLUSTER_HOTPROC_PID_STATUS },
+	{ CLUSTER_PID_SCHEDSTAT,    CLUSTER_HOTPROC_PID_SCHEDSTAT },
+	{ CLUSTER_PID_IO,	    CLUSTER_HOTPROC_PID_IO },
+	{ CLUSTER_PID_FD,	    CLUSTER_HOTPROC_PID_FD }
+};
+
+
 typedef struct {
     char		*name;
     dynproc_metric_t	*metrics;
@@ -51,9 +74,13 @@ typedef struct {
 
 static const char *dynproc_members[] = {
 	[DYNPROC_PROC]	    = "proc",
-	//[DYNPROC_HOTPROC]   = "hotproc",
+	[DYNPROC_HOTPROC]   = "hotproc",
 };
 
+/*
+ * Remember to add help text to help_text.h for any new dynamic metrics
+ */
+
 static dynproc_metric_t psinfo_metrics[] = {
 	{ .name = "pid",	    .cluster = CLUSTER_PID_STAT,	.item=0 },
 	{ .name = "cmd",	    .cluster = CLUSTER_PID_STAT,	.item=1 },
@@ -101,6 +128,7 @@ static dynproc_metric_t psinfo_metrics[] = {
 	{ .name = "delayacct_blkio_time",   .cluster = CLUSTER_PID_STAT,.item=44 },
 	{ .name = "guest_time",	    .cluster = CLUSTER_PID_STAT,	.item=45 },
 	{ .name = "cguest_time",    .cluster = CLUSTER_PID_STAT,	.item=46 },
+	{ .name = "environ",        .cluster = CLUSTER_PID_STAT,	.item=47 },
 	{ .name = "signal_s",	    .cluster = CLUSTER_PID_STATUS,	.item=16 },
 	{ .name = "blocked_s",	    .cluster = CLUSTER_PID_STATUS,	.item=17 },
 	{ .name = "sigignore_s",    .cluster = CLUSTER_PID_STATUS,	.item=18 },
@@ -110,6 +138,9 @@ static dynproc_metric_t psinfo_metrics[] = {
 	{ .name = "labels",	    .cluster = CLUSTER_PID_LABEL,	.item=0 },
 	{ .name = "vctxsw",	    .cluster = CLUSTER_PID_STATUS,	.item=29 },
 	{ .name = "nvctxsw",	    .cluster = CLUSTER_PID_STATUS,	.item=30 },
+	{ .name = "cpusallowed",    .cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_CPUSALLOWED },
+	{ .name = "ngid",	    .cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NGID },
+        { .name = "tgid",	    .cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_TGID },
 };
 
 static dynproc_metric_t id_metrics[] = {
@@ -148,6 +179,18 @@ static dynproc_metric_t memory_metrics[] = {
         { .name = "vmexe",  .cluster = CLUSTER_PID_STATUS,  .item=25 },
         { .name = "vmlib",  .cluster = CLUSTER_PID_STATUS,  .item=26 },
         { .name = "vmswap", .cluster = CLUSTER_PID_STATUS,  .item=27 },
+        { .name = "vmpeak",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMPEAK },
+	{ .name = "vmpin",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMPIN },
+	{ .name = "vmhwm",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMHWM },
+	{ .name = "vmpte",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_VMPTE },
+};
+
+static dynproc_metric_t namespace_metrics[] = {
+        { .name = "tgid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NSTGID },
+        { .name = "pid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NSPID },
+        { .name = "pgid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NSPGID },
+        { .name = "sid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_NSSID },
+        { .name = "envid",	.cluster = CLUSTER_PID_STATUS,	.item = PROC_PID_STATUS_ENVID },
 };
 
 static dynproc_metric_t io_metrics[] = {
@@ -177,31 +220,51 @@ static dynproc_group_t dynproc_groups[] = {
 	[DYNPROC_GROUP_IO]	  = { .name = "io",	    .metrics = io_metrics,	    .nmetrics = sizeof(io_metrics)/sizeof(dynproc_metric_t)},
 	[DYNPROC_GROUP_FD]	  = { .name = "fd",	    .metrics = fd_metrics,	    .nmetrics = sizeof(fd_metrics)/sizeof(dynproc_metric_t)},
 	[DYNPROC_GROUP_SCHEDSTAT] = { .name = "schedstat",  .metrics = schedstat_metrics,   .nmetrics = sizeof(schedstat_metrics)/sizeof(dynproc_metric_t) },
+	[DYNPROC_GROUP_NAMESPACE] = { .name = "namespaces", .metrics = namespace_metrics,   .nmetrics = sizeof(namespace_metrics)/sizeof(dynproc_metric_t) },
 };
 
+/*
+ * Get the hotproc cluster that corresponds to this proc cluster
+ */
+
+int
+get_hot_cluster(int proc_cluster)
+{
+    int i;
+    int num_mapings = sizeof(proc_hotproc_cluster_list)/(sizeof(int)*2);
+
+    for (i = 0; i < num_mapings; i++) {
+	if (proc_hotproc_cluster_list[i][0] == proc_cluster)
+	    return proc_hotproc_cluster_list[i][1];
+    }
+    return -1;
+}
+
 /*
  * Given a cluster/item return the name
  */
-
 int
-get_name( int cluster, int item, char *name ){
-
+get_name(int cluster, int item, char *name)
+{
     unsigned int tree, group, metric;
-
     int num_dynproc_trees = sizeof(dynproc_members)/sizeof(char*);
     int num_dynproc_groups = sizeof(dynproc_groups)/sizeof(dynproc_group_t);
 
-    for( tree = 0; tree < num_dynproc_trees; tree++){
-        for( group = 0; group < num_dynproc_groups; group++ ){
-
+    for (tree = 0; tree < num_dynproc_trees; tree++) {
+        for (group = 0; group < num_dynproc_groups; group++) {
             dynproc_metric_t * cur_metrics = dynproc_groups[group].metrics;
             int num_cur_metrics = dynproc_groups[group].nmetrics;
 
-            for( metric = 0; metric < num_cur_metrics; metric++){
+            for (metric = 0; metric < num_cur_metrics; metric++) {
 		int _cluster =  cur_metrics[metric].cluster;
                 int _item =  cur_metrics[metric].item;
-		if( _cluster == cluster && _item == item ){
-		    sprintf( name, "%s.%s", dynproc_groups[group].name, cur_metrics[metric].name); 
+
+		if (tree == DYNPROC_HOTPROC)
+		    _cluster = get_hot_cluster(_cluster);
+
+		if (_cluster == cluster && _item == item) {
+		    sprintf(name, "%s.%s",
+			dynproc_groups[group].name, cur_metrics[metric].name); 
 		    return 1;
 		}
 	    }
@@ -211,117 +274,47 @@ get_name( int cluster, int item, char *name ){
 }
 
 /*
- * Utility function to give a set of clusters used by a dynproc_metric_t array
+ * Loop through the structs and build the namespace
  */
 
-
-int
-get_clusters_used( dynproc_group_t dyngroup, int *clusters ){
-
-    int numclusters = 0;
-    int j, i;
-
-    for(i=0; i< dyngroup.nmetrics; i++){
-	int curcluster = dyngroup.metrics[i].cluster;
-	int skip = 0;
-	for(j=0; j<numclusters;j++){
-	    if( clusters[j] == curcluster ){
-		skip = 1;
-		break; //already collected this one
-	    }
-	}  
-	if( !skip ){
-	    clusters[numclusters] = curcluster;
-	    numclusters++;
-	    if( numclusters == MAX_CLUSTERS ){
-		// Not sure what to do here. MAX_CLUSTERS is much bigger than any # in use right now
-		fprintf(stderr, "Increase MAX_CLUSTERS in proc_dynamic.  Data is missing\n");
-		return numclusters;
-	    }
-	} 
-
-    }
-    return numclusters;
-}
-
-/*
- * Map proc cluster id's to new hotproc varients that don't conflict
- */
-/*
-static int
-get_hotproc_cluster( int cluster )
-{
-    switch(cluster){
-        case CLUSTER_PID_STAT:
-                return CLUSTER_HOTPROC_PID_STAT;
-        case CLUSTER_PID_STATM:
-                return CLUSTER_HOTPROC_PID_STATM;
-        case CLUSTER_PID_CGROUP:
-                return CLUSTER_HOTPROC_PID_CGROUP;
-        case CLUSTER_PID_LABEL:
-                return CLUSTER_HOTPROC_PID_LABEL;
-        case CLUSTER_PID_STATUS:
-                return CLUSTER_HOTPROC_PID_STATUS;
-        case CLUSTER_PID_SCHEDSTAT:
-                return CLUSTER_HOTPROC_PID_SCHEDSTAT;
-        case CLUSTER_PID_IO:
-                return CLUSTER_HOTPROC_PID_IO;
-        case CLUSTER_PID_FD:
-                return CLUSTER_HOTPROC_PID_FD;
-        default:
-                return -1;
-        }
-}
-*/
-
 static void
-build_dynamic_proc_tree( int domain )
+build_dynamic_proc_tree(int domain)
 {
-
     char entry[128];
     pmID pmid;
-    //int hotcluster = -1;
-
     unsigned int num_hash_entries=0;
-
     unsigned int tree, group, metric;
-
     int num_dynproc_trees = sizeof(dynproc_members)/sizeof(char*);
     int num_dynproc_groups = sizeof(dynproc_groups)/sizeof(dynproc_group_t);
 
-
-    for( tree = 0; tree < num_dynproc_trees; tree++){
-	for( group = 0; group < num_dynproc_groups; group++ ){
+    for (tree = 0; tree < num_dynproc_trees; tree++) {
+	for (group = 0; group < num_dynproc_groups; group++) {
 
 	    dynproc_metric_t * cur_metrics = dynproc_groups[group].metrics;
 	    int num_cur_metrics = dynproc_groups[group].nmetrics;
 
-	    for( metric = 0; metric < num_cur_metrics; metric++){
+	    for (metric = 0; metric < num_cur_metrics; metric++) {
 
-		snprintf(entry, sizeof(entry), "%s.%s.%s", dynproc_members[tree], dynproc_groups[group].name, cur_metrics[metric].name);
+		snprintf(entry, sizeof(entry), "%s.%s.%s",
+			dynproc_members[tree], dynproc_groups[group].name,
+			cur_metrics[metric].name);
 
 		int cluster =  cur_metrics[metric].cluster;
 		int item =  cur_metrics[metric].item;
 
+		if (tree == DYNPROC_HOTPROC)
+		    cluster = get_hot_cluster(cluster);
+
 		pmid = pmid_build(domain, cluster, item);
 		__pmAddPMNSNode(dynamic_proc_tree, pmid, entry);
 
 		num_hash_entries++;
-
-		/* This should always be true */
-		//if( (hotcluster = get_hotproc_cluster( cluster )) != -1  ){
-		    //pmid = pmid_build(domain, hotcluster, item);
-		    //__pmAddPMNSNode(dynamic_proc_tree, pmid, entry);
-		 //   }
-		//else{
-		 //   fprintf(stderr, "Got non hotproc member while building tree: %d %d %d\n", domain, cluster, item);
-		//}
 	    }
 	}
     }
 
-    pmdaTreeRebuildHash( dynamic_proc_tree, num_hash_entries );
-
+    /* Must now call this in all cases when we update the tree */
+    pmdaTreeRebuildHash( dynamic_proc_tree, num_hash_entries);
 }
 
 /*
@@ -330,49 +323,53 @@ build_dynamic_proc_tree( int domain )
  * In this case we assume the only 2 metric groups are proc and hotproc
  *
  * I assume id=0 is proc and id=1 is hotproc.
+ * Not even called for 0 here
  *
- * This should be pretty simple.  Only metrics that are supposed to
- * be dynamic should flow thorugh here (correct?) so we don't do any checking
+ * Only metrics that are supposed to
+ * be dynamic should flow through here
  *
  */
 static void
 refresh_metrictable(pmdaMetric *source, pmdaMetric *dest, int id)
 {
-
-    //int domain = pmid_domain(source->m_desc.pmid);
-    //int cluster = pmid_cluster(source->m_desc.pmid);
-    //int hotcluster = -1;
+    int domain = pmid_domain(source->m_desc.pmid);
+    int cluster = pmid_cluster(source->m_desc.pmid);
+    int item = pmid_item(source->m_desc.pmid);
+    int hotcluster = -1;
 
     memcpy(dest, source, sizeof(pmdaMetric));
 
-
-    /*
-
-    if( id == 1 ){
-	hotcluster = get_hotproc_cluster(cluster);
-	if( hotcluster != -1 ){
-	    dest->m_desc.pmid = pmid_build(domain, hotcluster, id);
+    if (id == 1) {
+	hotcluster = get_hot_cluster(cluster);
+	if (hotcluster != -1) {
+	    dest->m_desc.pmid = pmid_build(domain, hotcluster, item);
+	    if (source->m_desc.indom == PM_INDOM_NULL) {
+		dest->m_desc.indom = PM_INDOM_NULL;
+	    }
+	    else {
+		/* Indom is not set for us with dynamic metrics */
+		dest->m_desc.indom = pmInDom_build(domain, HOTPROC_INDOM);
+	    }
 	}
-	else{
-	    fprintf(stderr, "Got bad hotproc cluster for %d %d %d\n", domain, cluster, id);
+	else {
+	    fprintf(stderr, "Got bad hotproc cluster for %d:%d:%d id=%d\n",
+		domain, cluster, item, id);
 	}
     }
-
-    */
-
+    else {
+	fprintf(stderr, "DYNAMIC PROC : refresh_metrictable called for %d:%d:%d id=%d\n", domain, cluster, item, id);
+	fprintf(stderr, "Did you try to add another dynamic proc tree? Implementation incomplete.\n");
+    }
 }
 
-
-/* Add the pmns entries based on appropriate cluster and namespace information 
+/* Add the pmns entries based on appropriate cluster and namespace information
  *
  * A little more complicated. Need to build up the full tree from all the structs at the top.
  *
  */
-
 static int
 refresh_dynamic_proc(pmdaExt *pmda, __pmnsTree **tree)
 {
-
     int sts, dom = pmda->e_domain;
 
     if (dynamic_proc_tree) {
@@ -385,37 +382,43 @@ refresh_dynamic_proc(pmdaExt *pmda, __pmnsTree **tree)
 	/* Call something that constructs the PMNS by multiple calls to __pmAddPMNSNode */
 	build_dynamic_proc_tree(dom);
 	*tree = dynamic_proc_tree;
+	/* Only return 1 if we are building the pmns */
         return 1;
     }
     return 0;
-
 }
 
 /* 
- * I think this is the total number of entries, not just the additional
- * after the first set.
+ * This should only return the number of additional entries, 
+ * do not inclue the "template" entries in pmda.c
+ *
+ * Otherwise you get "orphaned" entries that waste space
+ * 
  */
 
 static void
 size_metrictable(int *total, int *trees)
 {
-
     int i, num_leaf_nodes = 0;
-
     int num_dyn_groups =  sizeof(dynproc_groups)/sizeof(dynproc_group_t);
 
-    for( i = 0; i < num_dyn_groups; i++){
+    for (i = 0; i < num_dyn_groups; i++) {
 	num_leaf_nodes += dynproc_groups[i].nmetrics;
     }
 
-    *total = num_leaf_nodes; /* calc based on all the structs above.  Total number of leaf nodes right ??? */
-    *trees = sizeof(dynproc_members)/sizeof(char*); /* will be 2 for proc and hotproc */
+    *total = num_leaf_nodes; /* calc based on all the structs above. */
+    *trees = sizeof(dynproc_members)/sizeof(char*) -1; /* will be 1 (hotproc) */
 
     if (pmDebug & DBG_TRACE_LIBPMDA)
         fprintf(stderr, "interrupts size_metrictable: %d total x %d trees\n",
                 *total, *trees);
 }
 
+/*
+ * Header file of help text, generated from the original static help file by a perl script 
+ * Shared by both proc and hotproc
+ */
+
 #include "help_text.h"
 
 static int
@@ -423,21 +426,18 @@ dynamic_proc_text(pmdaExt *pmda, pmID pmid, int type, char **buf)
 {
     int item = pmid_item(pmid);
     int cluster = pmid_cluster(pmid);
-
     char name[128];
 
-    if( get_name( cluster, item, name )){
-    
-        int i=0;
+    if (get_name(cluster, item, name)) {
         int num_help_entries = sizeof(help_text)/sizeof(help_text_t);
-        for(i=0; i < num_help_entries; i++){
-	    if( strcmp(name, help_text[i].name) == 0 ){
-		if( (type & PM_TEXT_ONELINE) || help_text[i].longhelp[0] =='\0' ){
+        int i;
+
+        for (i = 0; i < num_help_entries; i++) {
+	    if (strcmp(name, help_text[i].name) == 0) {
+		if ((type & PM_TEXT_ONELINE) || help_text[i].longhelp[0] =='\0')
 		    *buf = help_text[i].shorthelp;
-		}
-		else{
+		else
 		    *buf = help_text[i].longhelp;
-		}
 		return 0;
 	    }
 	}
@@ -446,43 +446,18 @@ dynamic_proc_text(pmdaExt *pmda, pmID pmid, int type, char **buf)
     return 0;
 }
 
-
 /*
- * Build up the dynamic infrastructure. Call pmdaDynamicPMNS for each unique [hot]proc.foo
- * grouping.
- *
- * But share everything else as much as possible.
- *
+ * Build up the dynamic infrastructure.
  */
-
 void
 proc_dynamic_init(pmdaMetric *metrics, int nmetrics)
 {
+    int clusters[1]; /* Not needed, but kept for interface compatibility */
+    int nclusters = 0;
 
-    int clusters[MAX_CLUSTERS];
-    int nclusters;
-
-    char treename[128];
-
-    int i,j;
-    int num_dyngroups = sizeof(dynproc_groups)/sizeof(dynproc_group_t);
-    int num_dyntrees = sizeof(dynproc_members)/sizeof(char*);
-
-    for(i=0; i< num_dyngroups; i++){
-
-	nclusters = get_clusters_used( dynproc_groups[i], clusters );
-
-	for(j=0; j< num_dyntrees; j++){
-
-	    sprintf(treename, "%s.%s", dynproc_members[j], dynproc_groups[i].name);
-
-	    // Should the strdup/memcpy be inside pmdaDynamicPMNS? currently it just assignes the pointer, assuming a string literal or other static type.
-	    pmdaDynamicPMNS(strdup(treename),
+    pmdaDynamicPMNS("NOTNEEDED",
                     clusters, nclusters,
                     refresh_dynamic_proc, dynamic_proc_text,
                     refresh_metrictable, size_metrictable,
                     metrics, nmetrics);
-	}
-    }
 }
-
