@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2014 Red Hat.
+ * Copyright (c) 2012-2017 Red Hat.
  * Copyright (c) 2009-2010 Aconex. All Rights Reserved.
  * Copyright (c) 1995-2000,2009 Silicon Graphics, Inc. All Rights Reserved.
  *
@@ -16,7 +16,7 @@
  *
  * MMV PMDA
  *
- * This PMDA uses specially formatted files in either /var/tmp/mmv or some
+ * This PMDA uses specially formatted files from $PCP_TMP_DIR/mmv or some
  * other directory, as specified on the command line.  Each file represents
  * a separate "cluster" of values with flat name structure for each cluster.
  * Names for the metrics are optionally prepended with mmv and then the name
@@ -30,6 +30,7 @@
 #include "pmda.h"
 #include "./domain.h"
 #include <sys/stat.h>
+#include <inttypes.h>
 #include <ctype.h>
 
 static int isDSO = 1;
@@ -51,9 +52,9 @@ static pmdaOptions opts = {
 };
 
 static pmdaMetric * metrics;
-static int mcnt;
+static int mtot;
 static pmdaIndom * indoms;
-static int incnt;
+static int intot;
 
 static int reload;
 static __pmnsTree * pmns;
@@ -71,11 +72,14 @@ typedef struct {
     char *	name;			/* strdup client name */
     void *	addr;			/* mmap */
     mmv_disk_value_t * values;		/* values in mmap */
-    mmv_disk_metric_t *	metrics;	/* metric descs in mmap */
+    mmv_disk_metric_t * metrics1;	/* v1 metric descs in mmap */
+    mmv_disk_metric2_t * metrics2;	/* v2 metric descs in mmap */
     int		vcnt;			/* number of values */
-    int		mcnt;			/* number of metrics */
-    pid_t	pid;			/* process identifier */
+    int		mcnt1;			/* number of metrics */
+    int		mcnt2;			/* number of v2 metrics */
+    int		version;		/* v1/v2 version number */
     int		cluster;		/* cluster identifier */
+    pid_t	pid;			/* process identifier */
     __int64_t	len;			/* mmap region len */
     __uint64_t	gen;			/* generation number on open */
 } stats_t;
@@ -83,6 +87,18 @@ typedef struct {
 static stats_t * slist;
 static int scnt;
 
+#define MAX_MMV_COUNT 10000		/* enforce reasonable limits */
+#define MAX_MMV_CLUSTER ((1<<12)-1)
+
+/*
+ * Check cluster number validity (must be in range 0 .. 1<<12).
+ */
+static int
+valid_cluster(int requested)
+{
+    return (requested >= 0 && requested <= MAX_MMV_CLUSTER);
+}
+
 /*
  * Choose an unused cluster ID while honouring specific requests.
  * If a specific (non-zero) cluster is requested we always use it.
@@ -101,9 +117,15 @@ choose_cluster(int requested, const char *path)
 		i = 0;	/* restart, we're filling holes */
 	    }
 	}
+	if (!valid_cluster(next_cluster))
+	    return -EAGAIN;
+
 	return next_cluster;
     }
 
+    if (!valid_cluster(requested))
+	return -EINVAL;
+
     for (i = 0; i < scnt; i++) {
 	if (slist[i].cluster == requested) {
 	    if (pmDebug & DBG_TRACE_APPL0)
@@ -124,54 +146,78 @@ create_client_stat(const char *client, const char *path, size_t size)
     if (pmDebug & DBG_TRACE_APPL0)
 	__pmNotifyErr(LOG_DEBUG, "MMV: create_client_stat: %s, %s", client, path);
 
+    /* sanity check */
+    if (size < sizeof(mmv_disk_header_t))
+	return -EINVAL;
+
     if ((fd = open(path, O_RDONLY)) >= 0) {
 	void *m = __pmMemoryMap(fd, size, 0);
 
 	close(fd);
 	if (m != NULL) {
-	    mmv_disk_header_t * hdr = (mmv_disk_header_t *)m;
+	    mmv_disk_header_t header = *(mmv_disk_header_t *)m;
+	    size_t offset;
 	    int cluster;
 
-	    if (strncmp(hdr->magic, "MMV", 4)) {
+	    if (strncmp(header.magic, "MMV", 4)) {
 		__pmMemoryUnmap(m, size);
 		return -EINVAL;
 	    }
 
-	    if (hdr->version != MMV_VERSION) {
-		__pmNotifyErr(LOG_ERR, "%s: %s client version %d "
-				"not supported (current is %d)",
-				pmProgname, prefix, hdr->version, MMV_VERSION);
+	    if (header.version != MMV_VERSION1 &&
+		header.version != MMV_VERSION2) {
+		if (pmDebug & DBG_TRACE_APPL0)
+		    __pmNotifyErr(LOG_ERR,
+			"%s: %s client version %d unsupported (current is %d)",
+			pmProgname, prefix, header.version, MMV_VERSION);
 		__pmMemoryUnmap(m, size);
 		return -ENOSYS;
 	    }
 
-	    if (!hdr->g1 || hdr->g1 != hdr->g2) {
-		/* still in flux, wait till next time */
+	    if (header.g1 == 0 || header.g1 != header.g2) {
+		/* still in flux, wait until next refresh */
 		__pmMemoryUnmap(m, size);
 		return -EAGAIN;
 	    }
 
+	    /* must have entries for at least metric descs and values */
+	    if (header.tocs < 2) {
+		__pmMemoryUnmap(m, size);
+		return -EINVAL;
+	    }
+	    offset = header.tocs * sizeof(mmv_disk_toc_t);
+	    if (size < sizeof(mmv_disk_header_t) + offset) {
+		__pmMemoryUnmap(m, size);
+		return -EINVAL;
+	    }
+
 	    /* optionally verify the creator PID is running */
-	    if (hdr->process && (hdr->flags & MMV_FLAG_PROCESS) &&
-		!__pmProcessExists((pid_t)hdr->process)) {
+	    if (header.process && (header.flags & MMV_FLAG_PROCESS) &&
+		!__pmProcessExists((pid_t)header.process)) {
 		__pmMemoryUnmap(m, size);
 		return -ESRCH;
 	    }
 
-	    /* all checks out, we'll use this one */
-	    cluster = choose_cluster(hdr->cluster, path);
-	    if (pmDebug & DBG_TRACE_APPL0)
-		__pmNotifyErr(LOG_DEBUG, "MMV: %s: loading %s client: %d \"%s\"",
-				    pmProgname, prefix, cluster, path);
+	    if ((cluster = choose_cluster(header.cluster, path)) < 0) {
+		__pmMemoryUnmap(m, size);
+		return cluster;
+	    }
 
-	    slist = realloc(slist, sizeof(stats_t)*(scnt+1));
+	    /* all checks out so far, we'll use this one */
+	    if (pmDebug & DBG_TRACE_APPL0)
+		__pmNotifyErr(LOG_DEBUG, "MMV: loading %s client: %d \"%s\"",
+				    prefix, cluster, path);
+
+	    slist = realloc(slist, sizeof(stats_t) * (scnt + 1));
 	    if (slist != NULL) {
+		memset(&slist[scnt], 0, sizeof(stats_t));
 		slist[scnt].name = strdup(client);
 		slist[scnt].addr = m;
-		slist[scnt].pid = (pid_t)((hdr->flags & MMV_FLAG_PROCESS)? hdr->process : 0);
+		if ((header.flags & MMV_FLAG_PROCESS) != 0)
+		    slist[scnt].pid = (pid_t)header.process;
+		slist[scnt].version = header.version;
 		slist[scnt].cluster = cluster;
-		slist[scnt].mcnt = 0;
-		slist[scnt].gen = hdr->g1;
+		slist[scnt].gen = header.g1;
 		slist[scnt].len = size;
 		scnt++;
 	    } else {
@@ -181,10 +227,10 @@ create_client_stat(const char *client, const char *path, size_t size)
 		scnt = 0;
 	    }
 	} else {
-            __pmNotifyErr(LOG_ERR, "%s: failed to memory map \"%s\" - %s",
+	    __pmNotifyErr(LOG_ERR, "%s: failed to memory map \"%s\" - %s",
 				pmProgname, path, osstrerror());
 	}
-    } else {
+    } else if (pmDebug & DBG_TRACE_APPL0) {
 	__pmNotifyErr(LOG_ERR, "%s: failed to open client file \"%s\" - %s",
 				pmProgname, client, osstrerror());
     }
@@ -234,41 +280,46 @@ verify_metric_item(unsigned int item, char *name, stats_t *s)
 }
 
 static int
-create_metric(pmdaExt *pmda, stats_t *s, mmv_disk_metric_t *m, char *name, pmID pmid)
+create_metric(pmdaExt *pmda, stats_t *s, char *name, pmID pmid, unsigned indom,
+	mmv_metric_type_t type, mmv_metric_sem_t semantics, pmUnits units)
 {
     if (pmDebug & DBG_TRACE_APPL0)
 	__pmNotifyErr(LOG_DEBUG, "MMV: create_metric: %s - %s", name, pmIDStr(pmid));
 
-    metrics = realloc(metrics, sizeof(pmdaMetric) * (mcnt + 1));
+    metrics = realloc(metrics, sizeof(pmdaMetric) * (mtot + 1));
     if (metrics == NULL)  {
 	__pmNotifyErr(LOG_ERR, "cannot grow MMV metric list: %s", s->name);
 	return -ENOMEM;
     }
 
-    metrics[mcnt].m_user = NULL;
-    metrics[mcnt].m_desc.pmid = pmid;
+    metrics[mtot].m_user = NULL;
+    metrics[mtot].m_desc.pmid = pmid;
 
-    if (m->type == MMV_TYPE_ELAPSED) {
+    if (type == MMV_TYPE_ELAPSED) {
 	pmUnits unit = PMDA_PMUNITS(0,1,0,0,PM_TIME_USEC,0);
-	metrics[mcnt].m_desc.sem = PM_SEM_COUNTER;
-	metrics[mcnt].m_desc.type = MMV_TYPE_I64;
-	metrics[mcnt].m_desc.units = unit;
+	metrics[mtot].m_desc.sem = PM_SEM_COUNTER;
+	metrics[mtot].m_desc.type = MMV_TYPE_I64;
+	metrics[mtot].m_desc.units = unit;
     } else {
-	if (m->semantics)
-	    metrics[mcnt].m_desc.sem = m->semantics;
+	if (semantics)
+	    metrics[mtot].m_desc.sem = semantics;
 	else
-	    metrics[mcnt].m_desc.sem = PM_SEM_COUNTER;
-	metrics[mcnt].m_desc.type = m->type;
-	memcpy(&metrics[mcnt].m_desc.units, &m->dimension, sizeof(pmUnits));
+	    metrics[mtot].m_desc.sem = PM_SEM_COUNTER;
+	metrics[mtot].m_desc.type = type;
+	memcpy(&metrics[mtot].m_desc.units, &units, sizeof(pmUnits));
     }
-    metrics[mcnt].m_desc.indom = (!m->indom || m->indom == PM_INDOM_NULL) ?
-				PM_INDOM_NULL : pmInDom_build(pmda->e_domain,
-					(s->cluster << 11) | m->indom);
-    if (pmDebug & DBG_TRACE_APPL0)
-	__pmNotifyErr(LOG_DEBUG, "MMV: map_stats adding metric[%d] %s %s from %s\n",
-			mcnt, name, pmIDStr(pmid), s->name);
+    if (!indom || indom == PM_INDOM_NULL)
+	metrics[mtot].m_desc.indom = PM_INDOM_NULL;
+    else
+	metrics[mtot].m_desc.indom = 
+		pmInDom_build(pmda->e_domain, (s->cluster << 11) | indom);
 
-    mcnt++;
+    if (pmDebug & DBG_TRACE_APPL0)
+	__pmNotifyErr(LOG_DEBUG,
+			"MMV: map_stats adding metric[%d] %s %s from %s\n",
+			mtot, name, pmIDStr(pmid), s->name);
+
+    mtot++;
     __pmAddPMNSNode(pmns, pmid, name);
 
     return 0;
@@ -290,7 +341,7 @@ verify_indom_serial(pmdaExt *pmda, int serial, stats_t *s, pmInDom *p, pmdaIndom
     }
 
     *p = pmInDom_build(pmda->e_domain, (s->cluster << 11) | serial);
-    for (index = 0; index < incnt; index++) {
+    for (index = 0; index < intot; index++) {
 	*i = &indoms[index];
 	if (indoms[index].it_indom == *p)
 	    return -EEXIST;
@@ -300,82 +351,126 @@ verify_indom_serial(pmdaExt *pmda, int serial, stats_t *s, pmInDom *p, pmdaIndom
 }
 
 static int
-update_indom(pmdaExt *pmda, stats_t *s, mmv_disk_indom_t *id, pmdaIndom *ip)
+update_indom(pmdaExt *pmda, stats_t *s, __uint64_t offset, __uint32_t count,
+		mmv_disk_indom_t *id, pmdaIndom *ip)
 {
+    mmv_disk_instance_t *in1 = NULL;
+    mmv_disk_instance2_t *in2 = NULL;
+    mmv_disk_string_t *string;
     int i, j, size, newinsts = 0;
-    mmv_disk_instance_t *in = (mmv_disk_instance_t *)((char *)s->addr + id->offset);
 
     if (pmDebug & DBG_TRACE_APPL0)
 	__pmNotifyErr(LOG_DEBUG, "MMV: update_indom: %u (%d insts)",
 			id->serial, ip->it_numinst);
 
-    /* first calculate how many new instances, so we know what to alloc */
-    for (i = 0; i < id->count; i++) {
-	for (j = 0; j < ip->it_numinst; j++)
-	    if (ip->it_set[j].i_inst == in[i].internal)
-		continue;
-	if (j == ip->it_numinst)
-	    newinsts++;
+    if (s->version == MMV_VERSION1) {
+	in1 = (mmv_disk_instance_t *)((char *)s->addr + offset);
+	for (i = 0; i < count; i++) {
+	    for (j = 0; j < ip->it_numinst; j++) {
+		if (ip->it_set[j].i_inst == in1[i].internal)
+		    continue;
+	    }
+	    if (j == ip->it_numinst)
+		newinsts++;
+	}
+    } else if (s->version == MMV_VERSION2) {
+	in2 = (mmv_disk_instance2_t *)((char *)s->addr + offset);
+	for (i = 0; i < count; i++) {
+	    for (j = 0; j < ip->it_numinst; j++) {
+		if (ip->it_set[j].i_inst == in2[i].internal)
+		    continue;
+	    }
+	    if (j == ip->it_numinst)
+		newinsts++;
+	}
     }
-
     if (!newinsts)
 	return 0;
 
     /* allocate memory, then append new instances to the known set */
     size = sizeof(pmdaInstid) * (ip->it_numinst + newinsts);
     ip->it_set = (pmdaInstid *)realloc(ip->it_set, size);
-    if (ip->it_set != NULL) {
-	for (i = 0; i < id->count; i++) {
-	    for (j = 0; j < ip->it_numinst; j++)
-		if (ip->it_set[j].i_inst == in[j].internal)
-		    continue;
-	    if (j == ip->it_numinst) {
-		ip->it_set[j].i_inst = in[i].internal;
-		ip->it_set[j].i_name = in[i].external;
-		ip->it_numinst++;
-	    }
-	}
-    } else {
+    if (ip->it_set == NULL) {
 	__pmNotifyErr(LOG_ERR, "%s: cannot get memory for instance list in %s",
 			pmProgname, s->name);
 	ip->it_numinst = 0;
 	return -ENOMEM;
     }
+
+    if (s->version == MMV_VERSION1) {
+	for (i = 0; i < count; i++) {
+	    for (j = 0; j < ip->it_numinst; j++)
+		if (ip->it_set[j].i_inst == in1[i].internal)
+		    continue;
+	    if (j == ip->it_numinst) {
+		ip->it_set[j].i_inst = in1[i].internal;
+		ip->it_set[j].i_name = in1[i].external;
+		ip->it_numinst++;
+	    }
+	}
+    } else if (s->version == MMV_VERSION2) {
+	for (i = 0; i < count; i++) {
+	    for (j = 0; j < ip->it_numinst; j++)
+		if (ip->it_set[j].i_inst == in2[i].internal)
+		    continue;
+	    if (j == ip->it_numinst) {
+		string = (mmv_disk_string_t *)
+				((char *)s->addr + in2[i].external);
+		ip->it_set[j].i_inst = in2[i].internal;
+		ip->it_set[j].i_name = string->payload;
+		ip->it_numinst++;
+	    }
+	}
+    }
     return 0;
 }
 
 static int
-create_indom(pmdaExt *pmda, stats_t *s, mmv_disk_indom_t *id, pmInDom indom)
+create_indom(pmdaExt *pmda, stats_t *s, __uint64_t offset, __uint32_t count,
+		mmv_disk_indom_t *id, pmInDom indom)
 {
     int i;
     pmdaIndom *ip;
+    mmv_disk_instance_t *in1;
+    mmv_disk_instance2_t *in2;
+    mmv_disk_string_t *string;
 
     if (pmDebug & DBG_TRACE_APPL0)
 	__pmNotifyErr(LOG_DEBUG, "MMV: create_indom: %u", id->serial);
 
-    indoms = realloc(indoms, sizeof(pmdaIndom) * (incnt + 1));
+    indoms = realloc(indoms, sizeof(pmdaIndom) * (intot + 1));
     if (indoms == NULL) {
 	__pmNotifyErr(LOG_ERR, "%s: cannot grow indom list in %s",
 			pmProgname, s->name);
 	return -ENOMEM;
     }
-    ip = &indoms[incnt++];
+    ip = &indoms[intot++];
     ip->it_indom = indom;
-    ip->it_set = (pmdaInstid *)calloc(id->count, sizeof(pmdaInstid));
-    if (ip->it_set != NULL) {
-	mmv_disk_instance_t * in = (mmv_disk_instance_t *)
-				    ((char *)s->addr + id->offset);
-	ip->it_numinst = id->count;
-	for (i = 0; i < ip->it_numinst; i++) {
-	    ip->it_set[i].i_inst = in[i].internal;
-	    ip->it_set[i].i_name = in[i].external;
-	}
-    } else {
+    ip->it_set = (pmdaInstid *)calloc(count, sizeof(pmdaInstid));
+    if (ip->it_set == NULL) {
 	__pmNotifyErr(LOG_ERR, "%s: cannot get memory for instance list in %s",
 			pmProgname, s->name);
 	ip->it_numinst = 0;
 	return -ENOMEM;
     }
+
+    if (s->version == MMV_VERSION1) {
+	in1 = (mmv_disk_instance_t *)((char *)s->addr + offset);
+	ip->it_numinst = count;
+	for (i = 0; i < count; i++) {
+	    ip->it_set[i].i_inst = in1[i].internal;
+	    ip->it_set[i].i_name = in1[i].external;
+	}
+    } else if (s->version == MMV_VERSION2) {
+	in2 = (mmv_disk_instance2_t *)((char *)s->addr + offset);
+	ip->it_numinst = count;
+	for (i = 0; i < count; i++) {
+	    string = (mmv_disk_string_t *)
+				((char *)s->addr + in2[i].external);
+	    ip->it_set[i].i_inst = in2[i].internal;
+	    ip->it_set[i].i_name = string->payload;
+	}
+    }
     return 0;
 }
 
@@ -397,19 +492,21 @@ map_stats(pmdaExt *pmda)
 	return;
     }
 
-    /* hard-coded metrics (not from mmap'd files */
-    snprintf(name, sizeof(name), "%s.reload", prefix);
+    /* hard-coded metrics (not from mmap'd files) */
+    snprintf(name, sizeof(name), "%s.control.reload", prefix);
     __pmAddPMNSNode(pmns, pmid_build(pmda->e_domain, 0, 0), name);
-    snprintf(name, sizeof(name), "%s.debug", prefix);
+    snprintf(name, sizeof(name), "%s.control.debug", prefix);
     __pmAddPMNSNode(pmns, pmid_build(pmda->e_domain, 0, 1), name);
-    mcnt = 2;
+    snprintf(name, sizeof(name), "%s.control.files", prefix);
+    __pmAddPMNSNode(pmns, pmid_build(pmda->e_domain, 0, 2), name);
+    mtot = 3;
 
     if (indoms != NULL) {
-	for (i = 0; i < incnt; i++)
+	for (i = 0; i < intot; i++)
 	    free(indoms[i].it_set);
 	free(indoms);
 	indoms = NULL;
-	incnt = 0;
+	intot = 0;
     }
 
     if (slist != NULL) {
@@ -445,21 +542,46 @@ map_stats(pmdaExt *pmda)
 	free(files);
 
     for (i = 0; slist && i < scnt; i++) {
-	stats_t * s = slist + i;
-	mmv_disk_header_t * hdr = (mmv_disk_header_t *)s->addr;
-	mmv_disk_toc_t * toc = (mmv_disk_toc_t *)
+	stats_t *s = slist + i;
+	mmv_disk_indom_t *id;
+	mmv_disk_header_t *hdr = (mmv_disk_header_t *)s->addr;
+	mmv_disk_toc_t *toc = (mmv_disk_toc_t *)
 			((char *)s->addr + sizeof(mmv_disk_header_t));
 
 	for (j = 0; j < hdr->tocs; j++) {
-	    switch (toc[j].type) {
-		case MMV_TOC_METRICS: {
+	    __uint64_t offset = toc[j].offset;
+	    __uint32_t count = toc[j].count;
+	    __uint32_t type = toc[j].type;
+
+	    switch (type) {
+	    case MMV_TOC_METRICS:
+		if (count > MAX_MMV_COUNT) {
+		    if (pmDebug & DBG_TRACE_APPL0) {
+			__pmNotifyErr(LOG_ERR, "MMV: %s - "
+					"metrics count: %d > %d",
+					s->name, count, MAX_MMV_COUNT);
+		    }
+		    continue;
+		}
+		if (s->version == MMV_VERSION1) {
 		    mmv_disk_metric_t *ml = (mmv_disk_metric_t *)
-					((char *)s->addr + toc[j].offset);
+					((char *)s->addr + offset);
 
-		    s->metrics = ml;
-		    s->mcnt = toc[j].count;
+		    offset += (count * sizeof(mmv_disk_metric_t));
+		    if (s->len < offset) {
+			if (pmDebug & DBG_TRACE_APPL0) {
+			    __pmNotifyErr(LOG_INFO, "MMV: %s - "
+					"metrics offset: %"PRIu64" < %"PRIu64,
+					s->name, s->len, (int64_t)offset);
+			}
+			continue;
+		    }
 
-		    for (k = 0; k < toc[j].count; k++) {
+		    s->metrics1 = ml;
+		    s->mcnt1 = count;
+
+		    for (k = 0; k < count; k++) {
+			mmv_disk_metric_t *mp = &ml[k];
 			char name[MAXPATHLEN];
 			pmID pmid;
 
@@ -468,141 +590,351 @@ map_stats(pmdaExt *pmda)
 			    sprintf(name, "%s.", prefix);
 			else
 			    sprintf(name, "%s.%s.", prefix, s->name);
-			strcat(name, ml[k].name);
+			strcat(name, mp->name);
 			if (verify_metric_name(name, k, s) != 0)
 			    continue;
-			if (verify_metric_item(ml[k].item, name, s) != 0)
+			if (verify_metric_item(mp->item, name, s) != 0)
 			    continue;
 
-			pmid = pmid_build(pmda->e_domain, s->cluster, ml[k].item);
-			create_metric(pmda, s, &ml[k], name, pmid);
+			pmid = pmid_build(pmda->e_domain, s->cluster, mp->item);
+			create_metric(pmda, s, name, pmid, mp->indom,
+					mp->type, mp->semantics, mp->dimension);
 		    }
-		    break;
 		}
+		else if (s->version == MMV_VERSION2) {
+		    mmv_disk_metric2_t *ml = (mmv_disk_metric2_t *)
+					((char *)s->addr + offset);
 
-		case MMV_TOC_INDOMS: {
-		    mmv_disk_indom_t * id = (mmv_disk_indom_t *)
-					((char *)s->addr + toc[j].offset);
+		    offset += (count * sizeof(mmv_disk_metric2_t));
+		    if (s->len < offset) {
+			if (pmDebug & DBG_TRACE_APPL0) {
+			    __pmNotifyErr(LOG_INFO, "MMV: %s - "
+					"metrics offset: %"PRIu64" < %"PRIu64,
+					s->name, s->len, (int64_t)offset);
+			}
+			continue;
+		    }
 
-		    for (k = 0; k < toc[j].count; k++) {
-			int sts, serial = id[k].serial;
-			pmInDom pmindom;
-			pmdaIndom *ip;
+		    s->metrics2 = ml;
+		    s->mcnt2 = count;
 
-			sts = verify_indom_serial(pmda, serial, s, &pmindom, &ip);
-			if (sts == -EINVAL)
+		    for (k = 0; k < count; k++) {
+			mmv_disk_metric2_t *mp = &ml[k];
+			mmv_disk_string_t *string;
+			char buf[MMV_STRINGMAX];
+			char name[MAXPATHLEN];
+			__uint64_t mname;
+			pmID pmid;
+
+			mname = mp->name;
+			if (s->len < mname + sizeof(mmv_disk_string_t)) {
+			    if (pmDebug & DBG_TRACE_APPL0) {
+				__pmNotifyErr(LOG_INFO, "MMV: %s - "
+					"metrics2 name: %"PRIu64" < %"PRIu64,
+					s->name, s->len, mname);
+			    }
 			    continue;
-			else if (sts == -EEXIST)
-			    /* see if we have new instances to add here */
-			    update_indom(pmda, s, &id[k], ip);
+			}
+			string = (mmv_disk_string_t *)((char *)s->addr + mname);
+			memcpy(buf, string->payload, sizeof(buf));
+			buf[sizeof(buf)-1] = '\0';
+
+			/* build name, check its legitimate and unique */
+			if (hdr->flags & MMV_FLAG_NOPREFIX)
+			    sprintf(name, "%s.", prefix);
 			else
-			    /* first time we've observed this indom */
-			    create_indom(pmda, s, &id[k], pmindom);
+			    sprintf(name, "%s.%s.", prefix, s->name);
+			strcat(name, buf);
+
+			if (verify_metric_name(name, k, s) != 0)
+			    continue;
+			if (verify_metric_item(mp->item, name, s) != 0)
+			    continue;
+
+			pmid = pmid_build(pmda->e_domain, s->cluster, mp->item);
+			create_metric(pmda, s, name, pmid, mp->indom,
+					mp->type, mp->semantics, mp->dimension);
 		    }
-		    break;
+		}
+		break;
+
+	    case MMV_TOC_INDOMS:
+		if (count > MAX_MMV_COUNT) {
+		    if (pmDebug & DBG_TRACE_APPL0) {
+			__pmNotifyErr(LOG_ERR, "MMV: %s - "
+					"indoms count: %d > %d",
+					s->name, count, MAX_MMV_COUNT);
+		    }
+		    continue;
+		}
+		id = (mmv_disk_indom_t *)((char *)s->addr + offset);
+
+		offset += (count * sizeof(mmv_disk_indom_t));
+		if (s->len < offset) {
+		    if (pmDebug & DBG_TRACE_APPL0) {
+			__pmNotifyErr(LOG_ERR, "MMV: %s - "
+					"indoms offset: %"PRIu64" < %"PRIu64,
+					s->name, s->len, offset);
+		    }
+		    continue;
 		}
 
-		case MMV_TOC_VALUES: {
-		    s->vcnt = toc[j].count;
-		    s->values = (mmv_disk_value_t *)
-			((char *)s->addr + toc[j].offset);
-		    break;
-		}
+		for (k = 0; k < count; k++) {
+		    int sts, serial = id[k].serial;
+		    pmInDom pmindom;
+		    pmdaIndom *ip;
 
-		default:
-		    break;
+		    offset = id[k].offset;
+		    count = id[k].count;
+
+		    if (count > MAX_MMV_COUNT) {
+			if (pmDebug & DBG_TRACE_APPL0) {
+			    __pmNotifyErr(LOG_ERR, "MMV: %s - "
+					"indom[%d] count: %d > %d",
+					s->name, k, count, MAX_MMV_COUNT);
+			}
+			continue;
+		    }
+
+		    if (s->version == MMV_VERSION1) {
+			offset += (count * sizeof(mmv_disk_instance_t));
+			if (s->len < offset) {
+			    if (pmDebug & DBG_TRACE_APPL0) {
+				__pmNotifyErr(LOG_ERR, "MMV: %s - "
+					"indom[%d] offset: %"PRIu64" < %"PRIu64,
+					s->name, k, s->len, offset);
+			    }
+			    continue;
+			}
+			offset -= (count * sizeof(mmv_disk_instance_t));
+		    } else {
+			offset += (count * sizeof(mmv_disk_instance2_t));
+			if (s->len < offset) {
+			    if (pmDebug & DBG_TRACE_APPL0) {
+				__pmNotifyErr(LOG_ERR, "MMV: %s - "
+					"indom[%d] offset: %"PRIu64" < %"PRIu64,
+					s->name, k, s->len, offset);
+			    }
+			    continue;
+			}
+		        offset -= (count * sizeof(mmv_disk_instance2_t));
+		    }
+		    sts = verify_indom_serial(pmda, serial, s, &pmindom, &ip);
+		    if (sts == -EINVAL)
+			continue;
+		    else if (sts == -EEXIST)
+			/* see if we have new instances to add here */
+			update_indom(pmda, s, offset, count, &id[k], ip);
+		    else
+			/* first time we've observed this indom */
+			create_indom(pmda, s, offset, count, &id[k], pmindom);
+		}
+		break;
+
+	    case MMV_TOC_VALUES:
+		if (count > MAX_MMV_COUNT) {
+		    if (pmDebug & DBG_TRACE_APPL0) {
+			__pmNotifyErr(LOG_ERR, "MMV: %s - "
+					"values count: %d > %d",
+					s->name, count, MAX_MMV_COUNT);
+		    }
+		    continue;
+		}
+		offset += (count * sizeof(mmv_disk_value_t));
+		if (s->len < offset) {
+		    if (pmDebug & DBG_TRACE_APPL0) {
+			__pmNotifyErr(LOG_ERR, "MMV: %s - "
+					"values offset: %"PRIu64" < %"PRIu64,
+					s->name, s->len, offset);
+		    }
+		    continue;
+		}
+		offset -= (count * sizeof(mmv_disk_value_t));
+
+		s->vcnt = count;
+		s->values = (mmv_disk_value_t *)((char *)s->addr + offset);
+		break;
+
+	    default:
+		if (pmDebug & DBG_TRACE_APPL0) {
+		    __pmNotifyErr(LOG_DEBUG, "MMV: %s - bad TOC type (%x)",
+				    s->name, type);
+		}
+		break;
 	    }
 	}
     }
 
-    pmdaTreeRebuildHash(pmns, mcnt);	/* for reverse (pmid->name) lookups */
+    pmdaTreeRebuildHash(pmns, mtot);	/* for reverse (pmid->name) lookups */
     reload = need_reload;
 }
 
 static int
-mmv_lookup_stat_metric_value(pmID pmid, unsigned int inst,
-	stats_t **sout, mmv_disk_metric_t **mout, mmv_disk_value_t **vout)
+mmv_lookup_item1(int item, unsigned int inst,
+	stats_t *s, mmv_disk_value_t **value,
+	__uint64_t *shorttext, __uint64_t *helptext)
 {
-    __pmID_int * id = (__pmID_int *)&pmid;
-    mmv_disk_metric_t * m;
-    mmv_disk_value_t * v;
-    stats_t * s;
-    int si, mi, vi;
-    int sts = PM_ERR_PMID;
+    mmv_disk_metric_t *m1 = s->metrics1;
+    mmv_disk_value_t *v = s->values;
+    int mi, vi, sts = PM_ERR_PMID;
 
-    for (si = 0; si < scnt; si++) {
-	s = &slist[si];
-	if (s->cluster != id->cluster)
+    for (mi = 0; mi < s->mcnt1; mi++) {
+	if (m1[mi].item != item)
 	    continue;
 
-	m = s->metrics;
-	for (mi = 0; mi < s->mcnt; mi++) {
-	    if (m[mi].item != id->item)
-		continue;
-
-	    sts = PM_ERR_INST;
-	    v = s->values;
-	    for (vi = 0; vi < s->vcnt; vi++) {
-		mmv_disk_metric_t * mt = (mmv_disk_metric_t *)
+	sts = PM_ERR_INST;
+	for (vi = 0; vi < s->vcnt; vi++) {
+	    mmv_disk_metric_t *mt = (mmv_disk_metric_t *)
 			((char *)s->addr + v[vi].metric);
-		mmv_disk_instance_t * is = (mmv_disk_instance_t *)
+	    mmv_disk_instance_t *is = (mmv_disk_instance_t *)
 			((char *)s->addr + v[vi].instance);
 
-		if ((mt == &m[mi]) &&
-		    (mt->indom == PM_INDOM_NULL || mt->indom == 0 ||
-		     inst == PM_IN_NULL || is->internal == inst)) {
-		    *sout = s;
-		    *mout = &m[mi];
-		    *vout = &v[vi];
-		    return 0;
-		}
+	    if ((mt == &m1[mi]) &&
+		(mt->indom == PM_INDOM_NULL || mt->indom == 0 ||
+		inst == PM_IN_NULL || is->internal == inst)) {
+		if (shorttext)
+		    *shorttext = m1[mi].shorttext;
+		if (helptext)
+		    *helptext = m1[mi].helptext;
+		*value = &v[vi];
+		return m1[mi].type;
 	    }
 	}
     }
     return sts;
 }
 
+static int
+mmv_lookup_item2(int item, unsigned int inst,
+	stats_t *s, mmv_disk_value_t **value,
+	__uint64_t *shorttext, __uint64_t *helptext)
+{
+    mmv_disk_metric2_t *m2 = s->metrics2;
+    mmv_disk_value_t *v = s->values;
+    int mi, vi, sts = PM_ERR_PMID;
+
+    for (mi = 0; mi < s->mcnt2; mi++) {
+	if (m2[mi].item != item)
+	    continue;
+
+	sts = PM_ERR_INST;
+	for (vi = 0; vi < s->vcnt; vi++) {
+	    mmv_disk_metric2_t *mt = (mmv_disk_metric2_t *)
+			((char *)s->addr + v[vi].metric);
+	    mmv_disk_instance2_t *is = (mmv_disk_instance2_t *)
+			((char *)s->addr + v[vi].instance);
+
+	    if ((mt == &m2[mi]) &&
+		(mt->indom == PM_INDOM_NULL || mt->indom == 0 ||
+		inst == PM_IN_NULL || is->internal == inst)) {
+		if (shorttext)
+		    *shorttext = m2[mi].shorttext;
+		if (helptext)
+		    *helptext = m2[mi].helptext;
+		*value = &v[vi];
+		return m2[mi].type;
+	    }
+	}
+    }
+    return sts;
+}
+
+static int
+mmv_lookup_stat_metric(pmID pmid, unsigned int inst,
+	stats_t **stats, mmv_disk_value_t **value,
+	__uint64_t *shorttext, __uint64_t *helptext)
+{
+    __pmID_int *id = (__pmID_int *)&pmid;
+    int si, sts = PM_ERR_PMID;
+
+    for (si = 0; si < scnt; si++) {
+	stats_t *s = &slist[si];
+
+	if (s->cluster != id->cluster)
+	    continue;
+
+	sts = (s->version == MMV_VERSION1) ?
+	    mmv_lookup_item1(id->item, inst, s, value, shorttext, helptext):
+	    mmv_lookup_item2(id->item, inst, s, value, shorttext, helptext);
+	if (sts >= 0) {
+	    *stats = s;
+	    break;
+	}
+    }
+    return sts;
+}
+
+static int
+mmv_lookup_stat_metric_value(pmID pmid, unsigned int inst,
+	stats_t **stats, mmv_disk_value_t **value)
+{
+    return mmv_lookup_stat_metric(pmid, inst, stats, value, NULL, NULL);
+}
+
 /*
  * callback provided to pmdaFetch
  */
 static int
 mmv_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 {
-    __pmID_int * id = (__pmID_int *)&(mdesc->m_desc.pmid);
+    __pmID_int *id = (__pmID_int *)&(mdesc->m_desc.pmid);
 
     if (id->cluster == 0) {
-	if (id->item == 0) {
-	    atom->l = reload;
-	    return 1;
-	}
-	if (id->item == 1) {
-	    atom->l = pmDebug;
+	if (id->item <= 2) {
+	    atom->l = *(int *)mdesc->m_user;
 	    return 1;
 	}
 	return PM_ERR_PMID;
 
     } else if (scnt > 0) {	/* We have at least one source of metrics */
-	mmv_disk_string_t * str;
-	mmv_disk_metric_t * m;
-	mmv_disk_value_t * v;
-	stats_t * s;
-	int rv;
+	static int setup;
+	static float fNaN;
+	static double dNaN;
+	static pmAtomValue aNaN;
+	static char buffer[MMV_STRINGMAX];
+	mmv_disk_string_t *str;
+	mmv_disk_value_t *v;
+	__uint64_t offset;
+	stats_t *s;
+	int rv, fl;
 
-	rv = mmv_lookup_stat_metric_value(mdesc->m_desc.pmid, inst, &s, &m, &v);
+	rv = mmv_lookup_stat_metric_value(mdesc->m_desc.pmid, inst, &s, &v);
 	if (rv < 0)
 	    return rv;
+	fl = ((mmv_disk_header_t *)s->addr)->flags;
 
-	switch (m->type) {
+	if (!setup) {
+	    setup++;
+	    fNaN = (float)0.0 / (float)0.0;
+	    dNaN = (double)0.0 / (double)0.0;
+	    memset(&aNaN, -1, sizeof(pmAtomValue));
+	}
+
+	switch (rv) {
 	    case MMV_TYPE_I32:
 	    case MMV_TYPE_U32:
 	    case MMV_TYPE_I64:
 	    case MMV_TYPE_U64:
+		memcpy(atom, &v->value, sizeof(pmAtomValue));
+		if ((fl & MMV_FLAG_SENTINEL) &&
+		    (memcmp(atom, &aNaN, sizeof(*atom)) == 0))
+		    return 0;
+		break;
 	    case MMV_TYPE_FLOAT:
+		memcpy(atom, &v->value, sizeof(pmAtomValue));
+		if ((fl & MMV_FLAG_SENTINEL) && atom->f == fNaN)
+		    return 0;
+		break;
 	    case MMV_TYPE_DOUBLE:
 		memcpy(atom, &v->value, sizeof(pmAtomValue));
+		if ((fl & MMV_FLAG_SENTINEL) && atom->d == dNaN)
+		    return 0;
 		break;
 	    case MMV_TYPE_ELAPSED: {
 		atom->ll = v->value.ll;
+		if ((fl & MMV_FLAG_SENTINEL) &&
+		    (memcmp(atom, &aNaN, sizeof(*atom)) == 0))
+		    return 0;
 		if (v->extra < 0) {	/* inside a timed section */
 		    struct timeval tv; 
 		    __pmtimevalNow(&tv); 
@@ -611,8 +943,21 @@ mmv_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 		break;
 	    }
 	    case MMV_TYPE_STRING: {
-		str = (mmv_disk_string_t *)((char *)s->addr + v->extra);
-		atom->cp = str->payload;
+		offset = v->extra;
+		if (s->len < offset + sizeof(MMV_STRINGMAX)) {
+		    if (pmDebug & DBG_TRACE_APPL0)
+			__pmNotifyErr(LOG_ERR, "MMV: %s - "
+				"bad string value offset: %"PRIu64" < %"PRIu64,
+				s->name, s->len,
+				offset + sizeof(MMV_STRINGMAX));
+		    return PM_ERR_GENERIC;
+		}
+		str = (mmv_disk_string_t *)((char *)s->addr + offset);
+		memcpy(buffer, str->payload, sizeof(buffer));
+		if ((fl & MMV_FLAG_SENTINEL) && buffer[0] == '\0')
+		    return 0;
+		buffer[sizeof(buffer)-1] = '\0';
+		atom->cp = buffer;
 		break;
 	    }
 	    case MMV_TYPE_NOSUPPORT:
@@ -671,13 +1016,13 @@ mmv_reload_maybe(pmdaExt *pmda)
 	map_stats(pmda);
 
 	pmda->e_indoms = indoms;
-	pmda->e_nindoms = incnt;
-	pmdaRehash(pmda, metrics, mcnt);
+	pmda->e_nindoms = intot;
+	pmdaRehash(pmda, metrics, mtot);
 
 	if (pmDebug & DBG_TRACE_APPL0)
 	    __pmNotifyErr(LOG_DEBUG, 
 		      "MMV: %s: %d metrics and %d indoms after reload", 
-		      pmProgname, mcnt, incnt);
+		      pmProgname, mtot, intot);
     }
 }
 
@@ -689,6 +1034,58 @@ mmv_desc(pmID pmid, pmDesc *desc, pmdaExt *ep)
     return pmdaDesc(pmid, desc, ep);
 }
 
+static int
+mmv_lookup_metric_helptext(pmID pmid, int type, char **text)
+{
+    static char string[MMV_STRINGMAX];
+    mmv_disk_string_t *str;
+    mmv_disk_value_t *v;
+    __uint64_t st, lt;
+    size_t offset;
+    stats_t *s;
+
+    if (mmv_lookup_stat_metric(pmid, PM_IN_NULL, &s, &v, &st, &lt) < 0)
+	return PM_ERR_PMID;
+
+    if ((type & PM_TEXT_ONELINE) && st) {
+	offset = st + sizeof(mmv_disk_string_t);
+	if (s->len < offset) {
+	    if (pmDebug & DBG_TRACE_APPL0)
+		__pmNotifyErr(LOG_ERR, "MMV: %s - "
+				"bad shorttext offset: %"PRIu64" < %"PRIu64,
+				s->name, s->len, (uint64_t)offset);
+	    return PM_ERR_GENERIC;
+	}
+	offset -= sizeof(mmv_disk_string_t);
+
+	str = (mmv_disk_string_t *)((char *)s->addr + offset);
+	memcpy(string, str->payload, sizeof(string));
+	string[sizeof(string)-1] = '\0';
+	*text = string;
+	return 0;
+    }
+
+    if ((type & PM_TEXT_HELP) && lt) {
+	offset = lt + sizeof(mmv_disk_string_t);
+	if (s->len < offset) {
+	    if (pmDebug & DBG_TRACE_APPL0)
+		__pmNotifyErr(LOG_ERR, "MMV: %s - "
+				"bad helptext offset: %"PRIu64" < %"PRIu64,
+				s->name, s->len, (uint64_t)offset);
+	    return PM_ERR_GENERIC;
+	}
+	offset -= sizeof(mmv_disk_string_t);
+
+	str = (mmv_disk_string_t *)((char *)s->addr + offset);
+	memcpy(string, str->payload, sizeof(string));
+	string[sizeof(string)-1] = '\0';
+	*text = string;
+	return 0;
+    }
+
+    return PM_ERR_TEXT;
+}
+
 static int
 mmv_text(int ident, int type, char **buffer, pmdaExt *ep)
 {
@@ -697,48 +1094,37 @@ mmv_text(int ident, int type, char **buffer, pmdaExt *ep)
 
     mmv_reload_maybe(ep);
     if (pmid_cluster(ident) == 0) {
-	if (pmid_item(ident) == 0) {
-	    static char reloadoneline[] = "Control maps reloading";
-	    static char reloadtext[] = 
+	switch (pmid_item(ident)) {
+	    case 0: {
+		static char reloadoneline[] = "Control maps reloading";
+		static char reloadtext[] = 
 "Writing anything other then 0 to this metric will result in\n"
 "re-reading directory and re-mapping files.\n";
 
-	    *buffer = (type & PM_TEXT_ONELINE) ? reloadoneline : reloadtext;
-	    return 0;
-	}
-	else if (pmid_item(ident) == 1) {
-	    static char debugoneline[] = "Debug flag";
-	    static char debugtext[] =
+		*buffer = (type & PM_TEXT_ONELINE) ? reloadoneline : reloadtext;
+		return 0;
+	    }
+	    case 1: {
+		static char debugoneline[] = "Debug flag";
+		static char debugtext[] =
 "See pmdbg(1).  pmstore into this metric to change the debug value.\n";
 
-	    *buffer = (type & PM_TEXT_ONELINE) ? debugoneline : debugtext;
-	    return 0;
-	}
-	else
-	    return PM_ERR_PMID;
-    }
-    else {
-	mmv_disk_string_t * str;
-	mmv_disk_metric_t * m;
-	mmv_disk_value_t * v;
-	stats_t * s;
+		*buffer = (type & PM_TEXT_ONELINE) ? debugoneline : debugtext;
+		return 0;
+	    }
+	    case 2: {
+		static char filesoneline[] = "Memory mapped file count";
+		static char filestext[] =
+"Count of currently mapped and exported statistics files.\n";
 
-	if (mmv_lookup_stat_metric_value(ident, PM_IN_NULL, &s, &m, &v) != 0)
-	    return PM_ERR_PMID;
-
-	if ((type & PM_TEXT_ONELINE) && m->shorttext) {
-	    str = (mmv_disk_string_t *)((char *)s->addr + m->shorttext);
-	    *buffer = str->payload;
-	    return 0;
-	}
-	if ((type & PM_TEXT_HELP) && m->helptext) {
-	    str = (mmv_disk_string_t *)((char *)s->addr + m->helptext);
-	    *buffer = str->payload;
-	    return 0;
+		*buffer = (type & PM_TEXT_ONELINE) ? filesoneline : filestext;
+		return 0;
+	    }
 	}
+	return PM_ERR_PMID;
     }
 
-    return PM_ERR_TEXT;
+    return mmv_lookup_metric_helptext(ident, type, buffer);
 }
 
 static int
@@ -768,7 +1154,7 @@ mmv_store(pmResult *result, pmdaExt *ep)
 	__pmID_int * id = (__pmID_int *)&vsp->pmid;
 
 	if (id->cluster == 0) {
-	    for (m = 0; m < mcnt; m++) {
+	    for (m = 0; m < mtot; m++) {
 		__pmID_int * mid = (__pmID_int *)&(metrics[m].m_desc.pmid);
 
 		if (mid->cluster == 0 && mid->item == id->item) {
@@ -776,15 +1162,15 @@ mmv_store(pmResult *result, pmdaExt *ep)
 		    int sts;
 
 		    if (vsp->numval != 1 )
-			return PM_ERR_CONV;
+			return PM_ERR_BADSTORE;
 
 		    if ((sts = pmExtractValue(vsp->valfmt, &vsp->vlist[0],
 					PM_TYPE_32, &atom, PM_TYPE_32)) < 0)
 			return sts;
 		    if (id->item == 0)
-		    reload = atom.l;
+			reload = atom.l;
 		    else if (id->item == 1)
-		    	pmDebug = atom.l;
+			pmDebug = atom.l;
 		    else
 			return PM_ERR_PERMISSION;
 		}
@@ -845,16 +1231,18 @@ mmv_init(pmdaInterface *dp)
 	 * number of hard-coded metrics here has to match initializer
 	 * cases below, and pmns initialization in map_stats()
 	 */
-	mcnt = 2;
-	if ((metrics = malloc(mcnt*sizeof(pmdaMetric))) != NULL) {
+	mtot = 3;
+	if ((metrics = malloc(mtot * sizeof(pmdaMetric))) != NULL) {
 	    /*
 	     * all the hard-coded metrics have the same semantics
 	     */
-	    for (m = 0; m < mcnt; m++) {
+	    for (m = 0; m < mtot; m++) {
 		if (m == 0)
 		    metrics[m].m_user = &reload;
 		else if (m == 1)
 		    metrics[m].m_user = &pmDebug;
+		else if (m == 2)
+		    metrics[m].m_user = &scnt;
 		metrics[m].m_desc.pmid = pmid_build(dp->domain, 0, m);
 		metrics[m].m_desc.type = PM_TYPE_32;
 		metrics[m].m_desc.indom = PM_INDOM_NULL;
@@ -880,7 +1268,7 @@ mmv_init(pmdaInterface *dp)
 	pmdaSetFetchCallBack(dp, mmv_fetchCallBack);
 
 	pmdaSetFlags(dp, PMDA_EXT_FLAG_HASHED);
-	pmdaInit(dp, indoms, incnt, metrics, mcnt);
+	pmdaInit(dp, indoms, intot, metrics, mtot);
     }
 }
 
