@@ -1,7 +1,7 @@
 /*
  * PMCD privileged co-process (root) PMDA.
  *
- * Copyright (c) 2014-2015 Red Hat.
+ * Copyright (c) 2014-2017 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -17,7 +17,7 @@
 #include "pmapi.h"
 #include "impl.h"
 #include "pmda.h"
-#include <sys/stat.h>
+#include "pmdaroot.h"
 #include "root.h"
 #include "lxc.h"
 #include "docker.h"
@@ -38,7 +38,7 @@ static int socket_fd = -1;
 static int pmcd_fd = -1;
 
 static __pmFdSet connected_fds;
-static int maximum_fd;
+int root_maximum_fd;
 
 static const int features = PDUROOT_FLAG_HOSTNAME | \
 			    PDUROOT_FLAG_PROCESSID | \
@@ -48,6 +48,16 @@ static pmdaIndom root_indomtab[NUM_INDOMS];
 #define INDOM(x) (root_indomtab[x].it_indom)
 #define INDOMTAB_SZ (sizeof(root_indomtab)/sizeof(root_indomtab[0]))
 
+
+json_metric_desc json_metrics[] = {
+    { "State/Pid", 0, 1, {0}, ""},
+    { "Name", 0, 1, {0}, ""},
+    { "State/Running", CONTAINER_FLAG_RUNNING, 1, {0}, ""},
+    { "State/Paused", CONTAINER_FLAG_PAUSED, 1, {0}, ""},
+    { "State/Restarting", CONTAINER_FLAG_RESTARTING, 1, {0}, ""},
+};
+
+
 static pmdaMetric root_metrictab[] = {
     { NULL, { PMDA_PMID(0, CONTAINERS_ENGINE), PM_TYPE_STRING,
 	CONTAINERS_INDOM, PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,0,0,0,0) } },
@@ -61,6 +71,8 @@ static pmdaMetric root_metrictab[] = {
 	CONTAINERS_INDOM, PM_SEM_INSTANT, PMDA_PMUNITS(0,0,0,0,0,0) } },
     { NULL, { PMDA_PMID(0, CONTAINERS_RESTARTING), PM_TYPE_U32,
 	CONTAINERS_INDOM, PM_SEM_INSTANT, PMDA_PMUNITS(0,0,0,0,0,0) } },
+    { NULL, { PMDA_PMID(0, CONTAINERS_CGROUP), PM_TYPE_STRING,
+	CONTAINERS_INDOM, PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,0,0,0,0) } },
 };
 #define METRICTAB_SZ (sizeof(root_metrictab)/sizeof(root_metrictab[0]))
 
@@ -129,11 +141,19 @@ container_t *
 root_container_search(const char *query)
 {
     int inst, fuzzy, best = 0;
-    char *name = NULL;
+    char *name = (char *)query;
     container_t *cp = NULL, *found = NULL;
     container_engine_t *dp;
     pmInDom indom = INDOM(CONTAINERS_INDOM);
 
+    /* fast path - full instance name, always the best possible match */
+    if (query && PMDA_CACHE_ACTIVE ==
+	pmdaCacheLookupName(indom, name, &inst, (void **)&cp) &&
+	root_refresh_container_values(name, cp) >= 0) {
+	found = cp;
+	goto out;
+    }
+
     for (pmdaCacheOp(indom, PMDA_CACHE_WALK_REWIND);;) {
 	if ((inst = pmdaCacheOp(indom, PMDA_CACHE_WALK_NEXT)) < 0)
 	    break;
@@ -152,6 +172,7 @@ root_container_search(const char *query)
 	}
     }
 
+out:
     if (pmDebug & DBG_TRACE_ATTR) {
 	if (found) /* query must be non-NULL */
 	    __pmNotifyErr(LOG_DEBUG, "found container: %s (%s/%d) pid=%d\n",
@@ -204,9 +225,14 @@ root_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	    atom->cp = cp->engine->name;
 	    break;
 	case 1:		/* containers.name */
-	    atom->cp = *cp->name == '/' ? cp->name+1 : cp->name;
+	    if (cp->name)
+		atom->cp = *cp->name == '/' ? cp->name+1 : cp->name;
+	    else
+		atom->cp = "?";
 	    break;
 	case 2:		/* containers.pid */
+	    if (cp->pid <= 0)
+		return PMDA_FETCH_NOVALUES;
 	    atom->ul = cp->pid;
 	    break;
 	case 3:		/* containers.state.running */
@@ -218,6 +244,11 @@ root_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	case 5:		/* containers.state.restarting */
 	    atom->ul = (cp->flags & CONTAINER_FLAG_RESTARTING) != 0;
 	    break;
+	case 6:		/* containers.cgroup */
+	    if (cp->pid <= 0)
+		return PMDA_FETCH_NOVALUES;
+	    atom->cp = cp->cgroup;
+	    break;
 	default:
 	    return PM_ERR_PMID;
 	}
@@ -227,7 +258,7 @@ root_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	return PM_ERR_PMID;
     }
 
-    return 1;
+    return PMDA_FETCH_STATIC;
 }
 
 /* General utility routine for checking timestamp differences */
@@ -311,8 +342,8 @@ root_setup_socket(void)
 	exit(1);
     }
 
-    if (fd > maximum_fd)
-	maximum_fd = fd;
+    if (fd > root_maximum_fd)
+	root_maximum_fd = fd;
 
     __pmFD_ZERO(&connected_fds);
     __pmFD_SET(fd, &connected_fds);
@@ -344,9 +375,9 @@ typedef struct {
     int		fd;
 } root_client_t;
 
-static root_client_t *client;
-static int client_size;	/* highwater allocation mark */
-static int nclients;
+static root_client_t *root_client;
+static int root_client_size;	/* highwater allocation mark */
+static int nrootclients;
 static const int MIN_CLIENTS_ALLOC = 8;
 
 static int
@@ -354,24 +385,27 @@ root_new_client(void)
 {
     int i, sz;
 
-    for (i = 0; i < nclients; i++)
-	if (client[i].fd < 0)
+    for (i = 0; i < nrootclients; i++)
+	if (root_client[i].fd < 0)
 	    break;
 
-    if (i == client_size) {
-	client_size = client_size ? client_size * 2 : MIN_CLIENTS_ALLOC;
-	sz = sizeof(root_client_t) * client_size;
-	client = (root_client_t *)realloc(client, sz);
-	if (client == NULL) {
+    if (i == root_client_size) {
+	if (root_client_size == 0)
+	    root_client_size = MIN_CLIENTS_ALLOC;
+	else
+	    root_client_size *= 2;
+	sz = sizeof(root_client_t) * root_client_size;
+	root_client = (root_client_t *)realloc(root_client, sz);
+	if (root_client == NULL) {
 	    __pmNoMem("root_new_client", sz, PM_RECOV_ERR);
 	    root_close_socket();
 	    exit(1);
 	}
 	sz -= (sizeof(root_client_t) * i);
-	memset(&client[i], 0, sz);
+	memset(&root_client[i], 0, sz);
     }
-    if (i >= nclients)
-	nclients = i + 1;
+    if (i >= nrootclients)
+	nrootclients = i + 1;
     return i;
 }
 
@@ -380,8 +414,8 @@ root_delete_client(root_client_t *cp)
 {
     int		i;
 
-    for (i = 0; i < nclients; i++) {
-	if (cp == &client[i])
+    for (i = 0; i < nrootclients; i++) {
+	if (cp == &root_client[i])
 	    break;
     }
 
@@ -389,13 +423,13 @@ root_delete_client(root_client_t *cp)
 	__pmFD_CLR(cp->fd, &connected_fds);
 	__pmCloseSocket(cp->fd);
     }
-    if (i >= nclients - 1)
-	nclients = i;
-    if (cp->fd == maximum_fd) {
-	maximum_fd = (pmcd_fd > socket_fd) ? pmcd_fd : socket_fd;
-	for (i = 0; i < nclients; i++) {
-	    if (client[i].fd > maximum_fd)
-		maximum_fd = client[i].fd;
+    if (i >= nrootclients - 1)
+	nrootclients = i;
+    if (cp->fd == root_maximum_fd) {
+	root_maximum_fd = (pmcd_fd > socket_fd) ? pmcd_fd : socket_fd;
+	for (i = 0; i < nrootclients; i++) {
+	    if (root_client[i].fd > root_maximum_fd)
+		root_maximum_fd = root_client[i].fd;
 	}
     }
     cp->fd = -1;
@@ -413,8 +447,8 @@ root_accept_client(void)
 	if (neterror() == EPERM) {
 	    __pmNotifyErr(LOG_NOTICE, "root_accept_client(%d): %s\n",
 			fd, osstrerror());
-	    client[i].fd = -1;
-	    root_delete_client(&client[i]);
+	    root_client[i].fd = -1;
+	    root_delete_client(&root_client[i]);
 	    return NULL;
 	} else {
 	    __pmNotifyErr(LOG_ERR, "root_accept_client(%d): accept: %s\n",
@@ -423,12 +457,12 @@ root_accept_client(void)
 	    exit(1);
 	}
     }
-    if (fd > maximum_fd)
-	maximum_fd = fd;
+    if (fd > root_maximum_fd)
+	root_maximum_fd = fd;
     __pmFD_SET(fd, &connected_fds);
 
-    client[i].fd = fd;
-    return &client[i];
+    root_client[i].fd = fd;
+    return &root_client[i];
 }
 
 static void
@@ -571,8 +605,9 @@ root_cgroupname_request(root_client_t *cp, void *pdu, int pdulen)
     } else {
 	sts = 0;
 	pid = container->pid;
-	cgroup = container->cgroup;
-	length = strlen(container->cgroup);
+	/* skip leading slash, it is there just for exported metric value */
+	cgroup = &container->cgroup[1];
+	length = strlen(cgroup);
 	if (pmDebug & DBG_TRACE_ATTR)
 	    __pmNotifyErr(LOG_DEBUG, "container %s cgroup=%s\n", name, cgroup);
     }
@@ -580,6 +615,57 @@ root_cgroupname_request(root_client_t *cp, void *pdu, int pdulen)
 			pid, cgroup, length, sts);
 }
 
+static int
+root_startpmda_request(root_client_t *cp, void *pdu, int pdulen)
+{
+    size_t	len;
+    pid_t	pid;
+    int		infd, outfd;
+    int		sts, ipc, bad = 0;
+    char	name[MAXPMDALEN];
+    char	args[MAXPATHLEN];
+
+    if ((sts = __pmdaDecodeRootPDUStart(pdu, pdulen, NULL, NULL, NULL,
+			&ipc, name, sizeof(name), args, sizeof(args))) < 0)
+	return sts;
+    len = strlen(name);
+
+    if ((pid = root_create_agent(ipc, args, name, &infd, &outfd)) < 0)
+	bad = PM_ERR_GENERIC;
+
+    sts = __pmdaSendRootPDUStart(cp->fd, pid, infd, outfd, name, len, bad);
+    if (pmDebug & DBG_TRACE_APPL0) {
+	__pmNotifyErr(LOG_DEBUG, "Sent %s PMDA process to pmcd: "
+			"pid=%d infd=%d outfd=%d sts=%d\n",
+			name, pid, infd, outfd, bad);
+    }
+    if (outfd >= 0)
+	close(outfd);
+    if (infd >= 0)
+	close(infd);
+    return sts;
+}
+
+static int
+root_stoppmda_request(root_client_t *cp, void *pdu, int pdulen)
+{
+    int		sts, force = 0, code = 0, pid = -1;
+ 
+    if ((sts = __pmdaDecodeRootPDUStop(pdu, pdulen, &pid, NULL, &force)) < 0)
+	return sts;
+
+    if (force || pid >= 1) {
+	if (pid == -1)
+	    sts = -EINVAL;
+	else
+	    sts = __pmProcessTerminate(pid, force);
+    } else {
+	pid = root_agent_wait(&code);
+	sts = 0;
+    }
+    return __pmdaSendRootPDUStop(cp->fd, PDUROOT_STOPPMDA, pid, code, 0, sts);
+}
+
 static int
 root_recvpdu(int fd, __pmdaRootPDUHdr **hdr)
 {
@@ -587,7 +673,7 @@ root_recvpdu(int fd, __pmdaRootPDUHdr **hdr)
     __pmdaRootPDUHdr	*pdu = (__pmdaRootPDUHdr *)buffer;
     int			bytes;
 
-    if ((bytes = recv(fd, &buffer, sizeof(buffer), 0)) < 0) {
+    if ((bytes = recv(fd, (void *)&buffer, sizeof(buffer), 0)) < 0) {
 	__pmNotifyErr(LOG_ERR, "root_recvpdu: recv - %s\n", osstrerror());
 	return bytes;
     }
@@ -623,8 +709,8 @@ root_handle_client_input(__pmFdSet *fds)
     root_client_t	*cp;
     int			i, sts;
 
-    for (i = 0; i < nclients; i++) {
-	cp = &client[i];
+    for (i = 0; i < nrootclients; i++) {
+	cp = &root_client[i];
 	if (cp->fd == -1 || !__pmFD_ISSET(cp->fd, fds))
 	    continue;
 
@@ -646,18 +732,21 @@ root_handle_client_input(__pmFdSet *fds)
 	    sts = root_cgroupname_request(cp, (void *)php, sts);
 	    break;
 
-	/*
-	 * We expect to add functionality here over time, e.g.:
-	 * - PDU for (re)starting a PMDA & pass back fds;
-	 * - authentication requests via SASL;
-	 */
+	case PDUROOT_STARTPMDA_REQ:
+	    sts = root_startpmda_request(cp, (void *)php, sts);
+	    break;
+
+	case PDUROOT_STOPPMDA_REQ:
+	    sts = root_stoppmda_request(cp, (void *)php, sts);
+	    break;
 
 	default:
 	    sts = PM_ERR_IPC;
-	} 
+	}
 
 	if (sts < 0) {
-	    __pmNotifyErr(LOG_ERR, "bad protocol exchange (fd=%d)\n", cp->fd);
+	    __pmNotifyErr(LOG_ERR, "bad protocol exchange (fd=%d,type=%x)\n",
+				cp->fd, php->type);
 	    root_delete_client(cp);
 	}
     }
@@ -673,12 +762,13 @@ root_main(pmdaInterface *dp)
 
     pmcd_fd = __pmdaInFd(dp);
     __pmFD_SET(pmcd_fd, &connected_fds);
-    maximum_fd = (socket_fd > pmcd_fd) ? socket_fd : pmcd_fd;
+    root_maximum_fd = (socket_fd > pmcd_fd) ? socket_fd : pmcd_fd;
 
     for (;;) {
 	readable_fds = connected_fds;
-	maxfd = maximum_fd + 1;
+	maxfd = root_maximum_fd + 1;
 
+	setoserror(0);
 	sts = __pmSelectRead(maxfd, &readable_fds, NULL);
 	if (sts > 0) {
 	    if (__pmFD_ISSET(pmcd_fd, &readable_fds)) {
@@ -691,9 +781,9 @@ root_main(pmdaInterface *dp)
 		root_check_new_client(&readable_fds);
 	    root_handle_client_input(&readable_fds);
 	}
-	else if (sts == -1 && neterror() != EINTR) {
-	    __pmNotifyErr(LOG_ERR, "root_main select: %s\n", netstrerror());
-	    break;
+	else if (sts == -1 && oserror() != EINTR) {
+	    __pmNotifyErr(LOG_ERR, "root_main select: %s\n", osstrerror());
+	    continue;
 	}
     }
 }
@@ -703,20 +793,29 @@ root_check_user(void)
 {
 #ifdef HAVE_GETUID
     if (getuid() != 0) {
-	__pmNotifyErr(LOG_ERR, "must be run as root\n");
+	__pmNotifyErr(LOG_ERR, "must be run as root %d\n", getuid());
 	exit(1);
     }
 #endif
 }
 
+/*
+ * Perform early checking, and setup - before communicating with
+ * anyone else (incl. pmcd).
+ */
+static void
+root_prep(void)
+{
+    root_check_user();
+    root_setup_socket();
+    atexit(root_close_socket);
+}
+
 static void
 root_init(pmdaInterface *dp)
 {
-    root_check_user();
     root_setup_containers();
-    root_container_search(NULL);	/* potentially costly early scan */
-    root_setup_socket();
-    atexit(root_close_socket);
+    root_container_search(NULL); /* potentially costly early scan */
 
     dp->version.any.fetch = root_fetch;
     dp->version.any.instance = root_instance;
@@ -768,6 +867,7 @@ main(int argc, char **argv)
     }
 
     pmdaOpenLog(&dispatch);
+    root_prep();
     pmdaConnect(&dispatch);
     root_init(&dispatch);
     root_main(&dispatch);
