@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014 Red Hat.
+ * Copyright (c) 2014-2016 Red Hat.
  * Copyright (c) 1995-2002 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -27,6 +27,7 @@ static pmID		pmid_flags;
 static pmID		pmid_missed;
 static int		sflag;
 static int		xflag;		/* for -x (long timestamps) */
+static pmLogLabel	label;
 
 static pmLongOptions longopts[] = {
     PMAPI_OPTIONS_HEADER("Options"),
@@ -47,6 +48,7 @@ static pmLongOptions longopts[] = {
     { "", 0, 'x', 0, "include date in reported timestamps" },
     PMOPT_TIMEZONE,
     PMOPT_HOSTZONE,
+    PMOPT_VERSION,
     PMOPT_HELP,
     PMAPI_OPTIONS_END
 };
@@ -54,7 +56,7 @@ static pmLongOptions longopts[] = {
 static int overrides(int, pmOptions *);
 static pmOptions opts = {
     .flags = PM_OPTFLAG_DONE | PM_OPTFLAG_STDOUT_TZ | PM_OPTFLAG_BOUNDARIES,
-    .short_options = "aD:dilLmn:rS:sT:tv:xZ:z?",
+    .short_options = "aD:dilLmMn:rS:sT:tv:xZ:z?",
     .long_options = longopts,
     .short_usage = "[options] [archive [metricname ...]]",
     .override = overrides,
@@ -183,15 +185,15 @@ dump_parameter(pmValueSet *xvsp, int index, int *flagsp)
 {
     int		sts, flags = *flagsp;
     pmDesc	desc;
-    char	*name;
+    char	**names;
 
-    if (pmNameID(xvsp->pmid, &name) >= 0) {
+    if ((sts = pmNameAll(xvsp->pmid, &names)) >= 0) {
 	if (index == 0) {
 	    if (xvsp->pmid == pmid_flags) {
 		flags = *flagsp = xvsp->vlist[0].value.lval;
 		printf(" flags 0x%x", flags);
 		printf(" (%s) ---\n", pmEventFlagsStr(flags));
-		free(name);
+		free(names);
 		return;
 	    }
 	    printf(" ---\n");
@@ -200,11 +202,13 @@ dump_parameter(pmValueSet *xvsp, int index, int *flagsp)
 	    (xvsp->pmid == pmid_missed)) {
 	    printf("              ==> %d missed event records\n",
 		    xvsp->vlist[0].value.lval);
-	    free(name);
+	    free(names);
 	    return;
 	}
-	printf("                %s (%s):", pmIDStr(xvsp->pmid), name);
-	free(name);
+	printf("                %s (", pmIDStr(xvsp->pmid));
+	__pmPrintMetricNames(stdout, sts, names, " or ");
+	printf("):");
+	free(names);
     }
     else
 	printf("	            PMID: %s:", pmIDStr(xvsp->pmid));
@@ -218,7 +222,7 @@ dump_parameter(pmValueSet *xvsp, int index, int *flagsp)
 }
 
 static void
-dump_event(const char *name, pmValueSet *vsp, int index, int indom, int type)
+dump_event(int numnames, char **names, pmValueSet *vsp, int index, int indom, int type)
 {
     int		r;		/* event records */
     int		p;		/* event parameters */
@@ -231,7 +235,8 @@ dump_event(const char *name, pmValueSet *vsp, int index, int indom, int type)
 
     if (index > 0)
 	printf("            ");
-    printf("  %s (%s", pmIDStr(vsp->pmid), name);
+    printf("  %s (", pmIDStr(vsp->pmid));
+    __pmPrintMetricNames(stdout, numnames, names, " or ");
     if (indom != PM_INDOM_NULL) {
 	putchar('[');
 	if (pmNameInDom(indom, vp->inst, &iname) < 0)
@@ -313,13 +318,15 @@ dump_event(const char *name, pmValueSet *vsp, int index, int indom, int type)
 }
 
 static void
-dump_metric(const char *mname, pmValueSet *vsp, int index, int indom, int type)
+dump_metric(int numnames, char **names, pmValueSet *vsp, int index, int indom, int type)
 {
     pmValue	*vp = &vsp->vlist[index];
     char	*iname;
 
     if (index == 0) {
-	printf("  %s (%s):", pmIDStr(vsp->pmid), mname);
+	printf("  %s (", pmIDStr(vsp->pmid));
+	__pmPrintMetricNames(stdout, numnames, names, " or ");
+	printf("):");
 	if (vsp->numval > 1) {
 	    putchar('\n');
 	    printf("               ");
@@ -348,7 +355,7 @@ dump_result(pmResult *resp)
     int		i;
     int		j;
     int		n;
-    char	*mname = NULL;
+    char	**names;
     pmDesc	desc;
 
     if (sflag) {
@@ -361,12 +368,14 @@ dump_result(pmResult *resp)
 	char	       *ddmm;
 	char	       *yr;
 
-	ddmm = pmCtime(&resp->timestamp.tv_sec, timebuf);
+	ddmm = pmCtime((const time_t *)&resp->timestamp.tv_sec, timebuf);
 	ddmm[10] = '\0';
 	yr = &ddmm[20];
 	printf("%s ", ddmm);
 	__pmPrintStamp(stdout, &resp->timestamp);
 	printf(" %4.4s", yr);
+	if (xflag >= 2)
+	    printf(" (%.6f)", __pmtimevalSub(&resp->timestamp, &label.ll_start));
     }
     else
 	__pmPrintStamp(stdout, &resp->timestamp);
@@ -381,14 +390,18 @@ dump_result(pmResult *resp)
 
 	if (i > 0)
 	    printf("            ");
-	if ((n = pmNameID(vsp->pmid, &mname)) < 0)
-	    mname = strdup("<noname>");
+	names = NULL; /* silence coverity */
+	n = pmNameAll(vsp->pmid, &names);
 	if (vsp->numval == 0) {
-	    printf("  %s (%s): No values returned!\n", pmIDStr(vsp->pmid), mname);
+	    printf("  %s (", pmIDStr(vsp->pmid));
+	    __pmPrintMetricNames(stdout, n, names, " or ");
+	    printf("): No values returned!\n");
 	    goto next;
 	}
 	else if (vsp->numval < 0) {
-	    printf("  %s (%s): %s\n", pmIDStr(vsp->pmid), mname, pmErrStr(vsp->numval));
+	    printf("  %s (", pmIDStr(vsp->pmid));
+	    __pmPrintMetricNames(stdout, n, names, " or ");
+	    printf("): %s\n", pmErrStr(vsp->numval));
 	    goto next;
 	}
 
@@ -404,36 +417,38 @@ dump_result(pmResult *resp)
 	for (j = 0; j < vsp->numval; j++) {
 	    if (desc.type == PM_TYPE_EVENT ||
 		desc.type == PM_TYPE_HIGHRES_EVENT)
-		dump_event(mname, vsp, j, desc.indom, desc.type);
+		dump_event(n, names, vsp, j, desc.indom, desc.type);
 	    else
-		dump_metric(mname, vsp, j, desc.indom, desc.type);
+		dump_metric(n, names, vsp, j, desc.indom, desc.type);
 	}
 next:
-	if (mname)
-	    free(mname);
-	mname = NULL;
+	if (n > 0)
+	    free(names);
     }
 }
 
 static void
 dumpDesc(__pmContext *ctxp)
 {
-    int		i;
-    int		sts;
-    char	*p;
+    int			i;
+    int			sts;
+    char		**names;
     __pmHashNode	*hp;
-    pmDesc	*dp;
+    pmDesc		*dp;
 
     printf("\nDescriptions for Metrics in the Log ...\n");
     for (i = 0; i < ctxp->c_archctl->ac_log->l_hashpmid.hsize; i++) {
 	for (hp = ctxp->c_archctl->ac_log->l_hashpmid.hash[i]; hp != NULL; hp = hp->next) {
 	    dp = (pmDesc *)hp->data;
-	    sts = pmNameID(dp->pmid, &p);
+	    names = NULL; /* silence coverity */
+	    sts = pmNameAll(dp->pmid, &names);
 	    if (sts < 0)
 		printf("PMID: %s (%s)\n", pmIDStr(dp->pmid), "<noname>");
 	    else {
-		printf("PMID: %s (%s)\n", pmIDStr(dp->pmid), p);
-		free(p);
+		printf("PMID: %s (", pmIDStr(dp->pmid));
+		__pmPrintMetricNames(stdout, sts, names, " or ");
+		printf(")\n");
+		free(names);
 	    }
 	    __pmPrintDesc(stdout, dp);
 	}
@@ -486,6 +501,9 @@ dumpTI(__pmContext *ctxp)
     struct stat	sbuf;
     __pmLogTI	*tip;
     __pmLogTI	*lastp;
+    __pmLogCtl  *lcp;
+    
+    lcp = ctxp->c_archctl->ac_log;
 
     printf("\nTemporal Index\n");
     printf("             Log Vol    end(meta)     end(log)\n");
@@ -497,14 +515,14 @@ dumpTI(__pmContext *ctxp)
 	__pmPrintStamp(stdout, &tv);
 	printf("    %4d  %11d  %11d\n", tip->ti_vol, tip->ti_meta, tip->ti_log);
 	if (i == 0) {
-	    sprintf(path, "%s.meta", opts.archives[0]);
+	    sprintf(path, "%s.meta", lcp->l_name);
 	    if (stat(path, &sbuf) == 0)
 		meta_size = sbuf.st_size;
 	    else
 		meta_size = -1;
 	}
 	if (lastp == NULL || tip->ti_vol != lastp->ti_vol) { 
-	    sprintf(path, "%s.%d", opts.archives[0], tip->ti_vol);
+	    sprintf(path, "%s.%d", lcp->l_name, tip->ti_vol);
 	    if (stat(path, &sbuf) == 0)
 		log_size = sbuf.st_size;
 	    else {
@@ -560,21 +578,13 @@ dumpTI(__pmContext *ctxp)
 static void
 dumpLabel(int verbose)
 {
-    pmLogLabel	label;
     char	*ddmm;
     char	*yr;
-    int		sts;
-
-    if ((sts = pmGetArchiveLabel(&label)) < 0) {
-	fprintf(stderr, "%s: Cannot get archive label record: %s\n",
-		pmProgname, pmErrStr(sts));
-	exit(1);
-    }
 
     printf("Log Label (Log Format Version %d)\n", label.ll_magic & 0xff);
     printf("Performance metrics from host %s\n", label.ll_hostname);
 
-    ddmm = pmCtime(&label.ll_start.tv_sec, timebuf);
+    ddmm = pmCtime((const time_t *)&label.ll_start.tv_sec, timebuf);
     ddmm[10] = '\0';
     yr = &ddmm[20];
     printf("  commencing %s ", ddmm);
@@ -586,7 +596,7 @@ dumpLabel(int verbose)
 	printf("  ending     UNKNOWN\n");
     }
     else {
-	ddmm = pmCtime(&opts.finish.tv_sec, timebuf);
+	ddmm = pmCtime((const time_t *)&opts.finish.tv_sec, timebuf);
 	ddmm[10] = '\0';
 	yr = &ddmm[20];
 	printf("  ending     %s ", ddmm);
@@ -669,6 +679,25 @@ overrides(int opt, pmOptions *opts)
     return 0;
 }
 
+static int
+isSingleArchive(const char *name)
+{
+    struct stat	sbuf;
+
+    /* Do not allow a comma within the name. */
+    if (strchr(name, ',') != NULL)
+	return 0;
+
+    /* No not allow a directory */
+    if (stat(name, &sbuf) != 0)
+	return 1; /* Let pmNewContext(1) issue the error */
+
+    if (S_ISDIR(sbuf.st_mode))
+	return 0; /* It's a directory */
+
+    return 1; /* ok */
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -682,11 +711,14 @@ main(int argc, char *argv[])
     int			iflag = 0;
     int			Lflag = 0;
     int			lflag = 0;
+    int			Mflag = 0;
     int			mflag = 0;
     int			tflag = 0;
     int			vflag = 0;
     int			mode = PM_MODE_FORW;
     __pmContext		*ctxp;
+    pmResult		*raw_result;
+    pmResult		*skel_result = NULL;
     pmResult		*result;
     struct timeval	done;
 
@@ -718,6 +750,10 @@ main(int argc, char *argv[])
 	    mflag = 1;
 	    break;
 
+	case 'M':	/* report <mark> records */
+	    Mflag = 1;
+	    break;
+
 	case 'r':	/* read log in reverse chornological order */
 	    mode = PM_MODE_BACK;
 	    break;
@@ -736,16 +772,19 @@ main(int argc, char *argv[])
 	    break;
 
 	case 'x':	/* report Ddd Mmm DD <timestamp> YYYY */
-	    xflag = 1;
+			/* -xx reports numeric timeval also */
+	    xflag++;
 	    break;
 	}
     }
 
     if (opts.errors ||
+	(opts.flags & PM_OPTFLAG_EXIT) ||
 	(vflag && opts.optind != argc) ||
-	(!vflag && opts.optind > argc - 1)) {
+	(!vflag && opts.optind > argc - 1 && !opts.narchives)) {
+	sts = !(opts.flags & PM_OPTFLAG_EXIT);
 	pmUsageMessage(&opts);
-	exit(1);
+	exit(sts);
     }
 
     if (vflag) {
@@ -763,13 +802,21 @@ main(int argc, char *argv[])
 	mflag = 1;	/* default */
 
     /* delay option end processing until now that we have the archive name */
-    __pmAddOptArchive(&opts, argv[opts.optind]);
+    if (opts.narchives == 0)
+	__pmAddOptArchive(&opts, argv[opts.optind++]);
     opts.flags &= ~PM_OPTFLAG_DONE;
     __pmEndOptions(&opts);
 
+    /* For now, ensure that we have only a single archive. */
+    if (!isSingleArchive(opts.archives[0])) {
+	fprintf(stderr, "%s: Multiple archives are not supported\n",
+		pmProgname);
+	exit(1);
+    }
+
     if ((sts = ctxid = pmNewContext(PM_CONTEXT_ARCHIVE, opts.archives[0])) < 0) {
 	fprintf(stderr, "%s: Cannot open archive \"%s\": %s\n",
-		pmProgname, argv[opts.optind], pmErrStr(sts));
+		pmProgname, opts.archives[0], pmErrStr(sts));
 	exit(1);
     }
     /* complete TZ and time window option (origin) setup */
@@ -778,7 +825,6 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
-    opts.optind++;
     numpmid = argc - opts.optind;
     if (numpmid) {
 	numpmid = 0;
@@ -806,19 +852,37 @@ main(int argc, char *argv[])
 	}
     }
 
-    /*
-     * Note: ctxp->c_lock remains locked throughout ... __pmHandleToPtr()
-     *       is only called once, and a single context is used throughout
-     *       ... so there is no PM_UNLOCK(ctxp->c_lock) anywhere in the
-     *       pmdumplog code.
-     *       This works because ctxp->c_lock is a recursive lock and
-     *       pmdumplog is single-threaded.
-     */
+    if ((sts = pmGetArchiveLabel(&label)) < 0) {
+	fprintf(stderr, "%s: Cannot get archive label record: %s\n",
+		pmProgname, pmErrStr(sts));
+	exit(1);
+    }
+
+    if (numpmid > 0) {
+	/*
+	 * setup dummy pmResult
+	 */
+	skel_result = (pmResult *)malloc(sizeof(pmResult)+(numpmid-1)*sizeof(pmValueSet *));
+	if (skel_result == NULL) {
+	    fprintf(stderr, "%s: malloc(skel_result): %s\n", pmProgname, osstrerror());
+	    exit(1);
+
+	}
+    }
+
     if ((ctxp = __pmHandleToPtr(ctxid)) == NULL) {
 	fprintf(stderr, "%s: botch: __pmHandleToPtr(%d) returns NULL!\n",
 		pmProgname, ctxid);
 	exit(1);
     }
+    /*
+     * Note: This application is single threaded, and once we have ctxp
+     *	     the associated __pmContext will not move and will only be
+     *	     accessed or modified synchronously either here or in libpcp.
+     *	     We unlock the context so that it can be locked as required
+     *	     within libpcp.
+     */
+    PM_UNLOCK(ctxp->c_lock);
 
     pmSetMode(mode, &opts.start, 0);
 
@@ -871,16 +935,60 @@ main(int argc, char *argv[])
 	}
 	sts = 0;
 	for ( ; ; ) {
-	    if (numpmid == 0)
-		sts = pmFetchArchive(&result);
-	    else
-		sts = pmFetch(numpmid, pmid, &result);
+	    sts = __pmLogFetch(ctxp, 0, NULL, &raw_result);
 	    if (sts < 0)
 		break;
+	    if (numpmid == 0 || (raw_result->numpmid == 0 && Mflag)) {
+		/*
+		 * want 'em all or <mark> record ...
+		 */
+		result = raw_result;
+	    }
+	    else if (numpmid > 0) {
+		/*
+		 * cherry pick from raw_result if pmid matches one
+		 * of interest
+		 */
+		int	picked = 0;
+		int	j;
+		skel_result->timestamp = raw_result->timestamp;
+		for (j = 0; j < numpmid; j++)
+		    skel_result->vset[j] = NULL;
+		for (i = 0; i < raw_result->numpmid; i++) {
+		    for (j = 0; j < numpmid; j++) {
+			if (pmid[j] == raw_result->vset[i]->pmid) {
+			    skel_result->vset[j] = raw_result->vset[i];
+			    picked++;
+			    break;
+			}
+		    }
+		}
+		if (picked == 0) {
+		    /* no metrics of interest, skip this record */
+		    pmFreeResult(raw_result);
+		    continue;
+		}
+		skel_result->numpmid = picked;
+		if (picked != numpmid) {
+		    /* did not find 'em all ... shuffle time */
+		    int		j;
+		    i = 0;
+		    for (j = 0; j < numpmid; j++) {
+			if (skel_result->vset[j] != NULL)
+			    skel_result->vset[i++] = skel_result->vset[j];
+		    }
+		}
+		result = skel_result;
+	    }
+	    else {
+		/* not interesting */
+		pmFreeResult(raw_result);
+		continue;
+	    }
 	    if (first && mode == PM_MODE_BACK) {
 		first = 0;
 		printf("\nLog finished at %24.24s - dump in reverse order\n",
-			pmCtime(&result->timestamp.tv_sec, timebuf));
+			pmCtime((const time_t *)&result->timestamp.tv_sec, timebuf));
 	    }
 	    if ((mode == PM_MODE_FORW && tvcmp(result->timestamp, done) > 0) ||
 		(mode == PM_MODE_BACK && tvcmp(result->timestamp, done) < 0)) {
@@ -889,7 +997,7 @@ main(int argc, char *argv[])
 	    }
 	    putchar('\n');
 	    dump_result(result);
-	    pmFreeResult(result);
+	    pmFreeResult(raw_result);
 	}
 	if (sts != PM_ERR_EOL) {
 	    fprintf(stderr, "%s: pmFetch: %s\n", pmProgname, pmErrStr(sts));
