@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2014 Red Hat.
+ * Copyright (c) 2013-2015 Red Hat.
  * Copyright (c) 1995 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -62,7 +62,7 @@ typedef unsigned int	TickTime;	/* time counted in deltas */
 typedef double		RealTime;	/* wall clock time or interval */
 #define MINUTE		60		/* one minute of real time */
 #define DELAY_MAX	32		/* maximum initial evaluation delay */
-#define RETRY		5		/* retry interval */
+#define RETRY		5		/* initial retry interval */
 #define DELTA_DFLT	10		/* default sample interval */
 #define DELTA_MIN	0.1		/* minimum sample interval */
 
@@ -137,7 +137,8 @@ typedef struct metric {
     struct metric   *next;	/* fetch/wait list forward pointer */
     struct metric   *prev;	/* fetch/wait list backward pointer */
     Symbol	    mname;	/* metric name */
-    Symbol	    hname;	/* host name */
+    Symbol	    hconn;	/* host connection string (hostspec or archive filename) */
+    Symbol	    hname;	/* (derived) host name */
     pmDesc          desc;       /* pmAPI metric description */
     double	    conv;	/* conversion factor into canonical units */
     int		    specinst;	/* count of specific instances in rule and */
@@ -194,6 +195,7 @@ typedef struct host {
     struct host	    *next;	/* Host list forward pointer */
     struct host	    *prev;	/* Host list backward pointer */
     Symbol          name;       /* host machine */
+    Symbol          conn;       /* host machine connection */
     int	    	    down;	/* host is not delivering metrics */
     Metric	    *waits;	/* wait list of Metrics */
     Metric          *duds;	/* bad Metrics discovered during evaluation */
@@ -208,7 +210,7 @@ typedef struct task {
     RealTime      delta;	/* sample interval */
     TickTime      tick;		/* count up deltas */
     RealTime      eval;		/* scheduled evaluation time */
-    RealTime	  retry;	/* scheduled retry down Hosts and Metrics */
+    int		  retry;	/* delta for retry down Hosts and Metrics */
     int		  nrules;	/* number of rules in this task */
     Symbol	  *rules;	/* array of rules to be evaluated */
     Host          *hosts;	/* fetches to be executed and waiting */
@@ -252,7 +254,9 @@ typedef int Op;
 #define CND_MIN_TIME	21
 /* relational */
 #define CND_EQ		30
+#define CND_EQ_STR	62
 #define CND_NEQ		31
+#define CND_NEQ_STR	63
 #define CND_LT		32
 #define CND_LTE		33
 #define CND_GT		34
@@ -318,12 +322,12 @@ void *alloc(size_t);
 void *zalloc(size_t);
 void *ralloc(void *, size_t);
 void *aalloc(size_t, size_t);
-char *sdup(char *);
+char *sdup(const char *);
 
 Expr *newExpr(int, Expr *, Expr *, int, int, int, int, int);
 Profile *newProfile(Fetch *, pmInDom);
 Fetch *newFetch(Host *);
-Host *newHost(Task *, Symbol);
+Host *newHost(Task *, Symbol, Symbol);
 Task *newTask(RealTime, int);
 void newResult(Task *);
 
@@ -352,6 +356,7 @@ void rotate(Expr *);
 Expr *primary(Expr *, Expr *);
 void changeSmpls(Expr **, int);
 void instFetchExpr(Expr *);
+char *getStringValue(Expr *, int);
 
 /***********************************************************************
  * time methods
@@ -359,9 +364,7 @@ void instFetchExpr(Expr *);
 
 RealTime getReal(void);			/* return current time */
 void reflectTime(RealTime);		/* update time vars to reflect now */
-#define SLEEP_EVAL	0
-#define SLEEP_RETRY	1
-void sleepTight(Task *, int);		/* sleep until retry or eval time */
+void sleepTight(Task *);		/* sleep until eval time */
 void logRotate(void);			/* close current, start a new log */
 
 /*
@@ -395,8 +398,7 @@ extern char        *pmnsfile;	/* alternate namespace */
 extern Archive	   *archives;	/* archives given on command line */
 extern RealTime	   first;	/* archive starting point */
 extern RealTime	   last;	/* archive end point */
-extern char	   *dfltHostConn;  /* default PM_CONTEXT_HOST parameter  */
-extern char	   *dfltHostName;  /* pmContextGetHostName of host name */
+extern char	   *dfltHostConn;  /* host connspec or archive path  */
 extern RealTime	   dfltDelta;	/* default sample interval */
 extern RealTime    runTime;	/* run time interval */
 extern int	   hostZone;	/* timezone from host? */
@@ -410,6 +412,7 @@ extern int         applet;	/* applet mode? */
 extern int	   dowrap;	/* counter wrap? default no */
 extern int	   doexit;	/* signalled its time to exit */
 extern int	   dorotate;	/* log rotation was requested */
+extern int	   inrun;	/* parsing done, in run() */
 extern pmiestats_t *perf;	/* pmie performance data ptr */
 extern pmiestats_t instrument;	/* pmie performance data struct */
 
