@@ -3,6 +3,7 @@
  ***********************************************************************
  *
  * Copyright (c) 1995-2002 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2015 Red Hat
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -35,8 +36,6 @@ static void
 enque(Task *t)
 {
     Task	*q;
-    RealTime	tt;
-    RealTime	qt;
 
     q = taskq;
     if (q == NULL) {
@@ -45,10 +44,8 @@ enque(Task *t)
 	t->prev = NULL;
     }
     else {
-	tt = (t->retry && t->retry < t->eval) ? t->retry : t->eval;
 	while (q) {
-	    qt = (q->retry && q->retry < q->eval) ? q->retry : q->eval;
-	    if (tt <= qt) {
+	    if (t->eval <= q->eval) {
 		t->next = q;
 		t->prev = q->prev;
 		if (q->prev) q->prev->next = t;
@@ -104,7 +101,7 @@ typedef struct hstate {
 static hstate_t	*host_map = NULL;
 
 int
-host_state_changed(char *host, int state)
+host_state_changed(const char *host, int state) /* NB: host == connection string */
 {
     hstate_t	*hsp;
 
@@ -126,11 +123,11 @@ host_state_changed(char *host, int state)
     if (quiet)
 	; /* be quiet */
     else if (state == STATE_FAILINIT)
-	__pmNotifyErr(LOG_INFO, "Cannot connect to pmcd on host %s\n", host);
+	__pmNotifyErr(LOG_INFO, "Cannot connect to pmcd %s\n", host);
     else if (state == STATE_RECONN && hsp->state != STATE_INIT)
-	__pmNotifyErr(LOG_INFO, "Re-established connection to pmcd on host %s\n", host);
+	__pmNotifyErr(LOG_INFO, "Re-established connection to pmcd %s\n", host);
     else if (state == STATE_LOSTCONN)
-	__pmNotifyErr(LOG_INFO, "Lost connection to pmcd on host %s\n", host);
+	__pmNotifyErr(LOG_INFO, "Lost connection to pmcd %s\n", host);
 
     hsp->state = state;
     return 1;
@@ -151,7 +148,7 @@ enable(Task *t)
 	if (h->down) {
 	    if (reconnect(h)) {
 		h->down = 0;
-		host_state_changed(symName(h->name), STATE_RECONN);
+		host_state_changed(symName(h->conn), STATE_RECONN);
 	    }
 	}
 
@@ -181,6 +178,12 @@ enable(Task *t)
 
 	h = h->next;
     }
+
+    if (waiting(t) == 0) {
+	/* all clear now ... */
+	t->epoch = t->eval;
+	t->retry = 0;
+    }
 }
 
 
@@ -198,6 +201,13 @@ eval(Task *task)
     pmValueSet  *vset;
     int		i;
 
+#if PCP_DEBUG
+    if (pmDebug & DBG_TRACE_APPL2) {
+	fprintf(stderr, "Evaluating task:\n");
+	dumpTask(task);
+    }
+#endif
+
     /* fetch metrics */
     taskFetch(task);
 
@@ -323,19 +333,31 @@ findEval(Expr *x)
      *	1	arg1 has tspan 1, and must always have one metric value
      *	2	arg2 has tspan 1, and must always have one metric value
      */
-    if (x->arg1 && x->arg1->tspan == 1) {
-	for (m = x->arg1->metrics; m; m = m->next) {
-	    if (m->desc.indom == PM_INDOM_NULL) continue;
-	    if (m->specinst == 0) break;
+    if (x->arg1) {
+	if (x->arg1->tspan == 1) {
+	    for (m = x->arg1->metrics; m; m = m->next) {
+		if (m->desc.indom == PM_INDOM_NULL) continue;
+		if (m->specinst == 0) break;
+	    }
+	    if (m == NULL) arity |= 1;
+	}
+	else if (x->arg1->sem == SEM_CHAR) {
+	    /* string-valued constant */
+	    arity |= 1;
 	}
-	if (m == NULL) arity |= 1;
     }
-    if (x->arg2 && x->arg2->tspan == 1) {
-	for (m = x->arg2->metrics; m; m = m->next) {
-	    if (m->desc.indom == PM_INDOM_NULL) continue;
-	    if (m->specinst == 0) break;
+    if (x->arg2) {
+	if (x->arg2->tspan == 1) {
+	    for (m = x->arg2->metrics; m; m = m->next) {
+		if (m->desc.indom == PM_INDOM_NULL) continue;
+		if (m->specinst == 0) break;
+	    }
+	    if (m == NULL) arity |= 2;
+	}
+	else if (x->arg2->sem == SEM_CHAR) {
+	    /* string-valued constant */
+	    arity |= 2;
 	}
-	if (m == NULL) arity |= 2;
     }
 
     /*
@@ -613,6 +635,21 @@ findEval(Expr *x)
 	}
 	break;
 
+    case CND_EQ_STR:
+	if (arity & 1) {
+	    if (arity & 2)
+		x->eval = cndEqStr_1_1;
+	    else
+		x->eval = cndEqStr_1_n;
+	}
+	else {
+	    if (arity & 2)
+		x->eval = cndEqStr_n_1;
+	    else
+		x->eval = cndEqStr_n_n;
+	}
+	break;
+
     case CND_NEQ:
 	if (arity & 1) {
 	    if (arity & 2)
@@ -628,6 +665,21 @@ findEval(Expr *x)
 	}
 	break;
 
+    case CND_NEQ_STR:
+	if (arity & 1) {
+	    if (arity & 2)
+		x->eval = cndNeqStr_1_1;
+	    else
+		x->eval = cndNeqStr_1_n;
+	}
+	else {
+	    if (arity & 2)
+		x->eval = cndNeqStr_n_1;
+	    else
+		x->eval = cndNeqStr_n_n;
+	}
+	break;
+
     case CND_LT:
 	if (arity & 1) {
 	    if (arity & 2)
@@ -753,11 +805,16 @@ run(void)
     if (taskq == NULL)
 	return;
 
+    inrun = 1;
+
     /* initialize task scheduling */
     t = taskq;
     while (t) {
 	t->eval = t->epoch = start;
-	t->retry = 0;
+	if (waiting(t))
+	    t->retry = RETRY;
+	else
+	    t->retry = 0;
 	t->tick = 0;
 	t = t->next;
     }
@@ -765,25 +822,33 @@ run(void)
     /* evaluate and reschedule */
     t = taskq;
     for (;;) {
-	if (t->retry && t->retry < t->eval) {
-	    now = t->retry;
-	    if (now > stop)
-		break;
-	    sleepTight(t, SLEEP_RETRY);
+	now = t->eval;
+	if (now > stop)
+	    break;
+	sleepTight(t);
+	if (t->retry)
 	    enable(t);
-	    t->retry = waiting(t) ? now + RETRY : 0;
+	reflectTime(t->delta);
+	eval(t);
+	if (waiting(t) && t->retry == 0) {
+	    /* just failed host or metric availability */
+	    t->retry = RETRY;
+	}
+	if (t->retry > 0) {
+	    if (t->retry < t->delta) {
+		/* exponential back-off, ... */
+		t->eval = now + t->retry;
+		t->retry *= 2;
+	    }
+	    else {
+		/* ... capped at delta */
+		t->eval = now + t->delta;
+	    }
 	}
 	else {
-	    now = t->eval;
-	    if (now > stop)
-		break;
-	    reflectTime(t->delta);
-	    sleepTight(t, SLEEP_EVAL);
-	    eval(t);
+	    /* regular eval, host and metrics available */
 	    t->tick++;
 	    t->eval = t->epoch + t->tick * t->delta;
-	    if ((! t->retry) && waiting(t))
-		t->retry = now + RETRY;
 	}
 	taskq = t->next;
 	if (taskq) taskq->prev = NULL;
