@@ -1,7 +1,7 @@
 /*
  * pmlogextract - extract desired metrics from PCP archive logs
  *
- * Copyright (c) 2014 Red Hat.
+ * Copyright (c) 2014,2017 Red Hat.
  * Copyright (c) 1997-2002 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -50,18 +50,172 @@ static pmOptions opts = {
     .short_usage = "[options] input-archive output-archive",
 };
 
-const char *
-metricname(pmID pmid)
+/*
+ * extract metric name(s) from metadata pdu buffer
+ */
+void
+printmetricnames(FILE *f, __pmPDU *pdubuf)
 {
-    static char	*name = NULL;
-    if (name != NULL) {
-	free(name);
-	name = NULL;
+    if (ntohl(pdubuf[0]) > 8) {
+	/*
+	 * have at least one name ... names are packed
+	 * <len><name>... at the end of the buffer
+	 */
+	int	numnames = ntohl(pdubuf[7]);
+	char	*p = (char *)&pdubuf[8];
+	int	i;
+	__pmPDU	len;
+	for (i = 0; i < numnames; i++) {
+	    memmove((void *)&len, (void *)p, sizeof(__pmPDU));
+	    len = ntohl(len);
+	    p += sizeof(__pmPDU);
+	    if (i > 0) fprintf(f, ", ");
+	    fprintf(f, "%*.*s", len, len, p);
+	    p += len;
+	}
+    }
+    else
+	fprintf(f, "<noname>");
+}
+
+#define MATCH_NONE	0
+#define MATCH_SUBSET	1
+#define MATCH_SUPERSET	2
+#define MATCH_SOME	3
+#define MATCH_EQUAL	4
+/*
+ * compare sets of metric name(s) from two metadata pdu buffers
+ *
+ * returns MATCH_NONE if no intersection
+ * returns MATCH_SUBSET if {a names} is a proper subset of {b names}
+ * returns MATCH_SUPERSET if {a names} is a proper superset of {b names}
+ * returns MATCH_SOME if the intersection of {a names} and {b names} is
+ * not empty
+ * returns MATCH_EQUAL {a names} is identical to {b names}
+ */
+int
+matchnames(__pmPDU *a, __pmPDU *b)
+{
+    int		num_a = 0;
+    int		num_b = 0;
+    int		num_a_eq = 0;
+    int		num_b_eq = 0;
+    int		i_a;
+    int		i_b;
+    char	*p_a;
+    char	*p_b;
+    int		sts;
+
+    if (ntohl(a[0]) > 8) num_a = ntohl(a[7]);
+    if (ntohl(b[0]) > 8) num_b = ntohl(b[7]);
+
+    /*
+     * count number of names from {a names} that match names in {b names}
+     */
+    p_a = (char *)&a[8];
+    for (i_a = 0; i_a < num_a; i_a++) {
+	__pmPDU		len_a;
+
+	memmove((void *)&len_a, (void *)p_a, sizeof(__pmPDU));
+	len_a = ntohl(len_a);
+	p_a += sizeof(__pmPDU);
+	p_b = (char *)&b[8];
+	for (i_b = 0; i_b < num_b; i_b++) {
+	    __pmPDU		len_b;
+	    memmove((void *)&len_b, (void *)p_b, sizeof(__pmPDU));
+	    len_b = ntohl(len_b);
+	    p_b += sizeof(__pmPDU);
+
+	    if (len_a == len_b && strncmp(p_a, p_b, len_a) == 0) {
+		num_a_eq++;
+		break;
+	    }
+
+	    p_b += len_b;
+	}
+	p_a += len_a;
+    }
+
+
+    /*
+     * count number of names from {b names} that match names in {a names}
+     * ... this would not be necessary, but due to an apparent pmlogger
+     * bug, the _same_ name can appear in either or both of the {a names}
+     * and {b names} sets, which complicates the logic here.
+     */
+    p_b = (char *)&b[8];
+    for (i_b = 0; i_b < num_b; i_b++) {
+	__pmPDU		len_b;
+
+	memmove((void *)&len_b, (void *)p_b, sizeof(__pmPDU));
+	len_b = ntohl(len_b);
+	p_b += sizeof(__pmPDU);
+	p_a = (char *)&a[8];
+	for (i_a = 0; i_a < num_a; i_a++) {
+	    __pmPDU		len_a;
+	    memmove((void *)&len_a, (void *)p_a, sizeof(__pmPDU));
+	    len_a = ntohl(len_a);
+	    p_a += sizeof(__pmPDU);
+
+	    if (len_b == len_a && strncmp(p_b, p_a, len_b) == 0) {
+		num_b_eq++;
+		break;
+	    }
+
+	    p_a += len_a;
+	}
+	p_b += len_b;
+    }
+
+    if (num_a_eq == 0 && num_b_eq == 0)
+	sts = MATCH_NONE;
+    else if (num_a && num_a_eq == num_a && num_b  && num_b_eq == num_b) {
+	/* all names match but more occurrences in one log (i.e. dups) */
+	sts = MATCH_EQUAL;
+    }
+    else if (num_a == num_b) {
+	if (num_a_eq == num_a && num_b_eq == num_b) sts = MATCH_EQUAL;
+	else sts = MATCH_SOME;
+    }
+    else if (num_a > num_b) {
+	if (num_b_eq == num_b) sts = MATCH_SUPERSET;
+	else sts = MATCH_SOME;
+    }
+    else {
+	/* num_a < num_b */
+	if (num_a_eq == num_a) sts = MATCH_SUBSET;
+	else sts = MATCH_SOME;
+    }
+#ifdef PCP_DEBUG
+    if (pmDebug & DBG_TRACE_APPL1) {
+	fprintf(stderr, "matchnames: ");
+	printmetricnames(stderr, a);
+	fprintf(stderr, " : ");
+	printmetricnames(stderr, b);
+	fprintf(stderr, " num_a=%d num_b=%d num_a_eq=%d num_b_eq=%d -> %d\n", num_a, num_b, num_a_eq, num_b_eq, sts);
+    }
+#endif
+
+    return sts;
+}
+
+void
+printsem(FILE *f, int sem)
+{
+    switch (sem) {
+	case PM_SEM_COUNTER:
+	    fprintf(f, "counter");
+	    break;
+	case PM_SEM_INSTANT:
+	    fprintf(f, "instant");
+	    break;
+	case PM_SEM_DISCRETE:
+	    fprintf(f, "discrete");
+	    break;
+	default:
+	    fprintf(f, "unknown (%d)", sem);
+	    break;
     }
-    if (pmNameID(pmid, &name) == 0)
-	return(name);
-    name = NULL;
-    return pmIDStr(pmid);
 }
 
 /*
@@ -96,17 +250,6 @@ typedef struct {
     __pmValue_PDU	vlist[1];	/* zero or more */
 } vlist_t;
 
-/*
- *  This struct is not needed?
- */
-typedef struct {
-    /* __pmPDUHdr		hdr; */
-    __pmPDU		hdr;
-    __pmTimeval		timestamp;	/* when returned */
-    int			numpmid;	/* no. of PMIDs to follow */
-    __pmPDU		data[1];	/* zero or more */
-} result_t;
-
 /*
  *  Mark record
  */
@@ -203,8 +346,8 @@ tvcmp(__pmTimeval a, __pmTimeval b)
     return 0;
 }
 
-static void
-abandon()
+void
+abandon_extract(void)
 {
     char    fname[MAXNAMELEN];
     if (desperate == 0) {
@@ -227,15 +370,15 @@ abandon()
  *  report that archive is corrupted
  */
 static void
-_report(FILE *fp)
+_report(__pmFILE *fp)
 {
     off_t	here;
     struct stat	sbuf;
 
-    here = lseek(fileno(fp), 0L, SEEK_CUR);
+    here = __pmLseek(fp, 0L, SEEK_CUR);
     fprintf(stderr, "%s: Error occurred at byte offset %ld into a file of",
 	    pmProgname, (long int)here);
-    if (fstat(fileno(fp), &sbuf) < 0)
+    if (__pmFstat(fp, &sbuf) < 0)
 	fprintf(stderr, ": stat: %s\n", osstrerror());
     else
 	fprintf(stderr, " %ld bytes.\n", (long int)sbuf.st_size);
@@ -249,16 +392,16 @@ _report(FILE *fp)
 static void
 newvolume(char *base, __pmTimeval *tvp)
 {
-    FILE		*newfp;
+    __pmFILE		*newfp;
     int			nextvol = logctl.l_curvol + 1;
 
     if ((newfp = __pmLogNewFile(base, nextvol)) != NULL) {
 	struct timeval	stamp;
-	fclose(logctl.l_mfp);
+	__pmFclose(logctl.l_mfp);
 	logctl.l_mfp = newfp;
 	logctl.l_label.ill_vol = logctl.l_curvol = nextvol;
 	__pmLogWriteLabel(logctl.l_mfp, &logctl.l_label);
-	fflush(logctl.l_mfp);
+	__pmFflush(logctl.l_mfp);
 	stamp.tv_sec = ntohl(tvp->tv_sec);
 	stamp.tv_usec = ntohl(tvp->tv_usec);
 	fprintf(stderr, "%s: New log volume %d, at ", pmProgname, nextvol);
@@ -268,7 +411,7 @@ newvolume(char *base, __pmTimeval *tvp)
     else {
 	fprintf(stderr, "%s: Error: volume %d: %s\n",
 		pmProgname, nextvol, pmErrStr(-oserror()));
-	abandon();
+	abandon_extract();
     }
     flushsize = 100000;
 }
@@ -295,7 +438,7 @@ newlabel(void)
     if (inarchvers != PM_LOG_VERS02) {
 	fprintf(stderr,"%s: Error: illegal version number %d in archive (%s)\n",
 		pmProgname, inarchvers, iap->name);
-	abandon();
+	abandon_extract();
     }
 
     /* copy magic number, pid, host and timezone */
@@ -328,7 +471,7 @@ newlabel(void)
 		"archive: %s version: %d\n",
 		    pmProgname, inarch[0].name, inarchvers,
 		    iap->name, (iap->label.ll_magic & 0xff));
-	    abandon();
+	    abandon_extract();
         }
 
 	/* Ensure all archives of the same host */
@@ -339,7 +482,7 @@ newlabel(void)
 		    inarch[0].name, inarch[0].label.ll_hostname);
 	    fprintf(stderr, "archive: %s host: %s\n",
 		    iap->name, iap->label.ll_hostname);
-	    abandon();
+	    abandon_extract();
 	}
 
 	/* Ensure all archives of the same timezone */
@@ -370,11 +513,11 @@ newlabel(void)
 void
 writelabel_metati(int do_rewind)
 {
-    if (do_rewind) rewind(logctl.l_tifp);
+    if (do_rewind) __pmRewind(logctl.l_tifp);
     logctl.l_label.ill_vol = PM_LOG_VOL_TI;
     __pmLogWriteLabel(logctl.l_tifp, &logctl.l_label);
 
-    if (do_rewind) rewind(logctl.l_mdfp);
+    if (do_rewind) __pmRewind(logctl.l_mdfp);
     logctl.l_label.ill_vol = PM_LOG_VOL_META;
     __pmLogWriteLabel(logctl.l_mdfp, &logctl.l_label);
 }
@@ -404,7 +547,7 @@ mk_reclist_t(void)
     if ((rec = (reclist_t *)malloc(sizeof(reclist_t))) == NULL) {
 	fprintf(stderr, "%s: Error: cannot malloc space for record list.\n",
 		pmProgname);
-	abandon();
+	abandon_extract();
     }
 #ifdef PCP_DEBUG
     if (pmDebug & DBG_TRACE_APPL0) {
@@ -527,27 +670,47 @@ update_descreclist(int i)
     else {
 	curr = rdesc;
 	/* find matching record or last record */
-	while (curr->next != NULL && curr->desc.pmid != ntoh_pmID(iap->pb[META][2]))
-	    curr = curr->next;
-
 #ifdef PCP_DEBUG
 	if (pmDebug & DBG_TRACE_APPL1) {
-	    fprintf(stderr, "update_descreclist: pmid: last/match %s", metricname(curr->desc.pmid));
-	    fprintf(stderr, " new %s", metricname(ntoh_pmID(iap->pb[META][2])));
-	    fputc('\n', stderr);
+	    fprintf(stderr, "update_descreclist: looking for ");
+	    printmetricnames(stderr, iap->pb[META]);
+	    fprintf(stderr, " (pmid:%s)\n", pmIDStr(ntoh_pmID(iap->pb[META][2])));
 	}
 #endif
+	while (curr->next != NULL && curr->desc.pmid != ntoh_pmID(iap->pb[META][2])) {
+	    if (curr->pdu != NULL) {
+		if (matchnames(curr->pdu, iap->pb[META]) != MATCH_NONE) {
+		    fprintf(stderr, "%s: Error: metric ", pmProgname);
+		    printmetricnames(stderr, curr->pdu);
+		    fprintf(stderr, ": PMID changed from %s", pmIDStr(curr->desc.pmid));
+		    fprintf(stderr, " to %s!\n", pmIDStr(ntoh_pmID(iap->pb[META][2])));
+		    abandon_extract();
+		}
+#ifdef PCP_DEBUG
+		if (pmDebug & DBG_TRACE_APPL1) {
+		    fprintf(stderr, "update_descreclist: nomatch ");
+		    printmetricnames(stderr, curr->pdu);
+		    fprintf(stderr, " (pmid:%s)\n", pmIDStr(curr->desc.pmid));
+		}
+#endif
+	    }
+	    curr = curr->next;
+	}
     }
 
     if (curr->desc.pmid == ntoh_pmID(iap->pb[META][2])) {
 #ifdef PCP_DEBUG
 	if (pmDebug & DBG_TRACE_APPL1) {
+	    fprintf(stderr, "update_descreclist: pmid match ");
+	    printmetricnames(stderr, curr->pdu);
 	    fprintf(stderr, " type: old %s", pmTypeStr(curr->desc.type));
 	    fprintf(stderr, " new %s", pmTypeStr(ntohl(iap->pb[META][3])));
 	    fprintf(stderr, " indom: old %s", pmInDomStr(curr->desc.indom));
 	    fprintf(stderr, " new %s", pmInDomStr(ntoh_pmInDom(iap->pb[META][4])));
-	    fprintf(stderr, " sem: old %d", curr->desc.sem);
-	    fprintf(stderr, " new %d", (int)ntohl(iap->pb[META][5]));
+	    fprintf(stderr, " sem: old ");
+	    printsem(stderr, curr->desc.sem);
+	    fprintf(stderr, " new ");
+	    printsem(stderr, (int)ntohl(iap->pb[META][5]));
 	    fprintf(stderr, " units: old %s", pmUnitsStr(&curr->desc.units));
 	    pmup = (pmUnits *)&iap->pb[META][6];
 	    pmu = ntoh_pmUnits(*pmup);
@@ -555,29 +718,41 @@ update_descreclist(int i)
 	    fputc('\n', stderr);
 	}
 #endif
+	if (matchnames(curr->pdu, iap->pb[META]) != MATCH_EQUAL) {
+	    fprintf(stderr, "%s: Error: metric PMID %s", pmProgname, pmIDStr(curr->desc.pmid));
+	    fprintf(stderr, ": name changed from ");
+	    printmetricnames(stderr, curr->pdu);
+	    fprintf(stderr, " to ");
+	    printmetricnames(stderr, iap->pb[META]);
+	    fprintf(stderr, "!\n");
+	    abandon_extract();
+	}
 	if (curr->desc.type != ntohl(iap->pb[META][3])) {
-	    fprintf(stderr,
-		"%s: Error: metric %s: type changed from",
-		pmProgname, metricname(curr->desc.pmid));
+	    fprintf(stderr, "%s: Error: metric ", pmProgname);
+	    printmetricnames(stderr, curr->pdu);
+	    fprintf(stderr, ": type changed from");
 	    fprintf(stderr, " %s", pmTypeStr(curr->desc.type));
 	    fprintf(stderr, " to %s!\n", pmTypeStr(ntohl(iap->pb[META][3])));
-	    abandon();
+	    abandon_extract();
 	}
 	if (curr->desc.indom != ntoh_pmInDom(iap->pb[META][4])) {
-	    fprintf(stderr,
-		"%s: Error: metric %s: indom changed from",
-		pmProgname, metricname(curr->desc.pmid));
+	    fprintf(stderr, "%s: Error: metric ", pmProgname);
+	    printmetricnames(stderr, curr->pdu);
+	    fprintf(stderr, ": indom changed from");
 	    fprintf(stderr, " %s", pmInDomStr(curr->desc.indom));
 	    fprintf(stderr, " to %s!\n", pmInDomStr(ntoh_pmInDom(iap->pb[META][4])));
-	    abandon();
+	    abandon_extract();
 	}
 	if (curr->desc.sem != ntohl(iap->pb[META][5])) {
-	    fprintf(stderr,
-		"%s: Error: metric %s: semantics changed from",
-		pmProgname, metricname(curr->desc.pmid));
-	    fprintf(stderr, " %d", curr->desc.sem);
-	    fprintf(stderr, " to %d!\n", (int)ntohl(iap->pb[META][5]));
-	    abandon();
+	    fprintf(stderr, "%s: Error: metric ", pmProgname);
+	    printmetricnames(stderr, curr->pdu);
+	    fprintf(stderr, ": semantics changed from");
+	    fprintf(stderr, " ");
+	    printsem(stderr, curr->desc.sem);
+	    fprintf(stderr, " to ");
+	    printsem(stderr, (int)ntohl(iap->pb[META][5]));
+	    fprintf(stderr, "!\n");
+	    abandon_extract();
 	}
 	pmup = (pmUnits *)&iap->pb[META][6];
 	pmu = ntoh_pmUnits(*pmup);
@@ -587,12 +762,12 @@ update_descreclist(int i)
 	    curr->desc.units.scaleSpace != pmu.scaleSpace ||
 	    curr->desc.units.scaleTime != pmu.scaleTime ||
 	    curr->desc.units.scaleCount != pmu.scaleCount) {
-	    fprintf(stderr,
-		"%s: Error: metric %s: units changed from",
-		pmProgname, metricname(curr->desc.pmid));
+	    fprintf(stderr, "%s: Error: metric ", pmProgname);
+	    printmetricnames(stderr, curr->pdu);
+	    fprintf(stderr, ": units changed from");
 	    fprintf(stderr, " %s", pmUnitsStr(&curr->desc.units));
 	    fprintf(stderr, " to %s!\n", pmUnitsStr(&pmu));
-	    abandon();
+	    abandon_extract();
 	}
 	/* not adding, so META: discard new record */
 	free(iap->pb[META]);
@@ -697,7 +872,7 @@ write_rec(reclist_t *rec)
 	if (rec->pdu == NULL) {
 	    fprintf(stderr, "%s: Fatal Error!\n", pmProgname);
 	    fprintf(stderr,"    record is marked for write, but pdu is NULL\n");
-	    abandon();
+	    abandon_extract();
 	}
 
 #ifdef PCP_DEBUG
@@ -709,7 +884,7 @@ write_rec(reclist_t *rec)
 	    len = ntohl(h->len);
 	    type = ntohl(h->type);
 	    fprintf(stderr, "write_rec: record len=%d, type=%d @ offset=%d\n",
-	    	len, type, (int)(ftell(logctl.l_mdfp) - sizeof(__pmLogHdr)));
+	    	len, type, (int)(__pmFtell(logctl.l_mdfp) - sizeof(__pmLogHdr)));
 	    if (type == TYPE_DESC) {
 		pmDesc	*dp;
 		pmDesc	desc;
@@ -765,7 +940,7 @@ write_rec(reclist_t *rec)
 	if ((sts = _pmLogPut(logctl.l_mdfp, rec->pdu)) < 0) {
 	    fprintf(stderr, "%s: Error: _pmLogPut: meta data : %s\n",
 		    pmProgname, pmErrStr(sts));
-	    abandon();
+	    abandon_extract();
 	}
 	/* META: free PDU buffer */
 	free(rec->pdu);
@@ -809,7 +984,7 @@ write_metareclist(pmResult *result, int *needti)
 	    /* descriptor has not been found - this is bad
 	     */
 	    fprintf(stderr, "%s: Error: meta data (TYPE_DESC) for pmid %s has not been found.\n", pmProgname, pmIDStr(pmid));
-	    abandon();
+	    abandon_extract();
 	}
 	else {
 	    /* descriptor has been found
@@ -827,7 +1002,7 @@ write_metareclist(pmResult *result, int *needti)
 		 */
 		fprintf(stderr, "%s: Error: missing pdu for pmid %s\n",
 			pmProgname, pmIDStr(pmid));
-	        abandon();
+	        abandon_extract();
 	    }
 	    else {
 		/* descriptor is in list, has not been written, and has pdu
@@ -906,7 +1081,7 @@ _createmark(void)
     if (markp == NULL) {
 	fprintf(stderr, "%s: Error: mark_t malloc: %s\n",
 		pmProgname, osstrerror());
-	abandon();
+	abandon_extract();
     }
 #ifdef PCP_DEBUG
     if (pmDebug & DBG_TRACE_APPL0) {
@@ -973,12 +1148,14 @@ nextmeta(void)
 	if (iap->pb[META] != NULL) {
 	    fprintf(stderr, "%s: Fatal Error!\n", pmProgname);
 	    fprintf(stderr, "    iap->pb[META] is not NULL\n");
-	    abandon();
+	    abandon_extract();
 	}
 	if ((ctxp = __pmHandleToPtr(iap->ctx)) == NULL) {
 	    fprintf(stderr, "%s: botch: __pmHandleToPtr(%d) returns NULL!\n", pmProgname, iap->ctx);
-	    abandon();
+	    abandon_extract();
 	}
+	/* Need to hold c_lock for _pmLogGet() */
+
 	lcp = ctxp->c_archctl->ac_log;
 
 againmeta:
@@ -991,7 +1168,7 @@ againmeta:
 		fprintf(stderr, "%s: Error: _pmLogGet[meta %s]: %s\n",
 			pmProgname, iap->name, pmErrStr(sts));
 		_report(lcp->l_mdfp);
-		abandon();
+		abandon_extract();
 	    }
 	    PM_UNLOCK(ctxp->c_lock);
 	    continue;
@@ -1069,7 +1246,7 @@ againmeta:
 	else {
 	    fprintf(stderr, "%s: Error: unrecognised meta data type: %d\n",
 		    pmProgname, (int)ntohl(iap->pb[META][1]));
-	    abandon();
+	    abandon_extract();
 	}
 
 	PM_UNLOCK(ctxp->c_lock);
@@ -1121,18 +1298,19 @@ nextlog(void)
 
 	if ((ctxp = __pmHandleToPtr(iap->ctx)) == NULL) {
 	    fprintf(stderr, "%s: botch: __pmHandleToPtr(%d) returns NULL!\n", pmProgname, iap->ctx);
-	    abandon();
+	    abandon_extract();
 	}
+	/* Need to hold c_lock for __pmLogRead_ctx() */
 	lcp = ctxp->c_archctl->ac_log;
 
 againlog:
-	if ((sts=__pmLogRead(lcp, PM_MODE_FORW, NULL, &iap->_result, PMLOGREAD_NEXT)) < 0) {
+	if ((sts=__pmLogRead_ctx(ctxp, PM_MODE_FORW, NULL, &iap->_result, PMLOGREAD_NEXT)) < 0) {
 	    if (sts != PM_ERR_EOL) {
 		fprintf(stderr, "%s: Error: __pmLogRead[log %s]: %s\n",
 			pmProgname, iap->name, pmErrStr(sts));
 		_report(lcp->l_mfp);
 		if (sts != PM_ERR_LOGREC)
-		    abandon();
+		    abandon_extract();
 	    }
 	    /* if the first data record has not been written out, then
 	     * do not generate a mark record, and you may as well ignore
@@ -1416,7 +1594,7 @@ checkwinend(__pmTimeval now)
     if ((sts = __pmLogPutResult2(&logctl, markpdu)) < 0) {
 	fprintf(stderr, "%s: Error: __pmLogPutResult2: log data: %s\n",
 		pmProgname, pmErrStr(sts));
-	abandon();
+	abandon_extract();
     }
     written++;
     free(markpdu);
@@ -1481,7 +1659,7 @@ fprintf(stderr, " break!\n");
 	if (sts < 0) {
 	    fprintf(stderr, "%s: Error: __pmEncodeResult: %s\n",
 		    pmProgname, pmErrStr(sts));
-	    abandon();
+	    abandon_extract();
 	}
 
         /* switch volumes if required */
@@ -1494,7 +1672,7 @@ fprintf(stderr, " break!\n");
 	 * Even without a -v option, we may need to switch volumes
 	 * if the data file exceeds 2^31-1 bytes
 	 */
-	peek_offset = ftell(logctl.l_mfp);
+	peek_offset = __pmFtell(logctl.l_mfp);
 	peek_offset += ((__pmPDUHdr *)pb)->len - sizeof(__pmPDUHdr) + 2*sizeof(int);
 	if (peek_offset > 0x7fffffff) {
 	    newvolume(outarchname, (__pmTimeval *)&pb[3]);
@@ -1505,12 +1683,12 @@ fprintf(stderr, " break!\n");
 	write_metareclist(elm->res, &needti);
 
 	/* write out log record */
-	old_log_offset = ftell(logctl.l_mfp);
+	old_log_offset = __pmFtell(logctl.l_mfp);
 	assert(old_log_offset >= 0);
 	if ((sts = __pmLogPutResult2(&logctl, pb)) < 0) {
 	    fprintf(stderr, "%s: Error: __pmLogPutResult2: log data: %s\n",
 		    pmProgname, pmErrStr(sts));
-	    abandon();
+	    abandon_extract();
 	}
 	written++;
 
@@ -1518,7 +1696,7 @@ fprintf(stderr, " break!\n");
 	/* check whether we need to write TI (temporal index) */
 	if (old_log_offset == 0 ||
 	    old_log_offset == sizeof(__pmLogLabel)+2*sizeof(int) ||
-	    ftell(logctl.l_mfp) > flushsize)
+	    __pmFtell(logctl.l_mfp) > flushsize)
 		needti = 1;
 
 	/* make sure that we do not write out the temporal index more
@@ -1531,31 +1709,31 @@ fprintf(stderr, " break!\n");
 	if (needti) {
 	    titime = restime;
 
-	    fflush(logctl.l_mfp);
-	    fflush(logctl.l_mdfp);
+	    __pmFflush(logctl.l_mfp);
+	    __pmFflush(logctl.l_mdfp);
 
 	    if (old_log_offset == 0)
 		old_log_offset = sizeof(__pmLogLabel)+2*sizeof(int);
 
-            new_log_offset = ftell(logctl.l_mfp);
+            new_log_offset = __pmFtell(logctl.l_mfp);
 	    assert(new_log_offset >= 0);
-            new_meta_offset = ftell(logctl.l_mdfp);
+            new_meta_offset = __pmFtell(logctl.l_mdfp);
 	    assert(new_meta_offset >= 0);
 
-            fseek(logctl.l_mfp, (long)old_log_offset, SEEK_SET);
-            fseek(logctl.l_mdfp, (long)old_meta_offset, SEEK_SET);
+            __pmFseek(logctl.l_mfp, (long)old_log_offset, SEEK_SET);
+            __pmFseek(logctl.l_mdfp, (long)old_meta_offset, SEEK_SET);
 
             __pmLogPutIndex(&logctl, &restime);
 
-            fseek(logctl.l_mfp, (long)new_log_offset, SEEK_SET);
-            fseek(logctl.l_mdfp, (long)new_meta_offset, SEEK_SET);
+            __pmFseek(logctl.l_mfp, (long)new_log_offset, SEEK_SET);
+            __pmFseek(logctl.l_mdfp, (long)new_meta_offset, SEEK_SET);
 
-            old_log_offset = ftell(logctl.l_mfp);
+            old_log_offset = __pmFtell(logctl.l_mfp);
 	    assert(old_log_offset >= 0);
-            old_meta_offset = ftell(logctl.l_mdfp);
+            old_meta_offset = __pmFtell(logctl.l_mdfp);
 	    assert(old_meta_offset >= 0);
 
-            flushsize = ftell(logctl.l_mfp) + 100000;
+            flushsize = __pmFtell(logctl.l_mfp) + 100000;
         }
 
 	/* free PDU buffer */
@@ -1583,13 +1761,13 @@ writemark(inarch_t *iap)
     if (!iap->mark) {
 	fprintf(stderr, "%s: Fatal Error!\n", pmProgname);
 	fprintf(stderr, "    writemark called, but mark not set\n");
-	abandon();
+	abandon_extract();
     }
 
     if (p == NULL) {
 	fprintf(stderr, "%s: Fatal Error!\n", pmProgname);
 	fprintf(stderr, "    writemark called, but no pdu\n");
-	abandon();
+	abandon_extract();
     }
 
     p->timestamp.tv_sec = htonl(p->timestamp.tv_sec);
@@ -1598,7 +1776,7 @@ writemark(inarch_t *iap)
     if ((sts = __pmLogPutResult2(&logctl, iap->pb[LOG])) < 0) {
 	fprintf(stderr, "%s: Error: __pmLogPutResult2: log data: %s\n",
 		pmProgname, pmErrStr(sts));
-	abandon();
+	abandon_extract();
     }
     written++;
     free(iap->pb[LOG]);
@@ -1633,6 +1811,8 @@ main(int argc, char **argv)
     rindom = NULL;	/* list of meta indom records to write */
     rlready = NULL;
 
+    /* no derived or anon metrics, please */
+    __pmSetInternalState(PM_STATE_PMCS);
 
     /* process cmd line args */
     if (parseargs(argc, argv) < 0) {
@@ -1694,8 +1874,13 @@ main(int argc, char **argv)
 
 	if ((sts = pmGetArchiveEnd(&unused)) < 0) {
 	    fprintf(stderr, "%s: Error: cannot get end of archive (%s): %s\n",
-		    pmProgname, iap->name, pmErrStr(sts));
-	    exit(1);
+		pmProgname, iap->name, pmErrStr(sts));
+	    if (desperate) {
+		unused.tv_sec = INT_MAX;
+		unused.tv_usec = 0;
+	    }
+	    else
+		exit(1);
 	}
 
 	if (i == 0) {
@@ -1792,7 +1977,7 @@ main(int argc, char **argv)
     if (sts < 0) {
 	fprintf(stderr, "%s: Invalid time window specified: %s\n",
 		pmProgname, msg);
-	abandon();
+	abandon_extract();
     }
     winstart.tv_sec = winstart_tval.tv_sec;
     winstart.tv_usec = winstart_tval.tv_usec;
@@ -1834,7 +2019,7 @@ main(int argc, char **argv)
 	ilog = -1;
 	curlog.tv_sec = 0;
 	curlog.tv_usec = 0;
-	old_meta_offset = ftell(logctl.l_mdfp);
+	old_meta_offset = __pmFtell(logctl.l_mdfp);
 	assert(old_meta_offset >= 0);
 
 	/* nextlog() resets ilog, and curlog (to the smallest timestamp)
@@ -1900,7 +2085,7 @@ main(int argc, char **argv)
 	if (ilog < 0 || ilog >= inarchnum) {
 	    fprintf(stderr, "%s: Fatal Error!\n", pmProgname);
 	    fprintf(stderr, "    log file index = %d\n", ilog);
-	    abandon();
+	    abandon_extract();
 	}
 
 
@@ -1913,7 +2098,7 @@ main(int argc, char **argv)
 	    if (iap->_Nresult == NULL) {
 		fprintf(stderr, "%s: Fatal Error!\n", pmProgname);
 		fprintf(stderr, "    pick == LOG and _Nresult = NULL\n");
-		abandon();
+		abandon_extract();
 	    }
 	    insertresult(&rlready, iap->_Nresult);
 #if 0
@@ -1965,19 +2150,19 @@ main(int argc, char **argv)
         fprintf(stderr, "%s: Warning: no qualifying records found.\n",
                 pmProgname);
 cleanup:
-	abandon();
+	abandon_extract();
     }
     else {
 	/* write the last time stamp */
-	fflush(logctl.l_mfp);
-	fflush(logctl.l_mdfp);
+	__pmFflush(logctl.l_mfp);
+	__pmFflush(logctl.l_mdfp);
 
 	if (old_log_offset == 0)
 	    old_log_offset = sizeof(__pmLogLabel)+2*sizeof(int);
 
-	new_log_offset = ftell(logctl.l_mfp);
+	new_log_offset = __pmFtell(logctl.l_mfp);
 	assert(new_log_offset >= 0);
-	new_meta_offset = ftell(logctl.l_mdfp);
+	new_meta_offset = __pmFtell(logctl.l_mdfp);
 	assert(new_meta_offset >= 0);
 
 #if 0
@@ -1985,7 +2170,7 @@ cleanup:
 	    mintime.tv_sec, mintime.tv_usec, tmptime.tv_sec, tmptime.tv_usec, logend.tv_sec, logend.tv_usec, winend.tv_sec, winend.tv_usec, current.tv_sec, current.tv_usec);
 #endif
 
-	fseek(logctl.l_mfp, old_log_offset, SEEK_SET);
+	__pmFseek(logctl.l_mfp, old_log_offset, SEEK_SET);
 	__pmLogPutIndex(&logctl, &current);
 
 
