@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2014 Red Hat.
+ * Copyright (c) 2012-2017 Red Hat.
  * Copyright (c) 1995-2001 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -277,6 +277,9 @@ add_metric(pmValueSet *vsp, task_t **result)
     tp->t_pmidlist[i] = pmid;
     tp->t_namelist[i] = name;
     tp->t_desclist[i] = *dp;	/* struct assignment */
+    /* is this a derived metric? */
+    if (IS_DERIVED(pmid))
+	tp->t_dm++;
 
     rqp = (optreq_t *)calloc(1, sizeof(optreq_t));
     if (rqp == NULL) {
@@ -1104,8 +1107,7 @@ do_control(__pmPDU *pb)
 		newtp->t_state = PMLC_GET_STATE(reqstate);
 		if (PMLC_GET_ON(reqstate)) {
 		    newtp->t_delta = tdelta;
-		    newtp->t_afid = __pmAFregister(&tdelta, (void *)newtp,
-					       log_callback);
+		    newtp->t_afid = __pmAFsetup(NULL, &tdelta, (void *)newtp, log_callback);
 		}
 		else
 		    newtp->t_delta.tv_sec = newtp->t_delta.tv_usec = 0;
@@ -1281,7 +1283,7 @@ sendstatus(void)
 	ls.ls_timenow.tv_sec = (__int32_t)now.tv_sec;
 	ls.ls_timenow.tv_usec = (__int32_t)now.tv_usec;
 	ls.ls_vol = logctl.l_curvol;
-	ls.ls_size = ftell(logctl.l_mfp);
+	ls.ls_size = __pmFtell(logctl.l_mfp);
 	assert(ls.ls_size >= 0);
 
 	/* be careful of buffer size mismatches when copying strings */
