@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2016 Red Hat.
+ * Copyright (c) 2012-2017 Red Hat.
  * Copyright (c) 1995-2001,2003 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -39,6 +39,7 @@ int		primary;		/* Non-zero for primary pmlogger */
 char	    	*archBase;		/* base name for log files */
 char		*pmcd_host;
 char		*pmcd_host_conn;
+char		*pmcd_host_label;
 int		host_context = PM_CONTEXT_HOST;	 /* pmcd / local context mode */
 int		archive_version = PM_LOG_VERS02; /* Type of archive to create */
 int		linger = 0;		/* linger with no tasks/events */
@@ -81,7 +82,7 @@ run_done(int sts, char *msg)
 	__pmTimeval	tmp;
 	tmp.tv_sec = (__int32_t)last_stamp.tv_sec;
 	tmp.tv_usec = (__int32_t)last_stamp.tv_usec;
-	fseek(logctl.l_mfp, last_log_offset, SEEK_SET);
+	__pmFseek(logctl.l_mfp, last_log_offset, SEEK_SET);
 	__pmLogPutIndex(&logctl, &tmp);
     }
 
@@ -317,7 +318,7 @@ do_dialog(char cmd)
 	/* hack is close enough! */
 	now = 1;
 
-    archsize = vol_bytes + ftell(logctl.l_mfp);
+    archsize = vol_bytes + __pmFtell(logctl.l_mfp);
 
     nchar = add_msg(&p, 0, "");
     p[0] = '\0';
@@ -468,6 +469,7 @@ static pmLongOptions longopts[] = {
     { "check", 0, 'C', 0, "parse configuration and exit" },
     PMOPT_DEBUG,
     PMOPT_HOST,
+    { "labelhost", 1, 'H', "LABELHOST", "override the hostname written into the label" },
     { "log", 1, 'l', "FILE", "redirect diagnostics and trace output" },
     { "linger", 0, 'L', 0, "run even if not primary logger instance and nothing to log" },
     { "note", 1, 'm', "MSG", "descriptive note to be added to the port map file" },
@@ -491,7 +493,7 @@ static pmLongOptions longopts[] = {
 };
 
 static pmOptions opts = {
-    .short_options = "c:CD:h:l:K:Lm:n:op:Prs:T:t:uU:v:V:x:y?",
+    .short_options = "c:CD:h:H:l:K:Lm:n:op:Prs:T:t:uU:v:V:x:y?",
     .long_options = longopts,
     .short_usage = "[options] archive",
 };
@@ -613,6 +615,10 @@ main(int argc, char **argv)
 	    pmcd_host_conn = opts.optarg;
 	    break;
 
+	case 'H':		/* hostname to put in label*/
+	    pmcd_host_label = strndup(opts.optarg, PM_LOG_MAXHOSTLEN-1);
+	    break;
+
 	case 'l':		/* log file name */
 	    logfile = opts.optarg;
 	    break;
@@ -906,6 +912,10 @@ main(int argc, char **argv)
 	exit(1);
     }
 
+    if (pmcd_host_label != NULL) {
+	pmcd_host=pmcd_host_label;
+    }
+
     if ((sts = __pmLogCreate(pmcd_host, archBase, archive_version, &logctl)) < 0) {
 	fprintf(stderr, "__pmLogCreate: %s\n", pmErrStr(sts));
 	exit(1);
@@ -1214,7 +1224,7 @@ main(int argc, char **argv)
 int
 newvolume(int vol_switch_type)
 {
-    FILE	*newfp;
+    __pmFILE	*newfp;
     int		nextvol = logctl.l_curvol + 1;
     time_t	now;
     static char *vol_sw_strs[] = {
@@ -1223,7 +1233,7 @@ newvolume(int vol_switch_type)
     };
 
     vol_samples_counter = 0;
-    vol_bytes += ftell(logctl.l_mfp);
+    vol_bytes += __pmFtell(logctl.l_mfp);
     if (exit_bytes != -1) {
         if (vol_bytes >= exit_bytes) 
 	    run_done(0, "Byte limit reached");
@@ -1268,7 +1278,7 @@ newvolume(int vol_switch_type)
 	 *	this happens in do_work() over in callback.c
 	 */
 
-	fclose(logctl.l_mfp);
+	__pmFclose(logctl.l_mfp);
 	logctl.l_mfp = newfp;
 	logctl.l_label.ill_vol = logctl.l_curvol = nextvol;
 	__pmLogWriteLabel(logctl.l_mfp, &logctl.l_label);
@@ -1294,6 +1304,14 @@ disconnect(int sts)
 	fprintf(stderr, "%s: disconnect botch: cannot get context: %s\n", pmProgname, pmErrStr(ctx));
 	exit(1);
     }
+    /*
+     * Note: This application is single threaded, and once we have ctxp
+     *	     the associated __pmContext will not move and will only be
+     *	     accessed or modified synchronously either here or in libpcp.
+     *	     We unlock the context so that it can be locked as required
+     *	     within libpcp.
+     */
+    PM_UNLOCK(ctxp->c_lock);
 
     if (pmcdfd != -1) {
 	/*
@@ -1316,19 +1334,7 @@ disconnect(int sts)
 	}
 	numfds = maxfd() + 1;
 	ctxp->c_pmcd->pc_fd = -1;
-	/*
-	 * since there is only one context in play here, pc_refcnt is not
-	 * really in play, but reset it for consistency
-	 */
-	ctxp->c_pmcd->pc_refcnt = 0;
     }
-
-    /*
-     * for repeated calls here, c_lock may have been locked again,
-     * so although there is no channel cleanup to be done, release
-     * the lock
-     */
-    PM_UNLOCK(ctxp->c_lock);
 }
 
 int
@@ -1346,13 +1352,20 @@ reconnect(void)
 		pmProgname, pmErrStr(ctx));
 	exit(1);
     }
+    /*
+     * Note: This application is single threaded, and once we have ctxp
+     *	     the associated __pmContext will not move and will only be
+     *	     accessed or modified synchronously either here or in libpcp.
+     *	     We unlock the context so that it can be locked as required
+     *	     within libpcp.
+     */
+    PM_UNLOCK(ctxp->c_lock);
     sts = pmReconnectContext(ctx);
     if (sts >= 0) {
 	pmcdfd = ctxp->c_pmcd->pc_fd;
 	__pmFD_SET(pmcdfd, &fds);
 	numfds = maxfd() + 1;
     }
-    PM_UNLOCK(ctxp->c_lock);
     if (sts < 0)
 	return sts;
 
