@@ -1,7 +1,7 @@
 /*
  * pmlogrewrite - config-driven stream editor for PCP archives
  *
- * Copyright (c) 2013-2014 Red Hat.
+ * Copyright (c) 2013-2017 Red Hat.
  * Copyright (c) 2011 Ken McDonell.  All Rights Reserved.
  * Copyright (c) 1997-2002 Silicon Graphics, Inc.  All Rights Reserved.
  * 
@@ -82,15 +82,15 @@ int	wflag;				/* -w emit warnings */
  *  report that archive is corrupted
  */
 static void
-_report(FILE *fp)
+_report(__pmFILE *fp)
 {
     off_t	here;
     struct stat	sbuf;
 
-    here = lseek(fileno(fp), 0L, SEEK_CUR);
+    here = __pmLseek(fp, 0L, SEEK_CUR);
     fprintf(stderr, "%s: Error occurred at byte offset %ld into a file of",
 	    pmProgname, (long)here);
-    if (fstat(fileno(fp), &sbuf) < 0)
+    if (__pmFstat(fp, &sbuf) < 0)
 	fprintf(stderr, ": stat: %s\n", osstrerror());
     else
 	fprintf(stderr, " %ld bytes.\n", (long)sbuf.st_size);
@@ -106,14 +106,14 @@ _report(FILE *fp)
 void
 newvolume(int vol)
 {
-    FILE		*newfp;
+    __pmFILE		*newfp;
 
     if ((newfp = __pmLogNewFile(outarch.name, vol)) != NULL) {
-	fclose(outarch.logctl.l_mfp);
+	__pmFclose(outarch.logctl.l_mfp);
 	outarch.logctl.l_mfp = newfp;
 	outarch.logctl.l_label.ill_vol = outarch.logctl.l_curvol = vol;
 	__pmLogWriteLabel(outarch.logctl.l_mfp, &outarch.logctl.l_label);
-	fflush(outarch.logctl.l_mfp);
+	__pmFflush(outarch.logctl.l_mfp);
     }
     else {
 	fprintf(stderr, "%s: __pmLogNewFile(%s,%d) Error: %s\n",
@@ -153,34 +153,34 @@ writelabel(int do_rewind)
     off_t	old_offset;
 
     if (do_rewind) {
-	old_offset = ftell(outarch.logctl.l_tifp);
+	old_offset = __pmFtell(outarch.logctl.l_tifp);
 	assert(old_offset >= 0);
-	rewind(outarch.logctl.l_tifp);
+	__pmRewind(outarch.logctl.l_tifp);
     }
     outarch.logctl.l_label.ill_vol = PM_LOG_VOL_TI;
     __pmLogWriteLabel(outarch.logctl.l_tifp, &outarch.logctl.l_label);
     if (do_rewind)
-	fseek(outarch.logctl.l_tifp, (long)old_offset, SEEK_SET);
+	__pmFseek(outarch.logctl.l_tifp, (long)old_offset, SEEK_SET);
 
     if (do_rewind) {
-	old_offset = ftell(outarch.logctl.l_mdfp);
+	old_offset = __pmFtell(outarch.logctl.l_mdfp);
 	assert(old_offset >= 0);
-	rewind(outarch.logctl.l_mdfp);
+	__pmRewind(outarch.logctl.l_mdfp);
     }
     outarch.logctl.l_label.ill_vol = PM_LOG_VOL_META;
     __pmLogWriteLabel(outarch.logctl.l_mdfp, &outarch.logctl.l_label);
     if (do_rewind)
-	fseek(outarch.logctl.l_mdfp, (long)old_offset, SEEK_SET);
+	__pmFseek(outarch.logctl.l_mdfp, (long)old_offset, SEEK_SET);
 
     if (do_rewind) {
-	old_offset = ftell(outarch.logctl.l_mfp);
+	old_offset = __pmFtell(outarch.logctl.l_mfp);
 	assert(old_offset >= 0);
-	rewind(outarch.logctl.l_mfp);
+	__pmRewind(outarch.logctl.l_mfp);
     }
     outarch.logctl.l_label.ill_vol = 0;
     __pmLogWriteLabel(outarch.logctl.l_mfp, &outarch.logctl.l_label);
     if (do_rewind)
-	fseek(outarch.logctl.l_mfp, (long)old_offset, SEEK_SET);
+	__pmFseek(outarch.logctl.l_mfp, (long)old_offset, SEEK_SET);
 }
 
 /*
@@ -226,7 +226,8 @@ nextlog(void)
     lcp = inarch.ctxp->c_archctl->ac_log;
     old_vol = inarch.ctxp->c_archctl->ac_log->l_curvol;
 
-    if ((sts = __pmLogRead(lcp, PM_MODE_FORW, NULL, &inarch.rp, PMLOGREAD_NEXT)) < 0) {
+    sts = __pmLogRead_ctx(inarch.ctxp, PM_MODE_FORW, NULL, &inarch.rp, PMLOGREAD_NEXT);
+    if (sts < 0) {
 	if (sts != PM_ERR_EOL) {
 	    fprintf(stderr, "%s: Error: __pmLogRead[log %s]: %s\n",
 		    pmProgname, inarch.name, pmErrStr(sts));
@@ -874,6 +875,7 @@ main(int argc, char **argv)
     __pmTimeval	tstamp = { 0 };		/* for last log record */
     off_t	old_log_offset = 0;	/* log offset before last log record */
     off_t	old_meta_offset;
+    int		seen_event = 0;
 
     /* process cmd line args */
     if (parseargs(argc, argv) < 0) {
@@ -897,6 +899,14 @@ main(int argc, char **argv)
     }
     inarch.ctxp = __pmHandleToPtr(inarch.ctx);
     assert(inarch.ctxp != NULL);
+    /*
+     * Note: This application is single threaded, and once we have ctxp
+     *	     the associated __pmContext will not move and will only be
+     *	     accessed or modified synchronously either here or in libpcp.
+     *	     We unlock the context so that it can be locked as required
+     *	     within libpcp.
+     */
+    PM_UNLOCK(inarch.ctxp->c_lock);
 
     if ((sts = pmGetArchiveLabel(&inarch.label)) < 0) {
 	fprintf(stderr, "%s: Error: cannot get archive label record (%s): %s\n",
@@ -1068,11 +1078,11 @@ main(int argc, char **argv)
     while (1) {
 	static long	in_offset;		/* for -Dappl0 */
 
-	fflush(outarch.logctl.l_mdfp);
-	old_meta_offset = ftell(outarch.logctl.l_mdfp);
+	__pmFflush(outarch.logctl.l_mdfp);
+	old_meta_offset = __pmFtell(outarch.logctl.l_mdfp);
 	assert(old_meta_offset >= 0);
 
-	in_offset = ftell(inarch.ctxp->c_archctl->ac_log->l_mfp);
+	in_offset = __pmFtell(inarch.ctxp->c_archctl->ac_log->l_mfp);
 	stslog = nextlog();
 	if (stslog < 0) {
 #if PCP_DEBUG
@@ -1131,36 +1141,67 @@ main(int argc, char **argv)
 	 * that is not in the current log record
 	 */
 	for ( ; ; ) {
-	    pmID	pmid;			/* pmid for TYPE_DESC */
-	    pmInDom	indom;			/* indom for TYPE_INDOM */
-
 	    if (stsmeta == 0) {
-		in_offset = ftell(inarch.ctxp->c_archctl->ac_log->l_mdfp);
+		in_offset = __pmFtell(inarch.ctxp->c_archctl->ac_log->l_mdfp);
 		stsmeta = nextmeta();
 #if PCP_DEBUG
-		if (stsmeta < 0 && pmDebug & DBG_TRACE_APPL0)
-		    fprintf(stderr, "Metadata: read EOF @ offset=%ld\n", in_offset);
+		if (pmDebug & DBG_TRACE_APPL0) {
+		    if (stsmeta < 0)
+			fprintf(stderr, "Metadata: read EOF @ offset=%ld\n", in_offset);
+		    else if (stsmeta == TYPE_DESC)
+			fprintf(stderr, "Metadata: read PMID %s @ offset=%ld\n", pmIDStr(ntoh_pmID(inarch.metarec[2])), in_offset);
+		    else if (stsmeta == TYPE_INDOM)
+			fprintf(stderr, "Metadata: read InDom %s @ offset=%ld\n", pmInDomStr(ntoh_pmInDom((unsigned int)inarch.metarec[4])), in_offset);
+		}
 #endif
 	    }
 	    if (stsmeta < 0) {
 		break;
 	    }
+
 	    if (stsmeta == TYPE_DESC) {
-		int	i;
-		pmid = ntoh_pmID(inarch.metarec[2]);
-#if PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL0)
-		    fprintf(stderr, "Metadata: read PMID %s @ offset=%ld\n", pmIDStr(pmid), in_offset);
-#endif
+		pmDesc	*dp = (pmDesc *)&inarch.metarec[2];
+		pmID	pmid = ntoh_pmID(dp->pmid);
+		int	type = ntohl(dp->type);
 		/*
-		 * if pmid not in next pmResult, we're done ...
+		 * We used to apply an optimization here to stop
+		 * processing pmDesc metadata if the PMID of the
+		 * next metadata did not match one of the PMIDs
+		 * in the current pmResult.
+		 *
+		 * Unfortunately "event" records contain packed
+		 * PMIDs for the event records and pseudo PMIDs
+		 * for the metrics used to encode the event record
+		 * parameters ... these just confuse the optimization
+		 * and unpacking the event records does not help
+		 * because the pmDesc for the pseudo PMIDs are 
+		 * written in a batch when the first event record
+		 * is output, even if they may not appear in a packed
+		 * pmResult until later, or not at all.
+		 *
+		 * So if we've seen a metric of type PM_TYPE_EVENT
+		 * or PM_TYPE_HIGHRES_EVENT, just keep processing the
+		 * pmDesc metadata until it is exhausted, or we find
+		 * a pmInDom metadata record with a timestamp after the
+		 * current pmResult.
 		 */
-		for (i = 0; i < inarch.rp->numpmid; i++) {
-		    if (pmid == inarch.rp->vset[i]->pmid)
-			break;
+
+		if (!seen_event) {
+		    if (type == PM_TYPE_EVENT || type == PM_TYPE_HIGHRES_EVENT)
+			seen_event = 1;
+		    else {
+			/*
+			 * if pmid not in next pmResult, we're done ...
+			 */
+			for (i = 0; i < inarch.rp->numpmid; i++) {
+			    if (pmid == inarch.rp->vset[i]->pmid)
+				break;
+			}
+			if (i == inarch.rp->numpmid)
+			    break;
+		    }
 		}
-		if (i == inarch.rp->numpmid)
-		    break;
+
 		/*
 		 * rewrite if needed, delete if needed else output
 		 */
@@ -1169,11 +1210,6 @@ main(int argc, char **argv)
 	    else if (stsmeta == TYPE_INDOM) {
 		struct timeval	stamp;
 		__pmTimeval	*tvp = (__pmTimeval *)&inarch.metarec[2];
-		indom = ntoh_pmInDom((unsigned int)inarch.metarec[4]);
-#if PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL0)
-		    fprintf(stderr, "Metadata: read InDom %s @ offset=%ld\n", pmInDomStr(indom), in_offset);
-#endif
 		stamp.tv_sec = ntohl(tvp->tv_sec);
 		stamp.tv_usec = ntohl(tvp->tv_usec);
 		if (fixstamp(&stamp)) {
@@ -1214,20 +1250,20 @@ main(int argc, char **argv)
 	tstamp.tv_usec = inarch.rp->timestamp.tv_usec;
 
 	if (needti) {
-	    fflush(outarch.logctl.l_mdfp);
-	    fflush(outarch.logctl.l_mfp);
-	    new_meta_offset = ftell(outarch.logctl.l_mdfp);
+	    __pmFflush(outarch.logctl.l_mdfp);
+	    __pmFflush(outarch.logctl.l_mfp);
+	    new_meta_offset = __pmFtell(outarch.logctl.l_mdfp);
 	    assert(new_meta_offset >= 0);
-            fseek(outarch.logctl.l_mdfp, (long)old_meta_offset, SEEK_SET);
+            __pmFseek(outarch.logctl.l_mdfp, (long)old_meta_offset, SEEK_SET);
             __pmLogPutIndex(&outarch.logctl, &tstamp);
-            fseek(outarch.logctl.l_mdfp, (long)new_meta_offset, SEEK_SET);
+            __pmFseek(outarch.logctl.l_mdfp, (long)new_meta_offset, SEEK_SET);
 	    needti = 0;
 	    doneti = 1;
         }
 	else
 	    doneti = 0;
 
-	old_log_offset = ftell(outarch.logctl.l_mfp);
+	old_log_offset = __pmFtell(outarch.logctl.l_mfp);
 	assert(old_log_offset >= 0);
 
 	if (inarch.rp->numpmid == 0)
@@ -1239,31 +1275,31 @@ main(int argc, char **argv)
 
     if (!doneti) {
 	/* Final temporal index entry */
-	fflush(outarch.logctl.l_mfp);
-	fseek(outarch.logctl.l_mfp, (long)old_log_offset, SEEK_SET);
+	__pmFflush(outarch.logctl.l_mfp);
+	__pmFseek(outarch.logctl.l_mfp, (long)old_log_offset, SEEK_SET);
 	__pmLogPutIndex(&outarch.logctl, &tstamp);
     }
 
     if (iflag) {
 	/*
-	 * fsync() to make sure new archive is safe before we start
+	 * __pmFsync() to make sure new archive is safe before we start
 	 * renaming ...
 	 */
-	if (fsync(fileno(outarch.logctl.l_mdfp)) < 0) {
+	if (__pmFsync(outarch.logctl.l_mdfp) < 0) {
 	    fprintf(stderr, "%s: Error: fsync(%d) failed for output metadata file: %s\n",
-		pmProgname, fileno(outarch.logctl.l_mdfp), strerror(errno));
+		pmProgname, __pmFileno(outarch.logctl.l_mdfp), strerror(errno));
 		abandon();
 		/*NOTREACHED*/
 	}
-	if (fsync(fileno(outarch.logctl.l_mfp)) < 0) {
+	if (__pmFsync(outarch.logctl.l_mfp) < 0) {
 	    fprintf(stderr, "%s: Error: fsync(%d) failed for output data file: %s\n",
-		pmProgname, fileno(outarch.logctl.l_mfp), strerror(errno));
+		pmProgname, __pmFileno(outarch.logctl.l_mfp), strerror(errno));
 		abandon();
 		/*NOTREACHED*/
 	}
-	if (fsync(fileno(outarch.logctl.l_tifp)) < 0) {
+	if (__pmFsync(outarch.logctl.l_tifp) < 0) {
 	    fprintf(stderr, "%s: Error: fsync(%d) failed for output index file: %s\n",
-		pmProgname, fileno(outarch.logctl.l_tifp), strerror(errno));
+		pmProgname, __pmFileno(outarch.logctl.l_tifp), strerror(errno));
 		abandon();
 		/*NOTREACHED*/
 	}
