@@ -1,7 +1,7 @@
 /*
  * JSON web bridge for PMAPI.
  *
- * Copyright (c) 2011-2014 Red Hat.
+ * Copyright (c) 2011-2017 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -28,15 +28,19 @@ string uriprefix = "pmapi";
 string resourcedir;		/* set by -R option */
 string archivesdir = ".";	/* set by -A option */
 unsigned verbosity;		/* set by -v option */
+unsigned permissive;            /* set by -P option */
 unsigned maxtimeout = 300;	/* set by -t option */
 int dumpstats = 300;            /* set by -d option */
 map<string,unsigned> clients_usage;
-unsigned perm_context = 1;	/* set by -c option, changed by -h/-a/-L */
+unsigned long perm_context = 1;	/* set by -c option, changed by -h/-a/-L */
 unsigned new_contexts_p = 1;	/* cleared by -N option */
 unsigned graphite_p;		/* set by -G option */
+unsigned graphite_encode = 1;	/* unset by -X option */
 unsigned exit_p;		/* counted by SIG* handler */
 static __pmServerPresence *presence;
 unsigned multithread = 0;       /* set by -M option */
+unsigned graphite_timestep = 60;  /* set by -i option */
+unsigned graphite_archivedir = 0; /* set by -I option */
 string logfile = "";		/* set by -l option */
 string fatalfile = "/dev/tty";	/* fatal messages at startup go here */
 
@@ -243,7 +247,7 @@ mhd_respond (void *cls, struct MHD_Connection *connection, const char *url0,
         else if (graphite_p && (method == "GET" || method == "POST") && (url1 == "graphite")
                  && ((url2 == "render") || (url2 == "metrics") || (url2 == "rawdata")
                      || (url2 == "browser") || (url2 == "graphlot" && url3 == "findmetric"))) {
-            return pmgraphite_respond (connection, mhd_cc->params, url_tokens);
+            return pmgraphite_respond (connection, mhd_cc->params, url_tokens, url);
         }
         // graphite dashboard idiosyncracy; note absence of /graphite top level
         else if (graphite_p && (method == "GET" || method == "POST") && 
@@ -251,7 +255,7 @@ mhd_respond (void *cls, struct MHD_Connection *connection, const char *url0,
                   (url1 == "render"))) {
             url_tokens.insert (url_tokens.begin() + 1 /* empty #0 */,
                                string("graphite"));
-            return pmgraphite_respond (connection, mhd_cc->params, url_tokens);
+            return pmgraphite_respond (connection, mhd_cc->params, url_tokens, url);
         }
 
         /* pmresapi? */
@@ -352,6 +356,14 @@ server_dump_configuration ()
     // don't have to repeat.
 
     clog << "\tVerbosity level " << verbosity << endl;
+    if (permissive)
+        clog << "\tPermissive mode enabled" << endl;
+    else
+        clog << "\tPermissive mode disabled" << endl;
+    if (__pmServerHasFeature(PM_SERVER_FEATURE_CREDS_REQD))
+        clog << "\tMandatory authentication enabled" << endl;
+    else
+        clog << "\tMandatory authentication disabled" << endl;
     clog << "\tUsing libmicrohttpd " << MHD_get_version () << endl;
 
     cwd = getcwd (cwdpath, sizeof (cwdpath));
@@ -376,6 +388,7 @@ server_dump_configuration ()
     }
 
     clog << "\tGraphite API " << (graphite_p ? "enabled" : "disabled") << endl;
+    clog << "\tGraphite API name encoding " << (graphite_encode ? "long" : "short") << endl;
     clog << "\tGraphite API Cairo graphics rendering "
 #ifdef HAVE_CAIRO
          << "compiled-in"
@@ -383,7 +396,13 @@ server_dump_configuration ()
          << "unavailable"
 #endif
          << endl;
-
+    clog << "\tHTTP compression "
+#ifdef HAVE_ZLIB
+         << "compiled-in"
+#else
+         << "unavailable"
+#endif
+         << endl;
     if (dumpstats > 0) {
         clog << "\tPeriodic client statistics dumped roughly every " << dumpstats << "s" << endl;
     } else {
@@ -418,7 +437,9 @@ pmweb_shutdown (struct MHD_Daemon *d4, struct MHD_Daemon *d6)
     }
 
     /* No longer advertise pmwebd presence on the network. */
-    __pmServerUnadvertisePresence (presence);
+    if (presence) {
+        __pmServerUnadvertisePresence (presence);
+    }
 
     /*
      * Let's politely clean up all the active contexts.
@@ -445,7 +466,11 @@ option_overrides (int opt, pmOptions * opts)
     case 'M':
     case 'p':
     case 'd':
+    case 'S':
     case 't':
+    case 'i':
+    case 'I':
+    case 'X':
         return 1;
     }
     return 0;
@@ -457,19 +482,26 @@ longopts[] = {
     {"port", 1, 'p', "NUM", "listen on given TCP port [default 44323]"},
     {"ipv4", 0, '4', 0, "listen on IPv4 only"},
     {"ipv6", 0, '6', 0, "listen on IPv6 only"},
-    {"timeout", 1, 't', "SEC", "max time (seconds) for PMAPI polling [default 300]"},
-    {"resources", 1, 'R', "DIR", "serve non-API files from given directory"},
+    PMAPI_OPTIONS_HEADER ("Graphite options"),
     {"graphite", 0, 'G', 0, "enable graphite 0.9 API/backend emulation"},
+    {"graphite-noencode", 0, 'X', 0, "don't encode special characters that are now allowed by graphite"},
+    {"graphite-timestamp", 1, 'i', "SEC", "minimum graphite timestep (s) [default 60]"},
+    {"graphite-archivedir", 0, 'I', 0, "prefer archive directories [default OFF]"},
     PMAPI_OPTIONS_HEADER ("Context options"),
+    {"timeout", 1, 't', "SEC", "max time (seconds) for PMAPI polling [default 300]"},
     {"context", 1, 'c', "NUM", "set next permanent-binding context number"},
     {"host", 1, 'h', "HOST", "permanent-bind next context to PMCD on host"},
     {"archive", 1, 'a', "FILE", "permanent-bind next context to archive"},
     {"local-PMDA", 0, 'L', 0, "permanent-bind next context to local PMDAs"},
+    {"permissive", 0, 'P', 0, "allow unix: and local-context modes"},
     {"", 0, 'N', 0, "disable remote new-context requests"},
     {"", 1, 'A', "DIR", "permit remote new-archive-context under dir [default CWD]"},
     PMAPI_OPTIONS_HEADER ("Other"),
     PMOPT_DEBUG,
+    {"resources", 1, 'R', "DIR", "serve non-API files from given directory"},
     {"log", 1, 'l', "FILE", "redirect diagnostics and trace output"},
+    { "", 0, 'S', 0, "disable service advertisement" },
+    { "", 0, 'C', 0, "allow only authenticated clients" },
     {"verbose", 0, 'v', 0, "increase verbosity"},
 #ifdef HAVE_PTHREAD_H
     {"threads", 1, 'M', 0, "allow multiple threads [default 0]"},
@@ -491,7 +523,9 @@ main (int argc, char *argv[])
     int     ctx;
     int     mhd_ipv4 = 1;
     int     mhd_ipv6 = 1;
+    int     localmode = 0;
     int    port = PMWEBD_PORT;
+    static char utc_timezone[] = "TZ=UTC"; /* static for putenv safety through shutdown */
     char *   endptr;
     struct MHD_Daemon * d4 = NULL;
     struct MHD_Daemon * d6 = NULL;
@@ -499,19 +533,24 @@ main (int argc, char *argv[])
 
     // NB: important to standardize on a single default timezone, since
     // we'll be interacting with web clients from anywhere, and dealing
-    // with pcp servers/archvies from anywhere else.
-    (void) setenv ("TZ", "UTC", 1);
+    // with pcp servers/archives from anywhere else.
+    (void) putenv (utc_timezone);
 
     umask (022);
     char * username_str;
     __pmGetUsername (&username_str);
+    __pmServerSetFeature (PM_SERVER_FEATURE_DISCOVERY);
 
-    opts.short_options = "A:a:c:D:h:Ll:NM:p:R:Gt:U:vx:d:46?";
+    opts.short_options = "A:a:c:CD:h:Ll:NM:Pp:R:Gi:It:U:vx:d:SX46?";
     opts.long_options = longopts;
     opts.override = option_overrides;
 
     while ((c = pmGetOptions (argc, argv, &opts)) != EOF) {
         switch (c) {
+        case 'P':
+            permissive = 1;
+            break;
+
         case 'p':
             port = (int) strtol (opts.optarg, &endptr, 0);
             if (*endptr != '\0' || port < 0 || port > 65535) {
@@ -536,6 +575,22 @@ main (int argc, char *argv[])
             graphite_p = 1;
             break;
 
+        case 'X':
+            graphite_encode = 0;
+            break;
+
+        case 'i':
+            graphite_timestep = atoi (opts.optarg);
+            if (graphite_timestep <= 0) {
+                pmprintf ("%s: timestep too small %s\n", pmProgname, opts.optarg);
+                opts.errors++;
+            }
+            break;
+
+        case 'I':
+            graphite_archivedir = 1;
+            break;
+
         case 'A':
             archivesdir = opts.optarg;
             break;
@@ -570,6 +625,9 @@ main (int argc, char *argv[])
             break;
 
         case 'h':
+            if (strstr(opts.optarg, "unix:") != NULL ||
+                strstr(opts.optarg, "local:") != NULL)
+                localmode = 1;	// complete this check after arg parsing
             if ((ctx = pmNewContext (PM_CONTEXT_HOST, opts.optarg)) < 0) {
                 __pmNotifyErr (LOG_ERR, "new context failed\n");
                 exit (EXIT_FAILURE);
@@ -578,7 +636,7 @@ main (int argc, char *argv[])
                 __pmNotifyErr (LOG_ERR, "permanent bind failed\n");
                 exit (EXIT_FAILURE);
             }
-            __pmNotifyErr (LOG_INFO, "context (web%d=pm%d) created, host %s, permanent\n",
+            __pmNotifyErr (LOG_INFO, "context (web%lu=pm%d) created, host %s, permanent\n",
                            perm_context - 1, ctx, opts.optarg);
             break;
 
@@ -591,11 +649,12 @@ main (int argc, char *argv[])
                 __pmNotifyErr (LOG_ERR, "permanent bind failed\n");
                 exit (EXIT_FAILURE);
             }
-            __pmNotifyErr (LOG_INFO, "context (web%d=pm%d) created, archive %s, permanent\n",
+            __pmNotifyErr (LOG_INFO, "context (web%lu=pm%d) created, archive %s, permanent\n",
                            perm_context - 1, ctx, opts.optarg);
             break;
 
         case 'L':
+            localmode = 1;	// complete this check after arg parsing
             if ((ctx = pmNewContext (PM_CONTEXT_LOCAL, NULL)) < 0) {
                 __pmNotifyErr (LOG_ERR, "new context failed\n");
                 exit (EXIT_FAILURE);
@@ -604,7 +663,7 @@ main (int argc, char *argv[])
                 __pmNotifyErr (LOG_ERR, "permanent bind failed\n");
                 exit (EXIT_FAILURE);
             }
-            __pmNotifyErr (LOG_INFO, "context (web%d=pm%d) created, local, permanent\n",
+            __pmNotifyErr (LOG_INFO, "context (web%lu=pm%d) created, local, permanent\n",
                            perm_context - 1, ctx);
             break;
 
@@ -621,6 +680,14 @@ main (int argc, char *argv[])
             logfile = opts.optarg;
             break;
 
+	case 'C':	/* only allow authenticated clients */
+	    __pmServerSetFeature (PM_SERVER_FEATURE_CREDS_REQD);
+	    break;
+
+	case 'S':	/* disable pmwebd service advertising */
+	    __pmServerClearFeature (PM_SERVER_FEATURE_DISCOVERY);
+	    break;
+
         case 'U':
             /* run as user username */
             username_str = opts.optarg;
@@ -632,6 +699,16 @@ main (int argc, char *argv[])
         }
     }
 
+    if (!permissive && localmode) {
+        pmprintf ( "%s: non-permissive and local-context modes are mutually exclusive\n", pmProgname);
+        opts.errors++;
+    }
+
+    if (permissive && __pmServerHasFeature (PM_SERVER_FEATURE_CREDS_REQD)) {
+        pmprintf ( "%s: permissive and mandatory authentication modes are mutually exclusive\n", pmProgname);
+        opts.errors++;
+    }
+
     if (opts.errors) {
         pmUsageMessage (&opts);
         exit (EXIT_FAILURE);
@@ -644,13 +721,20 @@ main (int argc, char *argv[])
      * we could make use of MHD_USE_THREAD_PER_CONNECTION; we'd need
      * to add ample locking over pmwebd context structures etc.
      */
+#ifdef MHD_USE_EPOLL_TURBO
+#define MY_MHD_FLAGS MHD_USE_EPOLL_TURBO
+#else
+#define MY_MHD_FLAGS 0
+#endif
     if (mhd_ipv4)
-        d4 = MHD_start_daemon (0, port, NULL, NULL,	/* default accept policy */
+        d4 = MHD_start_daemon (MY_MHD_FLAGS,
+                               port, NULL, NULL,	/* default accept policy */
                                &mhd_respond, NULL,	/* handler callback */
                                MHD_OPTION_CONNECTION_TIMEOUT, maxtimeout, MHD_OPTION_NOTIFY_COMPLETED,
                                &mhd_respond_completed, NULL, MHD_OPTION_END);
     if (mhd_ipv6)
-        d6 = MHD_start_daemon (MHD_USE_IPv6, port, NULL, NULL,	/* default accept policy */
+        d6 = MHD_start_daemon (MHD_USE_IPv6 | MY_MHD_FLAGS,
+                               port, NULL, NULL,	/* default accept policy */
                                &mhd_respond, NULL,	/* handler callback */
                                MHD_OPTION_CONNECTION_TIMEOUT, maxtimeout, MHD_OPTION_NOTIFY_COMPLETED,
                                &mhd_respond_completed, NULL, MHD_OPTION_END);
@@ -659,14 +743,16 @@ main (int argc, char *argv[])
         pmweb_dont_start ();
     }
 
+#ifndef IS_MINGW
     /* lose root privileges if we have them */
-    if (geteuid () == 0) {
+    if (geteuid () == 0)
+#endif
         __pmSetProcessIdentity (username_str);
-    }
 
     /* tell the world we have arrived */
     __pmServerCreatePIDFile (PM_SERVER_WEBD_SPEC, 0);
-    presence = __pmServerAdvertisePresence (PM_SERVER_WEBD_SPEC, port);
+    if (__pmServerHasFeature (PM_SERVER_FEATURE_DISCOVERY))
+        presence = __pmServerAdvertisePresence (PM_SERVER_WEBD_SPEC, port);
 
     // (re)create log file, redirect stdout/stderr
     // NB: must be done after __pmSetProcessIdentity() for proper file permissions
@@ -707,7 +793,9 @@ main (int argc, char *argv[])
     __pmSetSignalHandler (SIGHUP, SIG_IGN);
     __pmSetSignalHandler (SIGINT, handle_signals);
     __pmSetSignalHandler (SIGTERM, handle_signals);
+#ifndef IS_MINGW
     __pmSetSignalHandler (SIGQUIT, handle_signals);
+#endif
     /* Not this one; might get it from pmcd momentary disconnection. */
     /* __pmSetSignalHandler(SIGPIPE, handle_signals); */
 
@@ -721,16 +809,16 @@ main (int argc, char *argv[])
         fd_set        rs;
         fd_set        ws;
         fd_set        es;
-        int        max = 0;
+        MHD_socket    maxsock = 0;
 
         /* Based upon MHD fileserver_example_external_select.c */
         FD_ZERO (&rs);
         FD_ZERO (&ws);
         FD_ZERO (&es);
-        if (d4 && MHD_YES != MHD_get_fdset (d4, &rs, &ws, &es, &max)) {
+        if (d4 && MHD_YES != MHD_get_fdset (d4, &rs, &ws, &es, &maxsock)) {
             break;		/* fatal internal error */
         }
-        if (d6 && MHD_YES != MHD_get_fdset (d6, &rs, &ws, &es, &max)) {
+        if (d6 && MHD_YES != MHD_get_fdset (d6, &rs, &ws, &es, &maxsock)) {
             break;		/* fatal internal error */
         }
 
@@ -755,7 +843,7 @@ main (int argc, char *argv[])
             tv.tv_sec = dumpstats;
         }
 
-        select (max + 1, &rs, &ws, &es, &tv);
+        select (maxsock + 1, &rs, &ws, &es, &tv);
 
         if (d4) {
             MHD_run (d4);
