@@ -1,7 +1,7 @@
 # pylint: disable=C0103
 """Wrapper module for libpcp_pmda - Performace Co-Pilot Domain Agent API
 #
-# Copyright (C) 2013-2014 Red Hat.
+# Copyright (C) 2013-2015 Red Hat.
 #
 # This file is part of the "pcp" module, the python interfaces for the
 # Performance Co-Pilot toolkit.
@@ -10,7 +10,7 @@
 # under the terms of the GNU General Public License as published by the
 # Free Software Foundation; either version 2 of the License, or (at your
 # option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 # or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -25,7 +25,7 @@ import os
 import cpmapi
 import cpmda
 from pcp.pmapi import pmContext as PCP
-from pcp.pmapi import pmInDom, pmDesc, pmUnits
+from pcp.pmapi import pmInDom, pmDesc, pmUnits, pmErr
 
 from ctypes.util import find_library
 from ctypes import CDLL, c_int, c_long, c_char_p, c_void_p, cast, byref
@@ -55,6 +55,8 @@ LIBPCP_PMDA.pmdaCacheLookupKey.argtypes = [
        POINTER(c_int), POINTER(c_void_p)]
 LIBPCP_PMDA.pmdaCacheOp.restype = c_int
 LIBPCP_PMDA.pmdaCacheOp.argtypes = [pmInDom, c_long]
+LIBPCP_PMDA.pmdaCacheResize.restype = c_int
+LIBPCP_PMDA.pmdaCacheResize.argtypes = [pmInDom, c_int]
 
 
 ##
@@ -86,7 +88,7 @@ class pmdaInstid(Structure):
     def __init__(self, instid, name):
         Structure.__init__(self)
         self.i_inst = instid
-        self.i_name = name
+        self.i_name = name.encode('utf-8')
 
     def __str__(self):
         return "pmdaInstid@%#lx index=%d name=%s" % (addressof(self), self.i_inst, self.i_name)
@@ -104,6 +106,39 @@ class pmdaIndom(Structure):
         self.it_indom = indom
         self.set_instances(indom, insts)
 
+    def __iter__(self):
+        # Generates an iterator for the cache.
+        if self.it_numinst <= 0:
+            LIBPCP_PMDA.pmdaCacheOp(self.it_indom,
+                                    cpmda.PMDA_CACHE_WALK_REWIND)
+            while 1:
+                inst = LIBPCP_PMDA.pmdaCacheOp(self.it_indom,
+                                               cpmda.PMDA_CACHE_WALK_NEXT)
+                if inst < 0:
+                    break
+                name = self.inst_name_lookup(inst)
+                if name:
+                    yield (inst, name)
+        else:
+            for i in range(self.it_numinst):
+                inst = self.it_set[i].i_inst
+                name = self.inst_name_lookup(inst)
+                if name:
+                    yield (inst, name)
+
+    def inst_name_lookup(self, instance):
+        if self.it_numinst <= 0:
+            name = (c_char_p)()
+            sts = LIBPCP_PMDA.pmdaCacheLookup(self.it_indom, instance,
+                                              byref(name), None)
+            if (sts == cpmda.PMDA_CACHE_ACTIVE):
+                return str(name.value.decode())
+        elif self.it_numinst > 0:
+            for inst in self.it_set:
+                if (inst.i_inst == instance):
+                    return str(inst.i_name.decode())
+        return None
+
     def set_list_instances(self, insts):
         instance_count = len(insts)
         if (instance_count == 0):
@@ -113,12 +148,15 @@ class pmdaIndom(Structure):
             instance_array[i].i_inst = insts[i].i_inst
             instance_array[i].i_name = insts[i].i_name
         self.it_set = instance_array
+        self.it_numinst = instance_count
 
     def set_dict_instances(self, indom, insts):
         LIBPCP_PMDA.pmdaCacheOp(indom, cpmda.PMDA_CACHE_INACTIVE)
         for key in insts.keys():
-            LIBPCP_PMDA.pmdaCacheStore(indom, cpmda.PMDA_CACHE_ADD, key, byref(insts[key]))
+            key8 = key.encode('utf-8')
+            LIBPCP_PMDA.pmdaCacheStore(indom, cpmda.PMDA_CACHE_ADD, key8, byref(insts[key]))
         LIBPCP_PMDA.pmdaCacheOp(indom, cpmda.PMDA_CACHE_SAVE)
+        self.it_numinst = -1
 
     def set_instances(self, indom, insts):
         if (insts == None):
@@ -133,6 +171,34 @@ class pmdaIndom(Structure):
     def __str__(self):
         return "pmdaIndom@%#lx indom=%#lx num=%d" % (addressof(self), self.it_indom, self.it_numinst)
 
+    def cache_load(self):
+        if self.it_numinst <= 0:
+            sts = LIBPCP_PMDA.pmdaCacheOp(self.it_indom, cpmda.PMDA_CACHE_LOAD)
+            if sts < 0:
+                raise pmErr(sts)
+        else:
+            raise pmErr(cpmapi.PM_ERR_NYI)
+
+    def cache_mark_active(self):
+        if self.it_numinst <= 0:
+            LIBPCP_PMDA.pmdaCacheOp(self.it_indom, cpmda.PMDA_CACHE_ACTIVE)
+        else:
+            raise pmErr(cpmapi.PM_ERR_NYI)
+
+    def cache_mark_inactive(self):
+        if self.it_numinst <= 0:
+            LIBPCP_PMDA.pmdaCacheOp(self.it_indom, cpmda.PMDA_CACHE_INACTIVE)
+        else:
+            raise pmErr(cpmapi.PM_ERR_NYI)
+
+    def cache_resize(self, maximum):
+        if self.it_numinst <= 0:
+            sts = LIBPCP_PMDA.pmdaCacheResize(self.it_indom, maximum)
+            if sts < 0:
+                raise pmErr(sts)
+        else:
+            raise pmErr(cpmapi.PM_ERR_NYI)
+
 class pmdaUnits(pmUnits):
     """ Wrapper class for PMDAs defining their metrics (avoids pmapi import) """
     def __init__(self, dimS, dimT, dimC, scaleS, scaleT, scaleC):
@@ -160,22 +226,48 @@ class MetricDispatch(object):
     # overloads
 
     def __init__(self, domain, name, logfile, helpfile):
-        self.clear_indoms()
-        self.clear_metrics()
-        cpmda.init_dispatch(domain, name, logfile, helpfile)
-
-    def clear_indoms(self):
         self._indomtable = []
         self._indoms = {}
         self._indom_oneline = {}
         self._indom_helptext = {}
-
-    def clear_metrics(self):
         self._metrictable = []
         self._metrics = {}
         self._metric_names = {}
+        self._metric_names_map = {}
         self._metric_oneline = {}
         self._metric_helptext = {}
+        cpmda.init_dispatch(domain, name, logfile, helpfile)
+
+    def clear_indoms(self):
+        # Notice we're using the following:
+        #
+        #   del list[:]
+        #   dict.clear()
+        #
+        # instead of:
+        #
+        #   list = []
+        #   dict = {}
+        #
+        # The latter creates a new list/dictionary and assigns it to
+        # the variable. But, the stashed list/dictionary reference
+        # down in cpmda still will point to the original
+        # list/dictionary. Clearing out the existing list/dictionary
+        # keeps the stashed references working the way we'd like.
+        del self._indomtable[:]
+        self._indoms.clear()
+        self._indom_oneline.clear()
+        self._indom_helptext.clear()
+
+    def clear_metrics(self):
+        # See note above in clear_indoms() about clearing
+        # lists/dictionaries.
+        del self._metrictable[:]
+        self._metrics.clear()
+        self._metric_names.clear()
+        self._metric_names_map.clear()
+        self._metric_oneline.clear()
+        self._metric_helptext.clear()
 
     def reset_metrics(self):
         self.clear_metrics()
@@ -192,18 +284,34 @@ class MetricDispatch(object):
 
     def add_metric(self, name, metric, oneline = '', text = ''):
         pmid = metric.m_desc.pmid
-        if (pmid in self._metric_names):
-            raise KeyError('attempt to add_metric with an existing name')
+        if (name in self._metric_names_map):
+            raise KeyError('attempt to add_metric with an existing name=%s' % (name))
         if (pmid in self._metrics):
             raise KeyError('attempt to add_metric with an existing PMID')
 
         self._metrictable.append(metric)
         self._metrics[pmid] = metric
         self._metric_names[pmid] = name
+        self._metric_names_map[name] = pmid
         self._metric_oneline[pmid] = oneline
         self._metric_helptext[pmid] = text
         cpmda.set_need_refresh()
 
+    def remove_metric(self, name, metric):
+        pmid = metric.m_desc.pmid
+        if (name not in self._metric_names_map):
+            raise KeyError('attempt to remove non-existant metric name=%s' % (name))
+        if (pmid not in self._metrics):
+            raise KeyError('attempt to remove_metric with non-existant PMID')
+
+        self._metrictable.remove(metric)
+        self._metrics.pop(pmid)
+        self._metric_names.pop(pmid)
+        self._metric_names_map.pop(name)
+        self._metric_oneline.pop(pmid)
+        self._metric_helptext.pop(pmid)
+        cpmda.set_need_refresh()
+
     def add_indom(self, indom, oneline = '', text = ''):
         indomid = indom.it_indom
         for entry in self._indomtable:
@@ -215,13 +323,21 @@ class MetricDispatch(object):
         self._indom_helptext[indomid] = text
 
     def replace_indom(self, indom, insts):
-        replacement = pmdaIndom(indom, insts)
+        # Note that this function can take a numeric indom or a
+        # pmdaIndom.
+        if isinstance(indom, pmdaIndom):
+            it_indom = indom.it_indom
+            replacement = indom
+        else:
+            it_indom = indom
+            replacement = pmdaIndom(it_indom, insts)
         # list indoms need to keep the table up-to-date for libpcp_pmda
         if (isinstance(insts, list)):
             for entry in self._indomtable:
-                if (entry.it_indom == indom):
+                if (entry.it_indom == it_indom):
                     entry = replacement
-        self._indoms[indom] = replacement
+                    break
+        self._indoms[it_indom] = replacement
 
     def inst_lookup(self, indom, instance):
         """
@@ -243,17 +359,7 @@ class MetricDispatch(object):
         a specific instance domain.
         """
         entry = self._indoms[indom]
-        if (entry.it_numinst < 0):
-            name = (c_char_p)()
-            sts = LIBPCP_PMDA.pmdaCacheLookup(indom, instance, byref(name), None)
-            if (sts == cpmda.PMDA_CACHE_ACTIVE):
-                return name.value
-        elif (entry.it_numinst > 0 and entry.it_indom == indom):
-            for inst in entry.it_set:
-                if (inst.i_inst == instance):
-                    return inst.i_name
-        return None
-
+        return entry.inst_name_lookup(instance)
 
 class PMDA(MetricDispatch):
     """ Defines a PCP performance metrics domain agent
@@ -333,17 +439,7 @@ class PMDA(MetricDispatch):
             cpmda.pmid_longtext_refresh(self._metric_helptext)
             cpmda.indom_oneline_refresh(self._indom_oneline)
             cpmda.indom_longtext_refresh(self._indom_helptext)
-            numindoms = len(self._indomtable)
-            ibuf = create_string_buffer(numindoms * sizeof(pmdaIndom))
-            indoms = cast(ibuf, POINTER(pmdaIndom))
-            for i in range(numindoms):
-                indoms[i] = self._indomtable[i]
-            nummetrics = len(self._metrictable)
-            mbuf = create_string_buffer(nummetrics * sizeof(pmdaMetric))
-            metrics = cast(mbuf, POINTER(pmdaMetric))
-            for i in range(nummetrics):
-                metrics[i] = self._metrictable[i]
-            cpmda.pmda_dispatch(ibuf.raw, numindoms, mbuf.raw, nummetrics)
+            cpmda.pmda_dispatch(self._indomtable, self._metrictable)
 
     @staticmethod
     def set_fetch(fetch):
@@ -365,6 +461,14 @@ class PMDA(MetricDispatch):
     def set_store_callback(store_callback):
         return cpmda.set_store_callback(store_callback)
 
+    @staticmethod
+    def set_refresh_all(refresh_all):
+        return cpmda.set_refresh_all(refresh_all)
+
+    @staticmethod
+    def set_refresh_metrics(refresh_metrics):
+        return cpmda.set_refresh_metrics(refresh_metrics)
+
     @staticmethod
     def set_user(username):
         return cpmapi.pmSetProcessIdentity(username)
@@ -404,4 +508,4 @@ class PMDA(MetricDispatch):
 #    set_unix_socket
 #    pmda_pmid_name(cluster,item)
 #    pmda_pmid_text(cluster,item)
-#    
+#
