@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Red Hat.
+ * Copyright (C) 2012-2017 Red Hat.
  * Copyright (C) 2009-2012 Michael T. Werner
  *
  * This file is part of the "pcp" module, the python interfaces for the
@@ -45,6 +45,8 @@
 #endif
 
 static pmOptions options;
+static char **argVector;
+static int argCount;
 static int longOptionsCount;
 static PyObject *optionCallback;
 static PyObject *overridesCallback;
@@ -157,6 +159,31 @@ setIdentity(PyObject *self, PyObject *args, PyObject *keywords)
     return Py_BuildValue("i", __pmSetProcessIdentity(name));
 }
 
+static PyObject *
+makeTime(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    struct tm tm;
+    int gmtoff = 0;
+    char *zone = NULL;
+    char *keyword_list[] = {"tm_sec", "tm_min", "tm_hour",
+			    "tm_mday", "tm_mon", "tm_year",
+			    "tm_wday", "tm_yday", "tm_isdst",
+			    "tm_gmtoff", "tm_zone", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, keywords,
+			"iiiiiiiiils:pmMktime", keyword_list,
+			&tm.tm_sec, &tm.tm_min, &tm.tm_hour,
+			&tm.tm_mday, &tm.tm_mon, &tm.tm_year,
+			&tm.tm_wday, &tm.tm_yday, &tm.tm_isdst,
+			&gmtoff, &zone))
+	return NULL;
+#ifdef IS_LINUX
+    tm.tm_gmtoff = gmtoff;
+    tm.tm_zone = zone;
+#endif
+    return Py_BuildValue("l", __pmMktime(&tm));
+}
+
 /*
  * Common command line option handling code - wrapping pmOptions
  */
@@ -309,16 +336,16 @@ setLongOptionArchiveFolio(PyObject *self, PyObject *args)
 }
 
 static PyObject *
-setLongOptionDebug(PyObject *self, PyObject *args)
+setLongOptionContainer(PyObject *self, PyObject *args)
 {
-    pmLongOptions option = PMOPT_DEBUG;
+    pmLongOptions option = PMOPT_CONTAINER;
     return addLongOptionObject(&option);
 }
 
 static PyObject *
-setLongOptionGuiMode(PyObject *self, PyObject *args)
+setLongOptionDebug(PyObject *self, PyObject *args)
 {
-    pmLongOptions option = PMOPT_GUIMODE;
+    pmLongOptions option = PMOPT_DEBUG;
     return addLongOptionObject(&option);
 }
 
@@ -357,13 +384,6 @@ setLongOptionOrigin(PyObject *self, PyObject *args)
     return addLongOptionObject(&option);
 }
 
-static PyObject *
-setLongOptionGuiPort(PyObject *self, PyObject *args)
-{
-    pmLongOptions option = PMOPT_GUIPORT;
-    return addLongOptionObject(&option);
-}
-
 static PyObject *
 setLongOptionStart(PyObject *self, PyObject *args)
 {
@@ -467,6 +487,21 @@ setShortUsage(PyObject *self, PyObject *args, PyObject *keywords)
     return Py_None;
 }
 
+static PyObject *
+setOptionContext(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    int context;
+    char *keyword_list[] = {"context", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, keywords,
+                        "i:pmSetOptionContext", keyword_list, &context))
+        return NULL;
+
+    options.context = context;
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject *
 setOptionFlags(PyObject *self, PyObject *args, PyObject *keywords)
 {
@@ -512,6 +547,58 @@ setOptionArchiveList(PyObject *self, PyObject *args, PyObject *keywords)
     return Py_None;
 }
 
+static PyObject *
+setOptionArchive(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    char *archive;
+    char *keyword_list[] = {"archive", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, keywords,
+			"s:pmSetOptionArchive", keyword_list, &archive))
+	return NULL;
+
+    if ((archive = strdup(archive ? archive : "")) == NULL)
+	return PyErr_NoMemory();
+    __pmAddOptArchive(&options, archive);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+setOptionContainer(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    char *container;
+    char *keyword_list[] = {PMLONGOPT_CONTAINER, NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, keywords,
+                        "s:pmSetOptionContainer", keyword_list, &container))
+        return NULL;
+
+    if ((container = strdup(container ? container : "")) == NULL)
+	return PyErr_NoMemory();
+    __pmAddOptContainer(&options, container);
+    free(container);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+setOptionHost(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    char *host;
+    char *keyword_list[] = {"host", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, keywords,
+			"s:pmSetOptionHost", keyword_list, &host))
+	return NULL;
+
+    if ((host = strdup(host ? host : "")) == NULL)
+	return PyErr_NoMemory();
+    __pmAddOptHost(&options, host);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject *
 setOptionHostList(PyObject *self, PyObject *args, PyObject *keywords)
 {
@@ -527,6 +614,32 @@ setOptionHostList(PyObject *self, PyObject *args, PyObject *keywords)
     return Py_None;
 }
 
+static PyObject *
+setOptionSpecLocal(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    char *spec;
+    char *keyword_list[] = {"spec", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, keywords,
+			"s:pmSetOptionSpecLocal", keyword_list, &spec))
+	return NULL;
+
+    if ((spec = strdup(spec ? spec : "")) == NULL)
+	return PyErr_NoMemory();
+    __pmSetLocalContextTable(&options, spec);
+    free(spec);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+setOptionLocalPMDA(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    __pmSetLocalContextFlag(&options);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject *
 setOptionSamples(PyObject *self, PyObject *args, PyObject *keywords)
 {
@@ -632,18 +745,48 @@ options_callback(int opt, pmOptions *opts)
     }
 }
 
+/*
+ * Access command line operands in a way that handles the reordering
+ * that can happen via pmgetopt_r(3) in non-POSIXLY_CORRECT mode.
+ */
+static PyObject *
+getOperands(PyObject *self, PyObject *args)
+{
+    PyObject *result;
+    int i, length = 0;
+
+    /* Caller must perform pmGetOptions before running this, check */
+    if (!(options.flags & PM_OPTFLAG_DONE)) {
+	PyErr_SetString(PyExc_RuntimeError, "pmGetOptions is not yet done");
+	return NULL;
+    }
+
+    if (argCount > 0)
+	length = argCount - options.optind;
+    if (length <= 0) {
+	Py_INCREF(Py_None);
+	return Py_None;
+    }
+
+    if ((result = PyList_New(length)) == NULL)
+	return PyErr_NoMemory();
+
+    for (i = 0; i < length; i++) {
+	PyObject *pyarg = Py_BuildValue("s", argVector[options.optind + i]);
+	Py_INCREF(pyarg);
+	PyList_SET_ITEM(result, i, pyarg);
+    }
+    Py_INCREF(result);
+    return result;
+}
+
+/* backward compatibility only, use the getOperands interface now */
 static PyObject *
 getNonOptionsFromList(PyObject *self, PyObject *args, PyObject *keywords)
 {
-    int argc, length, i;
-    PyObject *result;
     PyObject *pyargv = NULL;
     char *keyword_list[] = {"argv", NULL};
 
-    /* Caller must perform pmGetOptions before running this, check */
-    if (!(options.flags & PM_OPTFLAG_DONE))
-	return NULL;
-
     if (!PyArg_ParseTupleAndKeywords(args, keywords,
 			"O:pmGetNonOptionsFromList", keyword_list, &pyargv))
     if (pyargv == NULL)
@@ -654,31 +797,13 @@ getNonOptionsFromList(PyObject *self, PyObject *args, PyObject *keywords)
 	return NULL;
     }
 
-    length = 0;
-    if ((argc = PyList_GET_SIZE(pyargv)) > 0)
-	length = argc - options.optind;
-
-    if (length <= 0) {
-	Py_INCREF(Py_None);
-	return Py_None;
-    }
-
-    if ((result = PyList_New(length)) == NULL)
-	return PyErr_NoMemory();
-
-    for (i = 0; i < length; i++) {
-	PyObject *pyarg = PyList_GET_ITEM(pyargv, options.optind + i);
-	PyList_SET_ITEM(result, i, pyarg);
-    }
-    Py_INCREF(result);
-    return result;
+    return getOperands(self, args);
 }
 
 static PyObject *
 getOptionsFromList(PyObject *self, PyObject *args, PyObject *keywords)
 {
-    int i, argc;
-    char **argv;
+    int i;
     PyObject *pyargv = NULL;
     char *keyword_list[] = {"argv", NULL};
 
@@ -697,15 +822,15 @@ getOptionsFromList(PyObject *self, PyObject *args, PyObject *keywords)
 	return NULL;
     }
 
-    if ((argc = PyList_GET_SIZE(pyargv)) <= 0) {
+    if ((argCount = PyList_GET_SIZE(pyargv)) <= 0)
 	return Py_BuildValue("i", 0);
-    }
 
-    if ((argv = malloc(argc * sizeof(char *))) == NULL) {
+    if ((argVector = malloc(argCount * sizeof(char *))) == NULL) {
+	argCount = 0;
 	return PyErr_NoMemory();
     }
 
-    for (i = 0; i < argc; i++) {
+    for (i = 0; i < argCount; i++) {
 	PyObject *pyarg = PyList_GET_ITEM(pyargv, i);
 #if PY_MAJOR_VERSION >= 3
 	char *string = PyUnicode_AsUTF8(pyarg);
@@ -713,21 +838,23 @@ getOptionsFromList(PyObject *self, PyObject *args, PyObject *keywords)
 	char *string = PyString_AsString(pyarg);
 #endif
 
-	/* argv[0] parameter will be used for pmProgname, so need to
-	 * ensure the memory that backs it will be with us forever.
+	/* All parameters may be referred back to later, e.g. via
+	 * pmProgname or getOperands (and others), so we need to
+	 * allocate the memory to hold these strings permanently.
          */
-	if (i == 0 && (string = strdup(string)) == NULL) {
-	    free(argv);
+	if ((string = strdup(string)) == NULL) {
+	    free(argVector);
+	    argCount = 0;
+	    argVector = NULL;
 	    return PyErr_NoMemory();
 	}
-	argv[i] = string;
+	argVector[i] = string;
     }
 
     if (overridesCallback)
 	options.override = override_callback;
-    while ((i = pmGetOptions(argc, argv, &options)) != -1)
+    while ((i = pmGetOptions(argCount, argVector, &options)) != -1)
 	options_callback(i, &options);
-    free(argv);
 
     if (options.flags & PM_OPTFLAG_EXIT)
 	return Py_BuildValue("i", PM_ERR_APPVERSION);
@@ -735,6 +862,14 @@ getOptionsFromList(PyObject *self, PyObject *args, PyObject *keywords)
     return Py_BuildValue("i", options.errors);
 }
 
+static PyObject *
+endOptions(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    __pmEndOptions(&options);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject *
 setContextOptions(PyObject *self, PyObject *args, PyObject *keywords)
 {
@@ -774,10 +909,52 @@ setContextOptions(PyObject *self, PyObject *args, PyObject *keywords)
     return Py_BuildValue("i", sts);
 }
 
+static void
+pmnsDecodeCallback(const char *name, void *closure)
+{
+    PyObject *arglist, *result;
+
+    arglist = Py_BuildValue("(s)", name);
+    if (arglist == NULL)
+        return;
+    result = PyEval_CallObject(closure, arglist);
+    Py_DECREF(arglist);
+    if (!result)
+        PyErr_Print();
+    else
+	Py_DECREF(result);
+}
+
+/*
+ * This pmTraversePMNS_r wrapper is specifically so that python3
+ * installs can pass out correctly decoded python strings rather
+ * than byte arrays.
+ */
+static PyObject *
+pmnsTraverse(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    PyObject *func;
+    char *keyword_list[] = {"name", "callback", NULL};
+    char *name;
+    int sts;
+
+    if (!PyArg_ParseTupleAndKeywords(args, keywords,
+		"sO:pmnsTraverse", keyword_list, &name, &func))
+	return NULL;
+    if (!PyCallable_Check(func)) {
+	PyErr_SetString(PyExc_TypeError, "pmnsTraverse needs a callable");
+	return NULL;
+    }
+    sts = pmTraversePMNS_r(name, pmnsDecodeCallback, func);
+    return Py_BuildValue("i", sts);
+}
+
 static PyObject *
 usageMessage(PyObject *self, PyObject *args)
 {
     pmUsageMessage(&options);
+    if (options.flags & PM_OPTFLAG_EXIT)
+	exit(0);
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -907,6 +1084,15 @@ getOptionStart_usec(PyObject *self, PyObject *args)
     return Py_None;
 }
 
+static PyObject *
+getOptionAlign_optarg(PyObject *self, PyObject *args)
+{
+    if (options.align_optarg)
+	return Py_BuildValue("s", options.align_optarg);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject *
 getOptionFinish_optarg(PyObject *self, PyObject *args)
 {
@@ -988,6 +1174,28 @@ getOptionTimezone(PyObject *self, PyObject *args)
     return Py_None;
 }
 
+static PyObject *
+getOptionHostZone(PyObject *self, PyObject *args)
+{
+    return Py_BuildValue("i", options.tzflag);
+}
+
+static PyObject *
+getOptionContainer(PyObject *self, PyObject *args)
+{
+    char *container;
+
+    if ((container = getenv("PCP_CONTAINER")) != NULL)
+	return Py_BuildValue("s", strdup(container));
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+getOptionLocalPMDA(PyObject *self, PyObject *args)
+{
+    return Py_BuildValue("i", options.Lflag);
+}
 
 static PyMethodDef methods[] = {
     { .ml_name = "PM_XTB_SET",
@@ -1005,6 +1213,9 @@ static PyMethodDef methods[] = {
     { .ml_name = "pmSetProcessIdentity",
 	.ml_meth = (PyCFunction) setIdentity,
         .ml_flags = METH_VARARGS | METH_KEYWORDS },
+    { .ml_name = "pmMktime",
+	.ml_meth = (PyCFunction) makeTime,
+        .ml_flags = METH_VARARGS | METH_KEYWORDS },
     { .ml_name = "pmResetAllOptions",
 	.ml_meth = (PyCFunction) resetAllOptions,
         .ml_flags = METH_NOARGS },
@@ -1029,12 +1240,12 @@ static PyMethodDef methods[] = {
     { .ml_name = "pmSetLongOptionArchiveFolio",
 	.ml_meth = (PyCFunction) setLongOptionArchiveFolio,
         .ml_flags = METH_NOARGS },
+    { .ml_name = "pmSetLongOptionContainer",
+        .ml_meth = (PyCFunction) setLongOptionContainer,
+        .ml_flags = METH_NOARGS },
     { .ml_name = "pmSetLongOptionDebug",
 	.ml_meth = (PyCFunction) setLongOptionDebug,
         .ml_flags = METH_NOARGS },
-    { .ml_name = "pmSetLongOptionGuiMode",
-	.ml_meth = (PyCFunction) setLongOptionGuiMode,
-        .ml_flags = METH_NOARGS },
     { .ml_name = "pmSetLongOptionHost",
 	.ml_meth = (PyCFunction) setLongOptionHost,
         .ml_flags = METH_NOARGS },
@@ -1053,9 +1264,6 @@ static PyMethodDef methods[] = {
     { .ml_name = "pmSetLongOptionOrigin",
 	.ml_meth = (PyCFunction) setLongOptionOrigin,
         .ml_flags = METH_NOARGS },
-    { .ml_name = "pmSetLongOptionGuiPort",
-	.ml_meth = (PyCFunction) setLongOptionGuiPort,
-        .ml_flags = METH_NOARGS },
     { .ml_name = "pmSetLongOptionStart",
 	.ml_meth = (PyCFunction) setLongOptionStart,
         .ml_flags = METH_NOARGS },
@@ -1086,6 +1294,9 @@ static PyMethodDef methods[] = {
     { .ml_name = "pmSetShortUsage",
 	.ml_meth = (PyCFunction) setShortUsage,
         .ml_flags = METH_VARARGS | METH_KEYWORDS },
+    { .ml_name = "pmSetOptionContext",
+        .ml_meth = (PyCFunction) setOptionContext,
+        .ml_flags = METH_VARARGS | METH_KEYWORDS },
     { .ml_name = "pmSetOptionFlags",
 	.ml_meth = (PyCFunction) setOptionFlags,
         .ml_flags = METH_VARARGS | METH_KEYWORDS },
@@ -1101,9 +1312,15 @@ static PyMethodDef methods[] = {
     { .ml_name = "pmGetOptionsFromList",
 	.ml_meth = (PyCFunction) getOptionsFromList,
         .ml_flags = METH_VARARGS | METH_KEYWORDS },
+    { .ml_name = "pmGetOperands",
+	.ml_meth = (PyCFunction) getOperands,
+        .ml_flags = METH_NOARGS },
     { .ml_name = "pmGetNonOptionsFromList",
 	.ml_meth = (PyCFunction) getNonOptionsFromList,
         .ml_flags = METH_VARARGS | METH_KEYWORDS },
+    { .ml_name = "pmEndOptions",
+        .ml_meth = (PyCFunction) endOptions,
+        .ml_flags = METH_NOARGS },
     { .ml_name = "pmSetContextOptions",
 	.ml_meth = (PyCFunction) setContextOptions,
         .ml_flags = METH_VARARGS | METH_KEYWORDS},
@@ -1131,6 +1348,9 @@ static PyMethodDef methods[] = {
     { .ml_name = "pmGetOptionStart_usec",
 	.ml_meth = (PyCFunction) getOptionStart_usec,
         .ml_flags = METH_NOARGS },
+    { .ml_name = "pmGetOptionAlign_optarg",
+	.ml_meth = (PyCFunction) getOptionAlign_optarg,
+        .ml_flags = METH_NOARGS },
     { .ml_name = "pmGetOptionFinish_optarg",
 	.ml_meth = (PyCFunction) getOptionFinish_optarg,
         .ml_flags = METH_NOARGS },
@@ -1164,15 +1384,42 @@ static PyMethodDef methods[] = {
     { .ml_name = "pmGetOptionTimezone",
 	.ml_meth = (PyCFunction) getOptionTimezone,
         .ml_flags = METH_NOARGS },
+    { .ml_name = "pmGetOptionHostZone",
+	.ml_meth = (PyCFunction) getOptionHostZone,
+        .ml_flags = METH_NOARGS },
+    { .ml_name = "pmGetOptionContainer",
+	.ml_meth = (PyCFunction) getOptionContainer,
+        .ml_flags = METH_NOARGS },
+    { .ml_name = "pmGetOptionLocalPMDA",
+	.ml_meth = (PyCFunction) getOptionLocalPMDA,
+        .ml_flags = METH_NOARGS },
+    { .ml_name = "pmSetOptionArchive",
+	.ml_meth = (PyCFunction) setOptionArchive,
+        .ml_flags = METH_VARARGS | METH_KEYWORDS },
     { .ml_name = "pmSetOptionArchiveList",
 	.ml_meth = (PyCFunction) setOptionArchiveList,
         .ml_flags = METH_VARARGS | METH_KEYWORDS },
     { .ml_name = "pmSetOptionArchiveFolio",
 	.ml_meth = (PyCFunction) setOptionArchiveFolio,
         .ml_flags = METH_VARARGS | METH_KEYWORDS },
+    { .ml_name = "pmSetOptionContainer",
+        .ml_meth = (PyCFunction) setOptionContainer,
+        .ml_flags = METH_VARARGS | METH_KEYWORDS },
+    { .ml_name = "pmSetOptionHost",
+	.ml_meth = (PyCFunction) setOptionHost,
+        .ml_flags = METH_VARARGS | METH_KEYWORDS },
     { .ml_name = "pmSetOptionHostList",
 	.ml_meth = (PyCFunction) setOptionHostList,
         .ml_flags = METH_VARARGS | METH_KEYWORDS },
+    { .ml_name = "pmSetOptionSpecLocal",
+        .ml_meth = (PyCFunction) setOptionSpecLocal,
+        .ml_flags = METH_VARARGS | METH_KEYWORDS },
+    { .ml_name = "pmSetOptionLocalPMDA",
+        .ml_meth = (PyCFunction) setOptionLocalPMDA,
+        .ml_flags = METH_VARARGS | METH_KEYWORDS },
+    { .ml_name = "pmnsTraverse",
+	.ml_meth = (PyCFunction) pmnsTraverse,
+        .ml_flags = METH_VARARGS | METH_KEYWORDS },
     { NULL }
 };
 
@@ -1206,6 +1453,8 @@ MOD_INIT(cpmapi)
     dict_add(dict, "HAVE_BITFIELDS_LTOR", 0);
     dict_add(dict, "HAVE_BITFIELDS_RTOL", 1);
 #endif
+    dict_add(dict, "PM_SIZEOF_SUSECONDS_T", PM_SIZEOF_SUSECONDS_T);
+    dict_add(dict, "PM_SIZEOF_TIME_T", PM_SIZEOF_TIME_T);
 
     dict_add(dict, "PM_SPACE_BYTE", PM_SPACE_BYTE);
     dict_add(dict, "PM_SPACE_KBYTE", PM_SPACE_KBYTE);
@@ -1252,6 +1501,7 @@ MOD_INIT(cpmapi)
     dict_add(dict, "PM_CONTEXT_ARCHIVE", PM_CONTEXT_ARCHIVE);
     dict_add(dict, "PM_CONTEXT_LOCAL", PM_CONTEXT_LOCAL);
     dict_add(dict, "PM_CONTEXT_TYPEMASK", PM_CONTEXT_TYPEMASK);
+    dict_add(dict, "PM_CTXFLAG_EXCLUSIVE", PM_CTXFLAG_EXCLUSIVE);
     dict_add(dict, "PM_CTXFLAG_SECURE", PM_CTXFLAG_SECURE);
     dict_add(dict, "PM_CTXFLAG_COMPRESS", PM_CTXFLAG_COMPRESS);
     dict_add(dict, "PM_CTXFLAG_RELAXED", PM_CTXFLAG_RELAXED);
@@ -1308,6 +1558,18 @@ MOD_INIT(cpmapi)
     dict_add(dict, "PM_EVENT_FLAG_PARENT", PM_EVENT_FLAG_PARENT);
     dict_add(dict, "PM_EVENT_FLAG_MISSED", PM_EVENT_FLAG_MISSED);
 
+    /*
+     * subset of the debug flags - all of 'em seems like overkill
+     * order here the same as the output from pmdbg -l
+     */
+    dict_add(dict, "PM_DEBUG_APPL0", DBG_TRACE_APPL0);
+    dict_add(dict, "PM_DEBUG_APPL1", DBG_TRACE_APPL1);
+    dict_add(dict, "PM_DEBUG_APPL2", DBG_TRACE_APPL2);
+
+    /*
+     * for ease of maintenance make the order of the error codes
+     * here the same as the output from pmerr -l
+     */
     edict_add(dict, edict, "PM_ERR_GENERIC", PM_ERR_GENERIC);
     edict_add(dict, edict, "PM_ERR_PMNS", PM_ERR_PMNS);
     edict_add(dict, edict, "PM_ERR_NOPMNS", PM_ERR_NOPMNS);
@@ -1322,6 +1584,7 @@ MOD_INIT(cpmapi)
     edict_add(dict, edict, "PM_ERR_PMID", PM_ERR_PMID);
     edict_add(dict, edict, "PM_ERR_INDOM", PM_ERR_INDOM);
     edict_add(dict, edict, "PM_ERR_INST", PM_ERR_INST);
+    edict_add(dict, edict, "PM_ERR_TYPE", PM_ERR_TYPE);
     edict_add(dict, edict, "PM_ERR_UNIT", PM_ERR_UNIT);
     edict_add(dict, edict, "PM_ERR_CONV", PM_ERR_CONV);
     edict_add(dict, edict, "PM_ERR_TRUNC", PM_ERR_TRUNC);
@@ -1334,8 +1597,8 @@ MOD_INIT(cpmapi)
     edict_add(dict, edict, "PM_ERR_MODE", PM_ERR_MODE);
     edict_add(dict, edict, "PM_ERR_LABEL", PM_ERR_LABEL);
     edict_add(dict, edict, "PM_ERR_LOGREC", PM_ERR_LOGREC);
-    edict_add(dict, edict, "PM_ERR_NOTARCHIVE", PM_ERR_NOTARCHIVE);
     edict_add(dict, edict, "PM_ERR_LOGFILE", PM_ERR_LOGFILE);
+    edict_add(dict, edict, "PM_ERR_NOTARCHIVE", PM_ERR_NOTARCHIVE);
     edict_add(dict, edict, "PM_ERR_NOCONTEXT", PM_ERR_NOCONTEXT);
     edict_add(dict, edict, "PM_ERR_PROFILESPEC", PM_ERR_PROFILESPEC);
     edict_add(dict, edict, "PM_ERR_PMID_LOG", PM_ERR_PMID_LOG);
@@ -1350,13 +1613,21 @@ MOD_INIT(cpmapi)
     edict_add(dict, edict, "PM_ERR_NOTCONN", PM_ERR_NOTCONN);
     edict_add(dict, edict, "PM_ERR_NEEDPORT", PM_ERR_NEEDPORT);
     edict_add(dict, edict, "PM_ERR_NONLEAF", PM_ERR_NONLEAF);
-    edict_add(dict, edict, "PM_ERR_TYPE", PM_ERR_TYPE);
-    edict_add(dict, edict, "PM_ERR_THREAD", PM_ERR_THREAD);
+    edict_add(dict, edict, "PM_ERR_PMDAREADY", PM_ERR_PMDAREADY);
+    edict_add(dict, edict, "PM_ERR_PMDANOTREADY", PM_ERR_PMDANOTREADY);
     edict_add(dict, edict, "PM_ERR_TOOSMALL", PM_ERR_TOOSMALL);
     edict_add(dict, edict, "PM_ERR_TOOBIG", PM_ERR_TOOBIG);
     edict_add(dict, edict, "PM_ERR_FAULT", PM_ERR_FAULT);
-    edict_add(dict, edict, "PM_ERR_PMDAREADY", PM_ERR_PMDAREADY);
-    edict_add(dict, edict, "PM_ERR_PMDANOTREADY", PM_ERR_PMDANOTREADY);
+    edict_add(dict, edict, "PM_ERR_THREAD", PM_ERR_THREAD);
+    edict_add(dict, edict, "PM_ERR_NOCONTAINER", PM_ERR_NOCONTAINER);
+    edict_add(dict, edict, "PM_ERR_BADSTORE", PM_ERR_BADSTORE);
+    edict_add(dict, edict, "PM_ERR_LOGOVERLAP", PM_ERR_LOGOVERLAP);
+    edict_add(dict, edict, "PM_ERR_LOGHOST", PM_ERR_LOGHOST);
+    edict_add(dict, edict, "PM_ERR_LOGCHANGETYPE", PM_ERR_LOGCHANGETYPE);
+    edict_add(dict, edict, "PM_ERR_LOGCHANGESEM", PM_ERR_LOGCHANGESEM);
+    edict_add(dict, edict, "PM_ERR_LOGCHANGEINDOM", PM_ERR_LOGCHANGEINDOM);
+    edict_add(dict, edict, "PM_ERR_LOGCHANGEUNITS", PM_ERR_LOGCHANGEUNITS);
+    edict_add(dict, edict, "PM_ERR_NEEDCLIENTCERT", PM_ERR_NEEDCLIENTCERT);
     edict_add(dict, edict, "PM_ERR_NYI", PM_ERR_NYI);
 
     return MOD_SUCCESS_VAL(module);
