@@ -47,6 +47,8 @@
 static pmdaInterface dispatch;
 static __pmnsTree *pmns;
 static int need_refresh;
+static PyObject *indom_list;	  	/* indom list */
+static PyObject *metric_list;	  	/* metric list */
 static PyObject *pmns_dict;		/* metric pmid:names dictionary */
 static PyObject *pmid_oneline_dict;	/* metric pmid:short text */
 static PyObject *pmid_longtext_dict;	/* metric pmid:long help */
@@ -58,11 +60,44 @@ static PyObject *refresh_func;
 static PyObject *instance_func;
 static PyObject *store_cb_func;
 static PyObject *fetch_cb_func;
+static PyObject *refresh_all_func;
+static PyObject *refresh_metrics_func;
+
+static Py_ssize_t nindoms;
+static pmdaIndom *indom_buffer;
+static Py_ssize_t nmetrics;
+static pmdaMetric *metric_buffer;
 
 #if PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION <= 5
 typedef int Py_ssize_t;
 #endif
 
+static void pmns_refresh(void);
+static void pmda_refresh_metrics(void);
+
+static void
+maybe_refresh_all(void)
+{
+    // Call the refresh metrics hook (if it exists).
+    if (refresh_metrics_func) {
+	PyObject *arglist, *result;
+
+	arglist = Py_BuildValue("()");
+	if (arglist == NULL)
+	    return;
+	result = PyEval_CallObject(refresh_metrics_func, arglist);
+	Py_DECREF(arglist);
+	// Just ignore the result.
+	Py_DECREF(result);
+    }
+
+    if (need_refresh) {
+	pmns_refresh();
+	pmda_refresh_metrics();
+	need_refresh = 0;
+    }
+}
+
 static void
 pmns_refresh(void)
 {
@@ -73,6 +108,10 @@ pmns_refresh(void)
     if (pmDebug & DBG_TRACE_LIBPMDA)
         fprintf(stderr, "pmns_refresh: rebuilding namespace\n");
 
+    // If there is nothing to do, just exit.
+    if (pmns_dict == NULL)
+	return;
+
     if (pmns)
         __pmFreePMNS(pmns);
 
@@ -105,9 +144,6 @@ pmns_refresh(void)
     }
 
     pmdaTreeRebuildHash(pmns, count); /* for reverse (pmid->name) lookups */
-    Py_DECREF(pmns_dict);
-    need_refresh = 0;
-    pmns_dict = NULL;
 }
 
 static PyObject *
@@ -273,32 +309,28 @@ indom_longtext_refresh(PyObject *self, PyObject *args, PyObject *keywords)
 int
 pmns_desc(pmID pmid, pmDesc *desc, pmdaExt *ep)
 {
-    if (need_refresh)
-        pmns_refresh();
+    maybe_refresh_all();
     return pmdaDesc(pmid, desc, ep);
 }
 
 int
 pmns_pmid(const char *name, pmID *pmid, pmdaExt *pmda)
 {
-    if (need_refresh)
-        pmns_refresh();
+    maybe_refresh_all();
     return pmdaTreePMID(pmns, name, pmid);
 }
 
 int
 pmns_name(pmID pmid, char ***nameset, pmdaExt *pmda)
 {
-    if (need_refresh)
-        pmns_refresh();
+    maybe_refresh_all();
     return pmdaTreeName(pmns, pmid, nameset);
 }
 
 int
 pmns_children(const char *name, int traverse, char ***kids, int **sts, pmdaExt *pmda)
 {
-    if (need_refresh)
-        pmns_refresh();
+    maybe_refresh_all();
     return pmdaTreeChildren(pmns, name, traverse, kids, sts);
 }
 
@@ -338,6 +370,36 @@ refresh_cluster(int cluster)
     return 0;
 }
 
+static int
+refresh_all_clusters(int numclusters, int *clusters)
+{
+    PyObject *arglist, *result, *list;
+    int i;
+    list = PyList_New(numclusters);
+    if (list == NULL){
+        __pmNotifyErr(LOG_ERR, "refresh: Unable to allocate memory");
+        return 1;
+    }
+    for (i = 0; i < numclusters; i++) {
+        PyObject *num = PyLong_FromLong(clusters[i]);
+        PyList_SET_ITEM(list, i, num);
+    }
+
+    arglist = Py_BuildValue("(N)", list);
+    if (arglist == NULL){
+        return -ENOMEM;
+    }
+    result = PyEval_CallObject(refresh_all_func, arglist);
+    Py_DECREF(list);
+    Py_DECREF(arglist);
+    if (result == NULL) {
+        PyErr_Print();
+        return -EAGAIN; /* exception thrown */
+    }
+    Py_DECREF(result);
+    return 0;
+}
+
 static int
 refresh(int numpmid, pmID *pmidlist)
 {
@@ -364,8 +426,12 @@ refresh(int numpmid, pmID *pmidlist)
         if (j == count)
             clusters[count++] = cluster;
     }
-    for (j = 0; j < count; j++)
-        sts |= refresh_cluster(clusters[j]);
+    if (refresh_all_func)
+        sts |= refresh_all_clusters(count, clusters);
+    if (refresh_func) {
+        for (j = 0; j < count; j++)
+            sts |= refresh_cluster(clusters[j]);
+    }
     free(clusters);
     return sts;
 }
@@ -375,11 +441,11 @@ fetch(int numpmid, pmID *pmidlist, pmResult **rp, pmdaExt *pmda)
 {
     int sts;
 
-    if (need_refresh)
-        pmns_refresh();
+    maybe_refresh_all();
     if (fetch_func && (sts = prefetch()) < 0)
         return sts;
-    if (refresh_func && (sts = refresh(numpmid, pmidlist)) < 0)
+    if ((refresh_func || refresh_all_func) &&
+        (sts = refresh(numpmid, pmidlist)) < 0)
         return sts;
     return pmdaFetch(numpmid, pmidlist, rp, pmda);
 }
@@ -407,8 +473,7 @@ instance(pmInDom indom, int a, char *b, __pmInResult **rp, pmdaExt *pmda)
 {
     int sts;
 
-    if (need_refresh)
-        pmns_refresh();
+    maybe_refresh_all();
     if (instance_func && (sts = preinstance(indom)) < 0)
         return sts;
     return pmdaInstance(indom, a, b, rp, pmda);
@@ -476,21 +541,40 @@ fetch_callback(pmdaMetric *metric, unsigned int inst, pmAtomValue *atom)
         default:
             __pmNotifyErr(LOG_ERR, "unsupported metric type in fetch callback");
             sts = -ENOTSUP;
+	    rc = code = 1;		/* Don't fall into code below. */
+	    break;
     }
 
     if (!rc || !code) {    /* tuple not parsed or atom contains bad value */
+	/* If PyArg_Parse() failed above, it could be because the
+	 * calling code returned a error code tuple that didn't look
+	 * like what we were looking for. An error code tuple looks
+	 * like '[PM_ERR_VALUE, 0]' (for example). In this case, we
+	 * don't want PyArg_Parse() to raise an error, so we'll clear
+	 * it out. */
+	PyErr_Clear();
+
         if (!PyArg_Parse(result, "(ii):fetch_cb_error", &sts, &code)) {
             __pmNotifyErr(LOG_ERR, "extracting error code in fetch callback");
             sts = -EINVAL;
         }
+	/* If we got a code of 1, that's means the fetch
+	 * worked. However, if we're here, the fetch didn't really
+	 * work. For example, we could have been expecting a string
+	 * value and instead got a numeric value. So, force an
+	 * error. */
+	else if (code == 1) {
+            __pmNotifyErr(LOG_ERR, "forcing error code in fetch callback");
+	    sts = PM_ERR_TYPE;
+	}
     }
     Py_DECREF(result);
     return sts;
 }
 
-int 
+int
 store_callback(__pmID_int *pmid, unsigned int inst, pmAtomValue av, int type)
-{       
+{
     int rc, code;
     int item = pmid->item;
     int cluster = pmid->cluster;
@@ -565,8 +649,7 @@ store(pmResult *result, pmdaExt *pmda)
     pmValueSet  *vsp;
     __pmID_int  *pmid;
 
-    if (need_refresh)
-        pmns_refresh();
+    maybe_refresh_all();
 
     if (store_cb_func == NULL)
 	return PM_ERR_PERMISSION;
@@ -597,8 +680,7 @@ text(int ident, int type, char **buffer, pmdaExt *pmda)
 {
     PyObject *dict, *value, *key;
 
-    if (need_refresh)
-        pmns_refresh();
+    maybe_refresh_all();
 
     if ((type & PM_TEXT_PMID) != 0) {
 	if ((type & PM_TEXT_ONELINE) != 0)
@@ -723,138 +805,236 @@ connect_pmcd(void)
     return Py_None;
 }
 
-#ifdef PyBUF_SIMPLE
-static PyObject *
-pmda_dispatch(PyObject *self, PyObject *args)
+static int
+update_indom_metric_buffers(void)
 {
-    int nindoms, nmetrics;
-    PyObject *ibuf, *mbuf;
-    pmdaMetric *metrics;
-    pmdaIndom *indoms;
-    Py_buffer mv, iv;
+    Py_ssize_t i;
+    PyObject *item;
+    Py_buffer buffer;
+    const void *ptr;
+    Py_ssize_t len;
+    int error = 0;
 
-    if (!PyArg_ParseTuple(args, "OiOi", &ibuf, &nindoms, &mbuf, &nmetrics))
-        return NULL;
+    if (indom_list == NULL || metric_list == NULL)
+	return 1;
 
-    if (!PyObject_CheckBuffer(ibuf)) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch expected buffer 1st arg");
-        return NULL;
+    // If we have old data, free it up. We have to keep it around
+    // since pmdaRehash() doesn't copy data, it just points to it.
+    if (indom_buffer) {
+	free(indom_buffer);
+	indom_buffer = NULL;
     }
-    if (!PyObject_CheckBuffer(mbuf)) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch expected buffer 3rd arg");
-        return NULL;
+    if (metric_buffer) {
+	free(metric_buffer);
+	metric_buffer = NULL;
     }
 
-    if (PyObject_GetBuffer(ibuf, &iv, PyBUF_SIMPLE)) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch failed to get indoms");
-        return NULL;
-    }
-    if (PyObject_GetBuffer(mbuf, &mv, PyBUF_SIMPLE)) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch failed to get metrics");
-	PyBuffer_Release(&iv);
-        return NULL;
+    // Figure out how many indoms/metrics we've got.
+    nindoms = PyList_Size(indom_list);
+    nmetrics = PyList_Size(metric_list);
+
+    // Allocate buffers to hold all the indoms/metrics.
+    indom_buffer = nindoms ? calloc(nindoms, sizeof(pmdaIndom)) : NULL;
+    metric_buffer = nmetrics ? calloc(nmetrics, sizeof(pmdaMetric)) : NULL;
+    if ((nindoms > 0 && indom_buffer == NULL)
+	|| (nmetrics > 0 && metric_buffer == NULL)) {
+	PyErr_SetString(PyExc_TypeError, "Unable to allocate memory");
+	error = 1;
     }
 
-    if (iv.len != nindoms * sizeof(pmdaIndom)) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch: invalid indom array");
-	PyBuffer_Release(&iv);
-	PyBuffer_Release(&mv);
-        return NULL;
-    }
-    if (mv.len != nmetrics * sizeof(pmdaMetric)) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch: invalid metric array");
-	PyBuffer_Release(&iv);
-	PyBuffer_Release(&mv);
-        return NULL;
+    // Copy the indoms.
+    for (i = 0; !error && i < nindoms; i++) {
+	item = PyList_GetItem(indom_list, i);
+	/* Newer buffer interface */
+	if (item && PyObject_CheckBuffer(item)) {
+	    // Attempt to extract buffer information from it.
+	    if (PyObject_GetBuffer(item, &buffer, PyBUF_ANY_CONTIGUOUS) == -1) {
+		PyErr_SetString(PyExc_TypeError,
+				"Unable to get indom item buffer");
+		error = 1;
+		break;
+	    }
+	    ptr = buffer.buf;
+	    len = buffer.len;
+	}
+	/* Older buffer interface */
+	else if (item && PyObject_CheckReadBuffer(item)) {
+	    // Attempt to extract information from the item.
+	    if (PyObject_AsReadBuffer(item, &ptr, &len) == -1) {
+		PyErr_SetString(PyExc_TypeError,
+				"Unable to get indom item buffer");
+		error = 1;
+		break;
+	    }
+	    buffer.buf = NULL;
+	}
+	else {
+ 	    PyErr_SetString(PyExc_TypeError, "Unable to retrieve indom");
+	    error = 1;
+	    break;
+	}
+
+	// The indom table is supposed to be composed of
+	// 'pmdaIndom(Structure)' items, which should be laid out
+	// like a 'pmdaIndom' structure in memory.
+	if (len != sizeof(pmdaIndom)) {
+	    PyErr_SetString(PyExc_TypeError, "Invalid indom item size");
+	    if (buffer.buf)
+		PyBuffer_Release(&buffer);
+	    error = 1;
+	    break;
+	}
+	indom_buffer[i] = *(pmdaIndom *)ptr;
+	if (buffer.buf)
+	    PyBuffer_Release(&buffer);
     }
 
-    indoms = nindoms ? (pmdaIndom *)iv.buf : NULL;
-    metrics = nmetrics ? (pmdaMetric *)mv.buf : NULL;
-    if (pmDebug & DBG_TRACE_LIBPMDA)
-	fprintf(stderr, "pmda_dispatch pmdaInit for metrics/indoms\n");
-    pmdaInit(&dispatch, indoms, nindoms, metrics, nmetrics);
-    if ((dispatch.version.any.ext->e_flags & PMDA_EXT_CONNECTED) != PMDA_EXT_CONNECTED) {
-	/*
-	 * connect_pmcd() not called before, so need pmdaConnect()
-	 * here before falling into the PDU-driven pmdaMain() loop.
-	 */
+    // Copy the metrics.
+    for (i = 0; !error && i < nmetrics; i++) {
+	item = PyList_GetItem(metric_list, i);
+	/* Newer buffer interface */
+	if (item && PyObject_CheckBuffer(item)) {
+	    // Attempt to extract buffer information from it.
+	    if (PyObject_GetBuffer(item, &buffer, PyBUF_ANY_CONTIGUOUS)
+		== -1) {
+		PyErr_SetString(PyExc_TypeError,
+				"Unable to get metric item buffer");
+		error = 1;
+		break;
+	    }
+	    ptr = buffer.buf;
+	    len = buffer.len;
+	}
+	/* Older buffer interface */
+	else if (item && PyObject_CheckReadBuffer(item)) {
+	    // Attempt to extract information from the item.
+	    if (PyObject_AsReadBuffer(item, &ptr, &len) == -1) {
+		PyErr_SetString(PyExc_TypeError,
+				"Unable to get metric item buffer");
+		error = 1;
+		break;
+	    }
+	    buffer.buf = NULL;
+	}
+	else {
+	    PyErr_SetString(PyExc_TypeError, "Unable to retrieve metric");
+	    error = 1;
+	    break;
+	}
+
+	// The metric table is supposed to be composed of
+	// 'pmdaMetric(Structure)' items, which should be laid out
+	// like a 'pmdaMetric' structure in memory.
+	if (len != sizeof(pmdaMetric)) {
+	    PyErr_SetString(PyExc_TypeError, "Invalid metric item size");
+	    if (buffer.buf)
+		PyBuffer_Release(&buffer);
+	    error = 1;
+	    break;
+	}
+	metric_buffer[i] = *(pmdaMetric *)ptr;
+	if (buffer.buf)
+	    PyBuffer_Release(&buffer);
+    }
+    if (error) {
+	if (indom_buffer) {
+	    free(indom_buffer);
+	    indom_buffer = NULL;
+	}
+	nindoms = 0;
+	if (metric_buffer) {
+	    free(metric_buffer);
+	    metric_buffer = NULL;
+	}
+	nmetrics = 0;
+    }
+    return error;
+}
+
+static void
+pmda_refresh_metrics(void)
+{
+    // Update the metrics/indoms.
+    if (!update_indom_metric_buffers()) {
 	if (pmDebug & DBG_TRACE_LIBPMDA)
-	    fprintf(stderr, "pmda_dispatch connect to pmcd\n");
-	pmdaConnect(&dispatch);
+	    fprintf(stderr,
+		    "pmda_refresh_metrics: rehash %ld indoms, %ld metrics\n",
+		    (long)nindoms, (long)nmetrics);
+	dispatch.version.any.ext->e_indoms = indom_buffer;
+	dispatch.version.any.ext->e_nindoms = nindoms;
+	pmdaRehash(dispatch.version.any.ext, metric_buffer, nmetrics);
+    }
+}
+
+static PyObject *
+pmda_dispatch(PyObject *self, PyObject *args, PyObject *keywords)
+{
+    char *keyword_list[] = {"indoms", "metrics", NULL};
+
+    if (indom_list) {
+	Py_DECREF(indom_list);
+	indom_list = NULL;
+    }
+    if (metric_list) {
+	Py_DECREF(metric_list);
+	metric_list = NULL;
     }
 
-    PyBuffer_Release(&iv);
-    PyBuffer_Release(&mv);
+    if (!PyArg_ParseTupleAndKeywords(args, keywords, "OO", keyword_list,
+				     &indom_list, &metric_list))
+        return NULL;
 
-    if (pmDebug & DBG_TRACE_LIBPMDA)
-        fprintf(stderr, "pmda_dispatch entering PDU loop\n");
+    if (indom_list && metric_list) {
+	// PyArg_ParseTupleAndKeywords() returns "borrowed"
+	// references. Since we're going to keep these objects around
+	// for use later, increase their reference counts.
+	Py_INCREF(indom_list);
+	Py_INCREF(metric_list);
 
-    pmdaMain(&dispatch);
+        if (!PyList_Check(indom_list) || !PyList_Check(metric_list)) {
+            __pmNotifyErr(LOG_ERR,
+			  "pmda_dispatch failed to get metrics/indoms (non-list types)");
+	    PyErr_SetString(PyExc_TypeError,
+			    "pmda_dispatch failed to get metrics/indoms (non-list types)");
+            Py_DECREF(indom_list);
+            indom_list = NULL;
+            Py_DECREF(metric_list);
+            metric_list = NULL;
+	    return NULL;
+	}
+    }
+    else {
+	__pmNotifyErr(LOG_ERR,
+		      "pmda_dispatch failed to get metric/indom lists");
+	PyErr_SetString(PyExc_TypeError,
+			"pmda_dispatch failed to get metric/indom lists");
+	return NULL;
+    }
+
+    // Update the indoms/metrics.
+    if (! update_indom_metric_buffers()) {
+	if (pmDebug & DBG_TRACE_LIBPMDA)
+	    fprintf(stderr, "pmda_dispatch pmdaInit for metrics/indoms\n");
+	pmdaInit(&dispatch, indom_buffer, nindoms, metric_buffer, nmetrics);
+	if ((dispatch.version.any.ext->e_flags & PMDA_EXT_CONNECTED)
+	    != PMDA_EXT_CONNECTED) {
+	    /*
+	     * connect_pmcd() not called before, so need pmdaConnect()
+	     * here before falling into the PDU-driven pmdaMain() loop.
+	     */
+	    if (pmDebug & DBG_TRACE_LIBPMDA)
+		fprintf(stderr, "pmda_dispatch connect to pmcd\n");
+	    pmdaConnect(&dispatch);
+	}
+
+	if (pmDebug & DBG_TRACE_LIBPMDA)
+	    fprintf(stderr, "pmda_dispatch entering PDU loop\n");
+	pmdaMain(&dispatch);
+    }
     Py_INCREF(Py_None);
     return Py_None;
 }
 
-#else	/* old-school python */
-static PyObject *
-pmda_dispatch(PyObject *self, PyObject *args)
-{
-    int nindoms, nmetrics, size;
-    PyObject *ibuf, *mbuf, *iv, *mv;
-    pmdaMetric *metrics = NULL;
-    pmdaIndom *indoms = NULL;
-
-    if (!PyArg_ParseTuple(args, "OiOi", &ibuf, &nindoms, &mbuf, &nmetrics))
-        return NULL;
-
-    size = nindoms * sizeof(pmdaIndom);
-    if ((iv = PyBuffer_FromObject(ibuf, 0, size)) == NULL) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch indom extraction");
-        return NULL;
-    }
-    if (!PyBuffer_Check(iv)) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch wants buffer 1st arg");
-        return NULL;
-    }
-    size = nmetrics * sizeof(pmdaMetric);
-    if ((mv = PyBuffer_FromObject(mbuf, 0, size)) == NULL) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch metric extraction");
-        return NULL;
-    }
-    if (!PyBuffer_Check(mv)) {
-        PyErr_SetString(PyExc_TypeError, "pmda_dispatch wants buffer 3rd arg");
-        return NULL;
-    }
-
-    if (pmDebug & DBG_TRACE_LIBPMDA)
-        fprintf(stderr, "pmda_dispatch pmdaInit for metrics/indoms\n");
-
-    PyBuffer_Type.tp_as_buffer->bf_getreadbuffer(iv, 0, (void *)&indoms);
-    PyBuffer_Type.tp_as_buffer->bf_getreadbuffer(mv, 0, (void *)&metrics);
-    if (pmDebug & DBG_TRACE_LIBPMDA)
-	fprintf(stderr, "pmda_dispatch pmdaInit for metrics/indoms\n");
-    pmdaInit(&dispatch, nindoms > 0 ? indoms : NULL, nindoms, nmetrics > 0 ? metrics : NULL, nmetrics);
-    if ((dispatch.version.any.ext->e_flags & PMDA_EXT_CONNECTED) != PMDA_EXT_CONNECTED) {
-	/*
-	 * connect_pmcd() not called before, so need pmdaConnect()
-	 * here before falling into the PDU-driven pmdaMain() loop.
-	 */
-	if (pmDebug & DBG_TRACE_LIBPMDA)
-	    fprintf(stderr, "pmda_dispatch connect to pmcd\n");
-	pmdaConnect(&dispatch);
-    }
-
-    Py_DECREF(ibuf);
-    Py_DECREF(mbuf);
-
-    if (pmDebug & DBG_TRACE_LIBPMDA)
-        fprintf(stderr, "pmda_dispatch entering PDU loop\n");
-
-    pmdaMain(&dispatch);
-    Py_INCREF(Py_None);
-    return Py_None;
-}
-#endif
-
 static PyObject *
 pmda_log(PyObject *self, PyObject *args, PyObject *keywords)
 {
@@ -945,7 +1125,7 @@ pmda_uptime(PyObject *self, PyObject *args, PyObject *keywords)
     if (!PyArg_ParseTupleAndKeywords(args, keywords,
                         "i:pmda_uptime", keyword_list, &now))
         return NULL;
-    
+
     days = now / (60 * 60 * 24);
     now %= (60 * 60 * 24);
     hours = now / (60 * 60);
@@ -1020,6 +1200,18 @@ set_fetch_callback(PyObject *self, PyObject *args)
     return set_callback(self, args, "O:set_fetch_callback", &fetch_cb_func);
 }
 
+static PyObject *
+set_refresh_all(PyObject *self, PyObject *args)
+{
+    return set_callback(self, args, "O:set_refresh_all", &refresh_all_func);
+}
+
+static PyObject *
+set_refresh_metrics(PyObject *self, PyObject *args)
+{
+    return set_callback(self, args, "O:set_refresh_metrics",
+			&refresh_metrics_func);
+}
 
 static PyMethodDef methods[] = {
     { .ml_name = "pmda_pmid", .ml_meth = (PyCFunction)pmda_pmid,
@@ -1033,7 +1225,7 @@ static PyMethodDef methods[] = {
     { .ml_name = "init_dispatch", .ml_meth = (PyCFunction)init_dispatch,
         .ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "pmda_dispatch", .ml_meth = (PyCFunction)pmda_dispatch,
-        .ml_flags = METH_VARARGS },
+        .ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "connect_pmcd", .ml_meth = (PyCFunction)connect_pmcd,
         .ml_flags = METH_NOARGS },
     { .ml_name = "pmns_refresh", .ml_meth = (PyCFunction)namespace_refresh,
@@ -1062,6 +1254,11 @@ static PyMethodDef methods[] = {
         .ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "set_fetch_callback", .ml_meth = (PyCFunction)set_fetch_callback,
         .ml_flags = METH_VARARGS|METH_KEYWORDS },
+    { .ml_name = "set_refresh_metrics",
+      .ml_meth = (PyCFunction)set_refresh_metrics,
+      .ml_flags = METH_VARARGS|METH_KEYWORDS },
+    { .ml_name = "set_refresh_all", .ml_meth = (PyCFunction)set_refresh_all,
+        .ml_flags = METH_VARARGS | METH_KEYWORDS },
     { .ml_name = "pmda_log", .ml_meth = (PyCFunction)pmda_log,
         .ml_flags = METH_VARARGS|METH_KEYWORDS },
     { .ml_name = "pmda_err", .ml_meth = (PyCFunction)pmda_err,
@@ -1082,7 +1279,7 @@ pmda_dict_add(PyObject *dict, char *sym, long val)
     Py_XDECREF(pyVal);
 }
 
-/* called when the module is initialized. */ 
+/* called when the module is initialized. */
 MOD_INIT(cpmda)
 {
     PyObject *module, *dict;
